/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: client.c,v 1.6 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>
#ifdef LATENCYTEST
#include <sys/time.h> /* for gettimeofday() */
#endif
#ifdef _WIN32
#include <winsock2.h>
#define SHUT_WR SD_SEND
typedef int socklen_t;
#define ssize_t SSIZE_T
#else
#ifdef _sx
#define SHUT_WR 1
#endif
#include <unistd.h>
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#define SOCKET int
#define closesocket close
#endif /* _WIN32 */

#include "client.h"

#include <sys/types.h> /* for int32_t and int64_t */
#ifdef _sx
typedef long int64_t;
#else
#ifdef _WIN32
typedef int int32_t;
typedef long long int64_t;
#endif /* _WIN32 */
#endif /* _sx */

#define should_not_reach_here  fprintf(stderr, \
  "*** should not reach here (%s, line %d)\n", __FILE__, __LINE__)

static struct sockaddr_in sa;
static SOCKET sock;

int silc_debug = 1;

ssize_t
send_all(SOCKET sock, void *buf, size_t len, int flags)
{
  unsigned char *p;
  size_t m;
  ssize_t n;

  p = buf;
  m = len;
  while (m > 0) {
#if 0
    n = send(sock, p, m, flags);
#else
    n = send(sock, p, ((m < 1024 * 1024) ? m : 1024 * 1024), flags);
#endif
    if (n < 0) {
#if defined(DEBUG) && defined(_WIN32)
      printf("WSAGetLastError() = %d\n", WSAGetLastError());
#endif
      return -1;
    }
    p += n;
    m -= n;
  }
  return len;
}

ssize_t
recv_all(SOCKET sock, void *buf, size_t len, int flags)
{
  unsigned char *p;
  size_t m;
  ssize_t n;

  p = buf;
  m = len;
  while (m > 0) {
    n = recv(sock, p, m, flags);
    if (n < 0) {
#if defined(DEBUG) && defined(_WIN32)
      printf("WSAGetLastError() = %d\n", WSAGetLastError());
#endif
      return -1;
    }
    p += n;
    m -= n;
  }
  return len;
}

static void
send_ack(SOCKET sock)
{
  char buf[32] = "ACK";
  send_all(sock, buf, 32, 0);
}

static void
recv_ack(SOCKET sock)
{
  char buf[32];
  recv_all(sock, buf, 32, 0);
}

#if 1
#define bulk_recv_all recv_all
#else
int
bulk_recv_all(SOCKET sock, void *buf, size_t len, int flags)
{
  unsigned char *p;
  int m, n;

  p = buf;
  while (len > 0) {
    m = (len > 4096) ? 4096 : len;
    n = recv_all(sock, p, m, flags);
    if (n < 0)
      return -1;
#ifdef DEBUG
    if (n < m)
      printf("bulk_recv_all: received %ld out of %ld bytes\n", n, m);
#endif
    len -= m;
    p += m;
  }
  return 0;
}
#endif

#if 0 /* experimental */
#include <zlib.h>
#define CHUNK 65536
ssize_t
bulk_send_all(SOCKET sock, void *buf, size_t len, int flags)
{
  z_stream strm;
  unsigned char out[CHUNK];
  int status, flush;
  size_t size;

#ifdef DEBUG
  printf("bulk_send_all: len=%ld\n", len);
#endif
  strm.zalloc = Z_NULL;
  strm.zfree = Z_NULL;
  strm.opaque = Z_NULL;
  if (deflateInit(&strm, Z_DEFAULT_COMPRESSION) != Z_OK)
    return -1;
  strm.next_in = buf;
  strm.avail_in = len;
  strm.next_out = out;
  strm.avail_out = CHUNK;
  flush = Z_NO_FLUSH;
  for (;;) {
    if (strm.avail_in == 0) {
      flush = Z_FINISH;
    }
    status = deflate(&strm, flush);
    if (status == Z_STREAM_END)
      break;
    if (strm.avail_out == 0) {
      size = CHUNK;
      if (send_all(sock, &size, sizeof(size_t), flags) < 0)
	goto deflateError;
      recv_ack(sock);
      if (send_all(sock, out, size, flags) < 0)
	goto deflateError;
      recv_ack(sock);
      strm.next_out = out;
      strm.avail_out = CHUNK;
    }
  }
  if ((size = CHUNK - strm.avail_out) > 0) {
    if (send_all(sock, &size, sizeof(size_t), flags) < 0)
      goto deflateError;
    recv_ack(sock);
    if (send_all(sock, out, size, flags) < 0)
      goto deflateError;
    recv_ack(sock);
  }
  deflateEnd(&strm);
  send_ack(sock);
  return len;

 deflateError:
  deflateEnd(&strm);
  return -1;
}

#else

#if 1
#define bulk_send_all send_all
#else
int
bulk_send_all(SOCKET sock, void *buf, size_t len, int flags)
{
  unsigned char *p;
  int m, n;

  p = buf;
  while (len > 0) {
    m = (len > 4096) ? 4096 : len;
    n = send_all(sock, p, m, flags);
    if (n < 0)
      return -1;
#ifdef DEBUG
    if (n < m)
      printf("bulk_send_all: sent %ld out of %ld bytes\n", n, m);
#endif
    len -= m;
    p += m;
  }
  return 0;
}
#endif

#endif

char **format;
int num_formats;

int
recv_format_names(SOCKET sock)
{
  size_t size;
  char *buf, *p;
  int i, j;

  num_formats = 0;
  recv_all(sock, &size, (int)sizeof(size_t), 0);
  if (size == 0)
    return 0;
  buf = (char *)malloc(size);
  recv_all(sock, buf, size, 0);
  for (p = buf; p < buf + size; p++) {
    if (*p == '\n')
      num_formats++;
  }
#ifdef DEBUG
  if (silc_debug)
    printf("number of formats = %d\n", num_formats);
#endif
  format = (char **)malloc(sizeof(char *) * num_formats);
  p = buf;    
  for (i = 0; i < num_formats; i++) {
    for (j = 0; p[j] != '\n'; j++);
    format[i] = (char *)malloc(j + 1);
    strncpy(format[i], p, j);
    format[i][j] = '\0';
    p += j + 1;
#ifdef DEBUG
    if (silc_debug)
      printf("  %d: \"%s\"\n", i, format[i]);
#endif
  }
  free(buf);
  return num_formats;

#if 0
 onError:
  free(buf);
  while (i > 0)
    free(format[--i]);
  free(format);
  num_formats = 0;
  return -1;
#endif
}

const char *
silc_get_format_name(int id)
{
  if (id < num_formats)
    return format[id];
  return NULL;
}

int
silc_get_format_id(const char *name)
{
  int i;

  for (i = 0; i < num_formats; i++)
    if (strcmp(format[i], name) == 0)
      return i;
  return -1;
}

int
silc_get_number_of_formats()
{
  return num_formats;
}

static int
SILC_INIT_1(int origin)
{
  FILE *fp;
  char buf[256];
  char host[256];
  char *p;
  struct hostent *hp;
  int port;
  unsigned short n = 0x1234;
#ifdef _WIN32
  WSADATA wsaData;
#endif

#ifdef DEBUG
  p = getenv("SILC_DEBUG");
  if (p)
    silc_debug = atoi(p);
#endif
#ifdef _WIN32
  WSAStartup(MAKEWORD(2, 0), &wsaData);
  sprintf(buf, "%s/.silc", getenv("USERPROFILE"));
#else
  sprintf(buf, "%s/.silc", getenv("HOME"));
#endif
  fp = fopen(buf, "r");
  fgets(buf, sizeof(buf), fp);
  sscanf(buf, "%s %d", host, &port);
  fclose(fp);
  sock = socket(AF_INET, SOCK_STREAM, 0);
  sa.sin_family = AF_INET;
  sa.sin_port = htons(port);
#if 0
  gethostname(buf, sizeof(buf));
  if (strcmp(host, buf) == 0)
    strcpy(host, "localhost");
#endif
  hp = gethostbyname(host);
  sa.sin_addr.s_addr = ((struct in_addr *)(hp->h_addr))->s_addr;
  if (connect(sock, (struct sockaddr *)&sa, sizeof(sa)) < 0) {
    perror("connect");
    return -1;
  }
#ifdef DEBUG
  if (silc_debug)
    printf("connected to %s on port %d\n", host, port);
#endif

  /* inform protocol version */
  strcpy(buf, "V1\n");
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform protocol version: %s", buf);
    return -1;
  }

  /* inform endianness */
  if (*(unsigned char *)&n == 0x12)
    strcpy(buf, "B\n"); /* big endian */
  else
    strcpy(buf, "L\n"); /* little endian */
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform endianness: %s", buf);
    return -1;
  }

  /* inform size of size_t */
  sprintf(buf, "S%d\n", (int)sizeof(size_t));
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform size of size_t: %s", buf);
    return -1;
  }

  /* receive list of format names */
  strcpy(buf, "F\n");
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3) || recv_format_names(sock) < 0) {
    fprintf(stderr, "Error: failed to receive list of format names");
    return -1;
  }

  /* inform origin */
  sprintf(buf, "%d\n", origin);
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform origin: %s", buf);
    return -1;
  }

  /* inform array format (column major) */
  sprintf(buf, "C\n");
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform array format: %s", buf);
    return -1;
  }

  /* inform number of processes */
  strcpy(buf, "T1\n");
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to inform number of processes: %s", buf);
    return -1;
  }

#ifdef LATENCYTEST
  strcpy(buf, "Y\n");
  send_all(sock, buf, 32, 0);
  recv_all(sock, buf, 32, 0);
  if (strncmp(buf, "200", 3) == 0) {
    int i;
    char buf;
    struct timeval s, t;
    double average, variance, sample[100];

    average = 0;
    for (i = 0; i < 100; i++) {
      gettimeofday(&s, NULL);
      send(sock, &buf, 1, 0);
      recv(sock, &buf, 1, 0);
      gettimeofday(&t, NULL);
      sample[i] = (t.tv_sec - s.tv_sec) + (t.tv_usec - s.tv_usec) / 1e+6;
      average += sample[i];
    }
    average /= 100;
    variance = 0;
    for (i = 0; i < 100; i++)
      variance += (sample[i] - average) * (sample[i] - average);
    variance /= 100;
    fprintf(stderr, "av. round-trip latency %.2e sec, variance %.2e, "
	    "std. error %.2e\n", average, variance, sqrt(variance) / 10);
  } else {
    fprintf(stderr, "Error: failed to measure latency\n");
    return -1;
  }
#endif /* LATENCYTEST */

  return 0;
}

int
SILC_INIT(void)
{
  return SILC_INIT_1(0);
}

int
SILC_FINALIZE(void)
{
  int i;
  char buf[256];

  if (num_formats > 0) {
    for (i = 0; i < num_formats; i++)
      free(format[i]);
    free(format);
  }
  shutdown(sock, SHUT_WR);
  while (1) {
    i = recv(sock, buf, sizeof(buf), 0);
    if (i <= 0) {
#ifdef DEBUG
      if (i < 0)
        fprintf(stderr, "Error: recv(2) failed while closing socket (ignored)\n");
#endif
      break;
    }
  }
  i = closesocket(sock);
#ifdef _WIN32
  WSACleanup();
#endif
  return i;
}

int
SILC_EXEC(const char *cmd)
{
  char buf[32+4096];
  int n;

  n = strlen(cmd);
  if (n == 0) {
    return 0;
  } else if (n > 4096) {
    fprintf(stderr, "Error: too long command");
    return -1;
  }
  sprintf(buf, ":%d:%s\n", n, cmd);
  n = strlen(buf);
  send_all(sock, buf, (n > 32) ? n : 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Request: %s", buf);
#endif
  recv_all(sock, buf, 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Response: %s", buf);
#endif
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to execute commands: %s", buf);
    return -1;
  }
  return 0;
}

int
SILC_PUT(const char *name, silc_envelope_t *object)
{
  char buf[256];
  size_t size;

  if (object->type == SILC_MATRIX_TYPE) {
    if (strcmp(object->format, SILC_FORMAT_DENSE) &&
	strcmp(object->format, SILC_FORMAT_CRS) &&
	strcmp(object->format, SILC_FORMAT_BAND)) {
      fprintf(stderr, "Error: unsupported format: \"%s\"\n", object->format);
      return -1;
    }
    object->format_id = silc_get_format_id(object->format);
    if (object->format_id < 0) {
      fprintf(stderr, "Error: unknown format: \"%s\"\n", object->format);
      return -1;
    }
  }
  sprintf(buf, ">%s\n", name);
  send_all(sock, buf, 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Request: %s", buf);
#endif
  recv_ack(sock);
  send_all(sock, object, sizeof(int) * 8, 0);
  recv_ack(sock);
  switch (object->type) {
  case SILC_SCALAR_TYPE:
    size = 1;
    break;
  case SILC_ROW_VECTOR_TYPE:
  case SILC_COLUMN_VECTOR_TYPE:
    send_all(sock, &object->length, sizeof(size_t), 0);
    recv_ack(sock);
    size = object->length;
    break;
  case SILC_MATRIX_TYPE:
    send_all(sock, &object->m, sizeof(size_t), 0);
    recv_ack(sock);
    send_all(sock, &object->n, sizeof(size_t), 0);
    recv_ack(sock);
    if (strcmp(object->format, SILC_FORMAT_DENSE) == 0) {
      size = object->m * object->n;
    } else if (strcmp(object->format, SILC_FORMAT_CRS) == 0) {
      send_all(sock, &object->nnz, sizeof(size_t), 0);
      recv_ack(sock);
      bulk_send_all(sock, object->row, sizeof(int) * (object->m + 1), 0);
      recv_ack(sock);
      bulk_send_all(sock, object->index, sizeof(int) * object->nnz, 0);
      recv_ack(sock);
      size = object->nnz;
    } else if (strcmp(object->format, SILC_FORMAT_BAND) == 0) {
      send_all(sock, &object->l, sizeof(size_t), 0);
      recv_ack(sock);
      send_all(sock, &object->u, sizeof(size_t), 0);
      recv_ack(sock);
      size = (object->l + object->u + 1) * object->n;
    } else {
      should_not_reach_here;
      return -1;
    }
    break;
  case SILC_CUBIC_ARRAY_TYPE:
    send_all(sock, &object->l, sizeof(size_t), 0);
    recv_ack(sock);
    send_all(sock, &object->m, sizeof(size_t), 0);
    recv_ack(sock);
    send_all(sock, &object->n, sizeof(size_t), 0);
    recv_ack(sock);
    size = object->l * object->m * object->n;
    break;
  default:
    should_not_reach_here;
  }
  switch (object->precision) {
  case SILC_INT:
    size = size * sizeof(int32_t); break;
  case SILC_LONG:
    size = size * sizeof(int64_t); break;
  case SILC_FLOAT:
    size = size * sizeof(float); break;
  case SILC_DOUBLE:
    size = size * sizeof(double); break;
  case SILC_COMPLEX:
    size = size * sizeof(float) * 2; break;
  case SILC_DOUBLE_COMPLEX:
    size = size * sizeof(double) * 2; break;
  default:
    should_not_reach_here;
  }
  bulk_send_all(sock, object->v, size, 0);
  recv_all(sock, buf, 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Response: %s", buf);
#endif
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to put data: %s", buf);
    return -1;
  }
  return 0;
}

int
SILC_GET(silc_envelope_t *object, const char *name)
{
  char buf[256];
  size_t size;

  sprintf(buf, "<%s\n", name);
  send_all(sock, buf, 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Request: %s", buf);
#endif
  recv_all(sock, buf, 32, 0);
#ifdef DEBUG
  if (silc_debug)
    printf("Response: %s", buf);
#endif
  if (strncmp(buf, "200", 3)) {
    fprintf(stderr, "Error: failed to get data: %s", buf);
    return -1;
  }
  send_ack(sock);
  recv_all(sock, object, sizeof(int) * 8, 0);
  send_ack(sock);
  switch (object->type) {
  case SILC_SCALAR_TYPE:
    size = 1;
    break;
  case SILC_ROW_VECTOR_TYPE:
  case SILC_COLUMN_VECTOR_TYPE:
    recv_all(sock, &object->length, sizeof(size_t), 0);
    send_ack(sock);
    size = object->length;
    break;
  case SILC_MATRIX_TYPE:
    object->format = silc_get_format_name(object->format_id);
    if (!object->format) {
      fprintf(stderr, "Error: unknown format: %d\n", object->format_id);
      return -1;
    }
    recv_all(sock, &object->m, sizeof(size_t), 0);
    send_ack(sock);
    recv_all(sock, &object->n, sizeof(size_t), 0);
    send_ack(sock);
    if (strcmp(object->format, SILC_FORMAT_DENSE) == 0) {
      size = object->m * object->n;
    } else if (strcmp(object->format, SILC_FORMAT_CRS) == 0) {
      recv_all(sock, &object->nnz, sizeof(size_t), 0);
      send_ack(sock);
      if (!object->v) {
	object->row = (void *)malloc(sizeof(int) * (object->m + 1));
	if (!object->row) {
	  fprintf(stderr, "Error: out of memory: row\n");
	  return -1;
	}
	object->index = (void *)malloc(sizeof(int) * object->nnz);
	if (!object->index) {
	  fprintf(stderr, "Error: out of memory: index\n");
	  return -1;
	}
      }
      bulk_recv_all(sock, object->row, sizeof(int) * (object->m + 1), 0);
      send_ack(sock);
      bulk_recv_all(sock, object->index, sizeof(int) * object->nnz, 0);
      send_ack(sock);
      size = object->nnz;
    } else if (strcmp(object->format, SILC_FORMAT_BAND) == 0) {
      recv_all(sock, &object->l, sizeof(size_t), 0);
      send_ack(sock);
      recv_all(sock, &object->u, sizeof(size_t), 0);
      send_ack(sock);
      size = (object->l + object->u + 1) * object->n;
    } else {
      fprintf(stderr, "Error: unknown format: \"%s\"\n", object->format);
      return -1;
    }
    break;
  case SILC_CUBIC_ARRAY_TYPE:
    recv_all(sock, &object->l, sizeof(size_t), 0);
    send_ack(sock);
    recv_all(sock, &object->m, sizeof(size_t), 0);
    send_ack(sock);
    recv_all(sock, &object->n, sizeof(size_t), 0);
    send_ack(sock);
    size = object->l * object->m * object->n;
    break;
  default:
    should_not_reach_here;
  }
  switch (object->precision) {
  case SILC_INT:
    size = size * sizeof(int32_t); break;
  case SILC_LONG:
    size = size * sizeof(int64_t); break;
  case SILC_FLOAT:
    size = size * sizeof(float); break;
  case SILC_DOUBLE:
    size = size * sizeof(double); break;
  case SILC_COMPLEX:
    size = size * sizeof(float) * 2; break;
  case SILC_DOUBLE_COMPLEX:
    size = size * sizeof(double) * 2; break;
  default:
    should_not_reach_here;
  }
  if (!object->v) {
    object->v = (void *)malloc(sizeof(int) * size);
    if (!object->v) {
      fprintf(stderr, "Error: out of memory: v\n");
      return -1;
    }
  }
  bulk_recv_all(sock, object->v, size, 0);
#ifdef COMMTEST
  send_ack(sock);
#endif
  return 0;
}

/* Fortran interface */

void
silc_init_(int *status)
{
  *status = SILC_INIT_1(1);
}

void
silc_finalize_(int *status)
{
  *status = SILC_FINALIZE();
}

void
silc_exec_(const char *cmd, int *status, int _len)
{
  char *s;

  s = malloc(_len+1);
  strncpy(s, cmd, _len);
  s[_len] = '\0';
  *status = SILC_EXEC(s);
  free(s);
}

void
silc_put_scalar_(const char *name, void *value, int *precision, int *status,
		 int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_SCALAR_TYPE;
  object.precision = *precision;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_put_row_vector_(const char *name, void *value, int *length,
		     int *precision, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_ROW_VECTOR_TYPE;
  object.precision = *precision;
  object.length = *length;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_put_column_vector_(const char *name, void *value, int *length,
			int *precision, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_COLUMN_VECTOR_TYPE;
  object.precision = *precision;
  object.length = *length;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_put_matrix_(const char *name, void *value, int *m, int *n,
		 int *precision, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_DENSE;
  object.precision = *precision;
  object.m = *m;
  object.n = *n;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_put_matrix_crs_(const char *name, void *value, int *m, int *n, int *nnz,
		     int *row, int *index, int *precision, int *status,
		     int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_CRS;
  object.precision = *precision;
  object.m = *m;
  object.n = *n;
  object.nnz = *nnz;
  object.row = row;
  object.index = index;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_put_cubic_array_(const char *name, void *value, int *l, int *m, int *n,
		      int *precision, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.type = SILC_CUBIC_ARRAY_TYPE;
  object.precision = *precision;
  object.m = *l;
  object.m = *m;
  object.n = *n;
  *status = SILC_PUT(s, &object);
  free(s);
}

void
silc_get_scalar_(const char *name, void *value, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  *status = SILC_GET(&object, s);
  free(s);
}

void
silc_get_row_vector_(const char *name, void *value, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  *status = SILC_GET(&object, s);
  free(s);
}

void
silc_get_column_vector_(const char *name, void *value, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  *status = SILC_GET(&object, s);
  free(s);
}

void
silc_get_matrix_(const char *name, void *value, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  *status = SILC_GET(&object, s);
  free(s);
}

void
silc_get_matrix_crs_(const char *name, void *value, int *row, int *index,
		     int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  object.row = row;
  object.index = index;
  *status = SILC_GET(&object, s);
  free(s);
}

void
silc_get_cubic_array_(const char *name, void *value, int *status, int _len)
{
  silc_envelope_t object;
  char *s;

  s = malloc(_len+1);
  strncpy(s, name, _len);
  s[_len] = '\0';
  object.v = value;
  *status = SILC_GET(&object, s);
  free(s);
}
