/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: client.h,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#ifndef SILC_CLIENT_H
#define SILC_CLIENT_H 1

#define SILC_SCALAR_TYPE         1
#define SILC_RANGE_TYPE          2
#define SILC_COLUMN_VECTOR_TYPE  4
#define SILC_ROW_VECTOR_TYPE     8
#define SILC_MATRIX_TYPE         16
#define SILC_CUBIC_ARRAY_TYPE    32

#define SILC_FLOAT           1
#define SILC_DOUBLE          2
#define SILC_COMPLEX         4
#define SILC_DOUBLE_COMPLEX  8
#define SILC_INT             16
#define SILC_LONG            32

typedef struct {
  int type;
  int precision;
  int format_id;
  int reserved[5];
  void *v;
  const char *format;
  /* SILC_ROW_VECTOR, SILC_COLUMN_VECTOR */
  size_t length;
  /* SILC_MATRIX_TYPE, SILC_CUBIC_ARRAY_TYPE */
  size_t m;
  size_t n;
  size_t l;
  size_t u;
  size_t nnz;
  int *row;
  int *index;
} silc_envelope_t;

#ifdef __cplusplus
extern "C" {
#endif

int SILC_INIT();
int SILC_FINALIZE();
int SILC_EXEC(const char *);
int SILC_PUT(const char *, silc_envelope_t *);
int SILC_GET(silc_envelope_t *, const char *);

#ifdef __cplusplus
}
#endif

/* supported formats */
#define SILC_FORMAT_DENSE  "SILC:dense (column major)"
#define SILC_FORMAT_CRS    "SILC:CRS"
#define SILC_FORMAT_BAND   "SILC:Band"

#endif /* SILC_CLIENT_H */
