/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: console.h,v 1.3 2007/10/31 06:22:07 kajiyama Exp $
 */

#ifndef CONSOLE_H
#define CONSOLE_H 1

/* expression */

typedef enum {
  ExprType = 0,
  BoolExprType,
  UnaryCondExprType,
  BinaryCondExprType,
  ServerSideExprType,
} expr_type_t;

typedef struct {
  expr_type_t type;
} expr_t;

typedef enum {
  NotOp,
  AndOp,
  OrOp,
  EqualOp,
  NotEqualOp,
  LessOp,
  LessEqualOp,
  GreaterOp,
  GreaterEqualOp,
} cond_op_t;

typedef struct {
  expr_type_t type;
  int bool;
} bool_expr_t; /* inherits expr_t */

#define BOOL(p) ((bool_expr_t *)(p))

typedef struct {
  expr_type_t type;
  cond_op_t op;
  expr_t *operand;
} unary_cond_expr_t; /* inherits expr_t */

#define UNARY_COND(p) ((unary_cond_expr_t *)(p))

typedef struct {
  expr_type_t type;
  cond_op_t op;
  expr_t *lhs, *rhs;
} binary_cond_expr_t; /* inherits expr_t */

#define BINARY_COND(p) ((binary_cond_expr_t *)(p))

typedef struct {
  expr_type_t type;
  char expr[1024];
} ss_expr_t; /* inherits expr_t */

#define SS_EXPR(p) ((ss_expr_t *)(p))

/* statement */

typedef enum {
  StmtType = 0,
  BlockStmtType,
  IfStmtType,
  WhileStmtType,
  ContinueStmtType,
  BreakStmtType,
  LoadStmtType,
  SaveStmtType,
  MessageStmtType,
  PPrintStmtType,
  ServerSideStmtType,
} stmt_type_t;

typedef struct {
  stmt_type_t type;
} stmt_t;

#define STMT(p) ((stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  int n_stmts;
  stmt_t **stmt;
} block_stmt_t; /* inherits stmt_t */

#define BLOCK(p) ((block_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  expr_t *cond_expr;
  stmt_t *then_block;
  stmt_t *else_block;
} if_stmt_t; /* inherits stmt_t */

#define IF(p) ((if_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  expr_t *cond_expr;
  stmt_t *body;
} while_stmt_t; /* inherits stmt_t */

#define WHILE(p) ((while_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
} continue_stmt_t; /* inherits stmt_t */

#define CONTINUE(p) ((continue_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
} break_stmt_t; /* inherits stmt_t */

#define BREAK(p) ((break_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  char path[1024];
  int n_names;
  char **name;
} load_stmt_t; /* inherits stmt_t */

#define LOAD(p) ((load_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  char path[1024];
  int n_names;
  char **name;
} save_stmt_t; /* inherits stmt_t */

#define SAVE(p) ((save_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  char msg[1024];
} message_stmt_t; /* inherits stmt_t */

#define MESSAGE(p) ((message_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  char expr[1024];
} pprint_stmt_t; /* inherits stmt_t */

#define PPRINT(p) ((pprint_stmt_t *)(p))

typedef struct {
  stmt_type_t type;
  char stmt[1024];
} ss_stmt_t; /* inherits stmt_t */

#define SS(p) ((ss_stmt_t *)(p))

/* console_lexer.l */
void set_text(char *);
extern int parser_lineno;

/* console_parser.y */
extern const char *parser_error;
int is_partial(char *);
stmt_t *parse_script(char *);
void free_statement(stmt_t *);
#ifdef DEBUG
void dump_statement(stmt_t *, int);
#endif

#endif /* CONSOLE_H */
