%{
/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: console_lexer.l,v 1.3 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include <ctype.h> /* for tolower() */
#include "console.h"
#include "console_parser.h"

static char text[1024], *text_ptr;
int parser_lineno;

void yyerror(const char *);

void set_text(char *s)
{
  text_ptr = strcpy(text, s);
  yyerror(NULL);
}

#define YY_INPUT(buf,result,max_size) \
        result = ((buf[0] = *text_ptr++) == '\0') ? YY_NULL : 1;

%}

%%
#[^\n]+                 ; /* ignore comment */
~'                      return TOK_TRANSPOSE;
~                       return TOK_COMPLEX_CONJUGATE;
\\                      return TOK_LEQ;
\/@=                    return TOK_DIVIDE_ELEMENTWISE_EQUAL;
\/@                     return TOK_DIVIDE_ELEMENTWISE;
\/=                     return TOK_DIVIDE_EQUAL;
\/                      return TOK_DIVIDE;
\-=                     return TOK_MINUS_EQUAL;
\-                      return TOK_MINUS;
\+=                     return TOK_PLUS_EQUAL;
\+                      return TOK_PLUS;
\*@=                    return TOK_MULTIPLY_ELEMENTWISE_EQUAL;
\*@                     return TOK_MULTIPLY_ELEMENTWISE;
\*=                     return TOK_MULTIPLY_EQUAL;
\*                      return TOK_MULTIPLY;
\%=                     return TOK_REMAINDER_EQUAL;
\%                      return TOK_REMAINDER;
==                      return TOK_DOUBLE_EQUAL;
=                       return TOK_EQUAL;
'~                      return TOK_TRANSPOSE;
'                       return TOK_CONJUGATE_TRANSPOSE;
;;                      return TOK_DOUBLE_SEMICOLON;
\<=                     return TOK_LESS_EQUAL;
\<                      return TOK_LESS;
\>=                     return TOK_GREATER_EQUAL;
\>                      return TOK_GREATER;
\!=                     return TOK_NOT_EQUAL;
if                      return TOK_IF;
else                    return TOK_ELSE;
while                   return TOK_WHILE;
continue                return TOK_CONTINUE;
break                   return TOK_BREAK;
and                     return TOK_AND;
or                      return TOK_OR;
not                     return TOK_NOT;
true                    return TOK_TRUE;
false                   return TOK_FALSE;
load                    return TOK_LOAD;
save                    return TOK_SAVE;
message                 return TOK_MESSAGE;
pprint                  return TOK_PPRINT;
prefer                  return TOK_PREFER;
[0-9]+\.[0-9]*[Dd][+\-][0-9]+    |
[0-9]*\.[0-9]+[Dd][+\-][0-9]+    {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
                            return TOK_REAL8_LITERAL;
			  }
			  yyerror("too long real8 literal");
                        }
[0-9]+\.[0-9]*`S        |
[0-9]*\.[0-9]+`S        |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+`S  |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+`S  {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
                            return TOK_REAL4_LITERAL;
			  }
			  yyerror("too long real4 literal");
                        }
[0-9]+\.[0-9]*`D        |
[0-9]*\.[0-9]+`D        |
[0-9]+\.[0-9]*          |
[0-9]*\.[0-9]+          |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+`D  |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+`D  |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+    |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+    {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
                            return TOK_REAL8_LITERAL;
			  }
			  yyerror("too long real8 literal");
                        }
[0-9]+`D                {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
			    return TOK_INT8_LITERAL;
                          }
			  yyerror("too long int8 literal");
                        }
[0-9]+`S                |
[0-9]+                  {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
			    return TOK_INT4_LITERAL;
                          }
			  yyerror("too long int4 literal");
                        }
[_A-Za-z][_A-Za-z0-9]*  {
			  if (yyleng+1 < sizeof(yylval.string)) {
                            strcpy(yylval.string, yytext);
			    return TOK_IDENTIFIER;
			  }
			  yyerror("too long identifier");
                        }
\"(\\[\\\"nrt]|\\x[0-9A-Fa-f][0-9A-Fa-f]|[^"])*\" {
			  int i, j;
                          unsigned int c1, c2;
                          i = 1;
                          j = 0;
			  while (i < yyleng-1 && j < sizeof(yylval.string)) {
			    if (yytext[i] != '\\') {
			      yylval.string[j++] = yytext[i++];
			      continue;
			    }
			    switch (yytext[i+1]) {
			    case '"': yylval.string[j++] = '"'; i+=2; break;
			    case '\\': yylval.string[j++] = '\\'; i+=2; break;
			    case 'n': yylval.string[j++] = '\n'; i+=2; break;
			    case 'r': yylval.string[j++] = '\r'; i+=2; break;
			    case 't': yylval.string[j++] = '\t'; i+=2; break;
			    case 'x':
                              c1 = tolower(yytext[i+2]) - '0';
                              c2 = tolower(yytext[i+3]) - '0';
			      yylval.string[j++] = (c1 << 8) | c2;
                              i += 4;
			      break;
			    default:
                              goto string_literal_error;
			    }
			  }
                          if (j < sizeof(yylval.string)) {
			    yylval.string[j] = '\0';
			    return TOK_STRING_LITERAL;
                          }
                         string_literal_error:
			  yyerror("invalid string literal");
                        }
[\[\](){}:;,]           return yytext[0];
[ \t]+                  ; /* ignore whitespace */
\\\n                    {
                          parser_lineno++;
                        }
\n                      {
                          parser_lineno++;
                          return TOK_EOL;
                        }
.                       yyerror("unexpected input");
%%
