/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: demo2.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <unistd.h>
#include <malloc.h>

#include "client.h"

void
print_matrix(const char *name, silc_envelope_t object)
{
  int i, j;
  double *v;
 
  if (object.type != SILC_MATRIX_TYPE ||
      object.precision != SILC_DOUBLE) {
    fprintf(stderr, "%s: not a matrix of double\n", name);
    return;
  }
  v = object.v;
  printf("MATRIX %s\n", name);
  for (i = 0; i < object.m; i++) {
    printf("(");
    for (j = 0; j < object.n; j++)
      printf("\t%lf", v[i + j * object.m]);
    printf("\t)\n");
  }
}

void
print_vector(const char *name, silc_envelope_t object)
{
  int i;
  double *v;
 
  if (object.type != SILC_COLUMN_VECTOR_TYPE ||
      object.precision != SILC_DOUBLE) {
    fprintf(stderr, "%s: not a column vector of double\n", name);
    return;
  }
  v = object.v;
  printf("COLUMN VECTOR %s\n", name);
  printf("(");
  for (i = 0; i < object.length; i++)
    printf("\t%lf", *v++);
  printf("\t)\n");
}

#define N 4

int main(int argc, char **argv)
{
  int i, j;
  silc_envelope_t object;
  double A[N * N];
  double b[N], x[N];

  A[0] = 4; A[4] = 3; A[8]  = 2; A[12] = 1;
  A[1] = 3; A[5] = 4; A[9]  = 3; A[13] = 2;
  A[2] = 2; A[6] = 3; A[10] = 4; A[14] = 3;
  A[3] = 1; A[7] = 2; A[11] = 3; A[15] = 4;

  b[0] = 2;
  b[1] = 4;
  b[2] = 3;
  b[3] = 5;

  SILC_INIT();

  object.v = A;
  object.type = SILC_MATRIX_TYPE;
  object.precision = SILC_DOUBLE;
  object.format = SILC_FORMAT_DENSE;
  object.m = N;
  object.n = N;
  print_matrix("A", object);
  SILC_PUT("A", &object);

  object.v = b;
  object.type = SILC_COLUMN_VECTOR_TYPE;
  object.precision = SILC_DOUBLE;
  object.length = N;
  print_vector("b", object);
  SILC_PUT("b", &object);

#if 1
  SILC_EXEC("x = leq_gs(A, b)");
  SILC_EXEC("x = leq_cg(A, b)");
#endif

  SILC_EXEC("prefer leq_cg");
  SILC_EXEC("x = A \\ b");

  object.v = x;
  SILC_GET(&object, "x");
  print_vector("x", object);

  SILC_EXEC("prefer leq_gs");
  SILC_EXEC("x = A \\ b");

  object.v = x;
  SILC_GET(&object, "x");
  print_vector("x", object);

  SILC_FINALIZE();
}
