/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: demo3.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <math.h>

#include "client.h"

extern double wtime(void);

int main(int argc, char *argv[])
{
  silc_envelope_t object;
  double *value, *b, *x;
  int *index, *row;
  int i, j, n;
  double norm, d;
  double t, elapsed;
  long N, NNZ;

  if (argc == 2)
    N = atol(argv[1]);
  else
    N = 1000;
  NNZ = N * 3 - 2;

  value = (double *)malloc(sizeof(double) * NNZ);
  b = (double *)malloc(sizeof(double) * N);
  x = (double *)malloc(sizeof(double) * N);
  row = (int *)malloc(sizeof(int) * (N + 1));
  index = (int *)malloc(sizeof(int) * NNZ);

  /* create a tridiagonal matrix in CRS format */
  for (i = 0; i < N; i++) {
    n = row[i] = (i == 0) ? 0 : i * 3 - 1;
    if (i == 0) {
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = -1.0;
      x[i] =  0.0;
    } else if (i < N - 1) {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = 0.0;
      x[i] = 0.0;
    } else {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      b[i] = -1.0;
      x[i] =  0.0;
      row[N] = n; /* number of non-zero elements */
    }
  }

  t = wtime();

  /* initialize SILC */
  SILC_INIT();

  /* put matrix A */
  object.v = value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_CRS;
  object.precision = SILC_DOUBLE;
  object.m = object.n = N;
  object.nnz = NNZ;
  object.row = row;
  object.index = index;
  SILC_PUT("A", &object);

  /* put vector b */
  object.v = b;
  object.type = SILC_COLUMN_VECTOR_TYPE;
  object.precision = SILC_DOUBLE;
  object.length = N;
  SILC_PUT("b", &object);

  /* solve linear equations */
  SILC_EXEC("x = A \\ b");

  /* get vector x */
  object.v = x;
  SILC_GET(&object, "x");

  /* finalize SILC */
  SILC_FINALIZE();

  elapsed = wtime() - t;
  printf("%lfs\n", elapsed);

#ifdef DEBUG
  /* check solution */
  norm = 0.0;
  for (i = 0; i < N; i++) {
    d = b[i];
    for (j = row[i]; j < row[i+1]; j++)
      d -= value[j] * x[index[j]];
    norm += d * d;
  }
  printf("||b-Ax|| = %e\n", sqrt(norm));
#endif
}
