/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: demo3_ssi_cg.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <malloc.h>
#include <math.h>

#include "ssi_leqlib.h"

#define N 1000
#define NNZ (N * 3 - 2)

int main(int argc, char **argv)
{
  SSI_MATRIX A;
  SSI_SCALAR *value, *b, *x, work[N * 6], params[SSI_PARAMS_LEN];
  int *index, *row, options[SSI_OPTIONS_LEN], status;
  int i, j, n;
  double norm, d;

  value = (SSI_SCALAR *)malloc(sizeof(SSI_SCALAR) * NNZ);
  b = (SSI_SCALAR *)malloc(sizeof(SSI_SCALAR) * N);
  x = (SSI_SCALAR *)malloc(sizeof(SSI_SCALAR) * N);
  row = (int *)malloc(sizeof(int) * (N + 1));
  index = (int *)malloc(sizeof(int) * NNZ);

  /* create a tridiagonal matrix in CRS format */
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i,n)
#endif
  for (i = 0; i < N; i++) {
    n = row[i] = (i == 0) ? 0 : i * 3 - 1;
    if (i == 0) {
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = -1.0;
      x[i] =  0.0;
    } else if (i < N - 1) {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = 0.0;
      x[i] = 0.0;
    } else {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      b[i] = -1.0;
      x[i] =  0.0;
      row[N] = n; /* number of non-zero elements */
    }
  }

  /* solve linear equations */
  A.matrix_type = SSI_MATRIX_TYPE_CRS;
  A.array_base  = SSI_MATRIX_ARRAY_BASE_0;
  A.matrix_base = SSI_MATRIX_ARRAY_BASE_0;
  A.n = N;
  A.nnz = NNZ;
  A.value = value;
  A.index = index;
  A.row = row;
  params[SSI_PARAMS_RESID] = 1.0e-10;
  options[SSI_OPTIONS_MAXITER] = N;
  options[SSI_OPTIONS_OUTPUT] = 0;
  status = ssi_cg(b, x, work, params, options, &A, NULL);
  if (status == SSIRET_MAXITER) {
    printf("too many iterations\n");
    return -1;
  }

  /* check solution */
  norm = 0.0;
  for (i = 0; i < N; i++) {
    d = b[i];
    for (j = row[i]; j < row[i+1]; j++)
      d -= value[j] * x[index[j]];
    norm += d * d;
  }
  printf("||b-Ax|| = %e\n", sqrt(norm));
  printf("iter = %d\n", options[SSI_OPTIONS_MAXITER]);
}
