/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: demo4.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>

#include "client.h"

struct CRS_double {
  int m, n, nnz;
  double value[256];
  int row[16], index[256];
};

struct CRS_complex {
  int m, n, nnz;
  struct { float real, image; } value[256];
  int row[16], index[256];
};

int
put_CRS_double(char *name, struct CRS_double *matrix)
{
  silc_envelope_t object;

  object.v = matrix->value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_CRS;
  object.precision = SILC_DOUBLE;
  object.m = matrix->m;
  object.n = matrix->n;
  object.nnz = matrix->nnz;
  object.row = matrix->row;
  object.index = matrix->index;
  return SILC_PUT(name, &object);
}

int
get_CRS_double(char *name, struct CRS_double *matrix)
{
  silc_envelope_t object;
  int status;

  object.v = matrix->value;
  object.row = matrix->row;
  object.index = matrix->index;
  status = SILC_GET(&object, name);
  if (status < 0)
    return -1;
  matrix->m = object.m;
  matrix->n = object.n;
  matrix->nnz = object.nnz;
  return status;
}

void
print_CRS_double(char *name, struct CRS_double *matrix)
{
  int i, j;

  for (i = 0; i < matrix->m; i++)
    for (j = matrix->row[i]; j < matrix->row[i+1]; j++)
      printf("%s[%d,%d] = %4.1lf\n",
	     name, i+1, matrix->index[j]+1, matrix->value[j]);
}

int
put_CRS_complex(char *name, struct CRS_complex *matrix)
{
  silc_envelope_t object;

  object.v = matrix->value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_CRS;
  object.precision = SILC_COMPLEX;
  object.m = matrix->m;
  object.n = matrix->n;
  object.nnz = matrix->nnz;
  object.row = matrix->row;
  object.index = matrix->index;
  return SILC_PUT(name, &object);
}

int
get_CRS_complex(char *name, struct CRS_complex *matrix)
{
  silc_envelope_t object;
  int status;

  object.v = matrix->value;
  object.row = matrix->row;
  object.index = matrix->index;
  status = SILC_GET(&object, name);
  if (status < 0)
    return -1;
  matrix->m = object.m;
  matrix->n = object.n;
  matrix->nnz = object.nnz;
  return status;
}

void
print_CRS_complex(char *name, struct CRS_complex *matrix)
{
  int i, j;

  for (i = 0; i < matrix->m; i++)
    for (j = matrix->row[i]; j < matrix->row[i+1]; j++)
      printf("%s[%d,%d] = %4.1f%+4.1fi\n", name, i+1, matrix->index[j]+1,
	     matrix->value[j].real, matrix->value[j].image);
}

int main(int argc, char *argv[])
{
  struct CRS_double A = {
    4, 4, 8,
    {11, 13, 22, 32, 33, 34, 41, 44},
    {0, 2, 3, 6, 8},
    {0, 2, 1, 1, 2, 3, 0, 3}};
  struct CRS_complex B = {
    4, 2, 5,
    {{11, 2}, {22, 2}, {31, 2}, {32, 2}, {41, 2}},
    {0, 1, 2, 4, 5},
    {0, 1, 0, 1, 0}};

  SILC_INIT();

  put_CRS_double("A", &A);
  SILC_EXEC("AA = A'");
  get_CRS_double("AA", &A);
  print_CRS_double("A'", &A);
  SILC_EXEC("AA = A[1,1]");
  SILC_EXEC("AA = A[1,2]");
  SILC_EXEC("AA = A[2,1]");
  SILC_EXEC("AA = A[2,2]");
  SILC_EXEC("AA = A[:,1]");
  SILC_EXEC("AA = A[:,2]");
  SILC_EXEC("AA = A[:,3]");
  SILC_EXEC("AA = A[:,4]");
  SILC_EXEC("AA = A[1,:]");
  SILC_EXEC("AA = A[2,:]");
  SILC_EXEC("AA = A[3,:]");
  SILC_EXEC("AA = A[4,:]");
  SILC_EXEC("AA = A[{4,3,2,1},{4,3,2,1}]");

  put_CRS_complex("B", &B);
  SILC_EXEC("BB = B'");
  get_CRS_complex("BB", &B);
  print_CRS_complex("B'", &B);
  SILC_EXEC("BB = B[1,1]");
  SILC_EXEC("BB = B[1,2]");
  SILC_EXEC("BB = B[2,1]");
  SILC_EXEC("BB = B[2,2]");
  SILC_EXEC("BB = B[:,1]");
  SILC_EXEC("BB = B[:,2]");
  SILC_EXEC("BB = B[1,:]");
  SILC_EXEC("BB = B[2,:]");
  SILC_EXEC("BB = B[3,:]");
  SILC_EXEC("BB = B[4,:]");
  SILC_EXEC("BB = B[{4,3,2,1},{2,1}]");

  SILC_FINALIZE();
}
