# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: makelap2.py,v 1.4 2007/10/31 06:15:43 kajiyama Exp $

# makelap2.py generates a problem of two-dimensional Laplacian equations
# by five-point difference approximation with uniform orthogonal mesh.

import os, sys, time

program_name = os.path.basename(sys.argv[0])

if len(sys.argv) != 2:
    sys.stderr.write("Usage: %s gridsize > output.mtx\n" % program_name)
    sys.exit(1)

gridsize = int(sys.argv[1])

def T(n):
    n = n * gridsize
    buffer = [(n, n, 4.0)]
    for i in range(n+1, n+gridsize):
        buffer.append((i, i, 4.0))
        buffer.append((i, i-1, -1.0))
    return buffer

def I(m, n):
    m = m * gridsize
    n = n * gridsize
    return [(m+i, n+i, -1.0) for i in range(gridsize)]

buffer = T(0)
for i in range(1, gridsize):
    buffer.extend(T(i))
    buffer.extend(I(i, i-1))

def p(x, y):
    if x[1] < y[1]:
        return -1
    elif x[1] > y[1]:
        return 1
    elif x[0] < y[0]:
        return -1
    elif x[0] > y[0]:
        return 1
    return 0
buffer.sort(p)

print "%%MatrixMarket matrix coordinate real symmetric\r"
print "% generated by", program_name, "on %s\r" % time.asctime()
print "%d %d %d\r" % (gridsize**2, gridsize**2, len(buffer))
for i, j, v in buffer:
    print "%10d %10d  %4.1f\r" % (i+1, j+1, v)
