# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: maketoeplitz.py,v 1.3 2007/10/31 06:15:43 kajiyama Exp $

# maketoeplitz.py generates a Toeplitz matrix in the Matrix Market
# format.

import getopt, os, sys, time

program_name = os.path.basename(sys.argv[0])

try:
    options, args = getopt.getopt(sys.argv[1:], "g:")
except getopt.error, e:
    sys.stderr.write(e)
    sys.exit(1)
gamma = 1.3
for opt, val in options:
    if opt == "-g":
        gamma = float(val)
if len(args) == 0:
    N = 200
else:
    N = int(args[0])
NNZ = 3 * N - 3

buffer = []
for i in range(N):
    if i >= 2:
        buffer.append((i, i-2, gamma))
    buffer.append((i, i, 2.0))
    if i < N - 1:
        buffer.append((i, i+1, 1.0))

print "%%MatrixMarket matrix coordinate real general\r"
print "% generated by", program_name, "on %s\r" % time.asctime()
print "%d %d %d\r" % (N, N, NNZ)
for i, j, v in buffer:
    print "%10d %10d  %4.1f\r" % (i+1, j+1, v)
