/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_dump_array.c,v 1.3 2007/10/31 06:15:43 kajiyama Exp $
 */

#include <stdio.h>
#include <malloc.h>

#include "mm_read.h"

int
main(int argc, char *argv[])
{
  FILE *fp;
  mm_header header;
  double *array;
  int i, j;

  if (argc < 2) {
    printf("usage: %s filename.mtx\n", argv[0]);
    return -1;
  }
  if ((fp = fopen(argv[1], "rb")) == NULL) {
    perror(argv[0]);
    return -1;
  }
  if (mm_read_header(fp, &header) < 0)
    return -1;
  if (mm_read_array(fp, &header, &array) < 0)
    return -1;
  fclose(fp);
  printf("%d x %d matrix\n", header.m, header.n);
  for (j = 0; j < header.n; j++)
    for (i = 0; i < header.m; i++)
      printf("%8d, %8d = %13e\n", i, j, array[i + header.m * j]);
  free(array);
  return 0;
}
