/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_read.c,v 1.4 2007/10/31 06:15:43 kajiyama Exp $
 */

#include <stdio.h>
#include <string.h>

#include "mm_read.h"

/* Note: The file passed to the argument `fp' has to be opened
   in the binary mode in Microsoft Windows. */
int
mm_read_header(FILE *fp, mm_header *header)
{
  char buf[1024];
  char f1[1024], f2[1024], f3[1024], f4[1024], f5[1024];

  if (fgets(buf, sizeof(buf), fp) == NULL) {
    fprintf(stderr, "Error: mm_read_header: cannot read header fields\n");
    return -1;
  }
  if (sscanf(buf, "%s %s %s %s %s", f1, f2, f3, f4, f5) != 5) {
    fprintf(stderr, "Error: mm_read_header: cannot parse header fields\n");
    return -1;
  }
  if (strcmp(f1, "%%MatrixMarket") || strcmp(f2, "matrix")) {
    fprintf(stderr, "Error: mm_read_header: invalid magic string.\n");
    return -1;
  }
  if (strcmp(f3, "coordinate") == 0)
    header->format = mm_coordinate;
  else if (strcmp(f3, "array") == 0)
    header->format = mm_array;
  else {
    fprintf(stderr, "Error: mm_read_header: invalid format.\n");
    return -1;
  }
  if (strcmp(f4, "integer") == 0)
    header->field = mm_integer;
  else if (strcmp(f4, "real") == 0)
    header->field = mm_real;
  else if (strcmp(f4, "complex") == 0)
    header->field = mm_complex;
  else {
    fprintf(stderr, "Error: mm_read_header: invalid field.\n");
    return -1;
  }
  if (strcmp(f5, "general") == 0)
    header->symmetry = mm_general;
  else if (strcmp(f5, "symmetric") == 0)
    header->symmetry = mm_symmetric;
  else if (strcmp(f5, "skew-symmetric") == 0)
    header->symmetry = mm_skew_symmetric;
  else {
    fprintf(stderr, "Error: mm_read_header: invalid symmetry.\n");
    return -1;
  }
  while (buf[0] == '%') {
    if (fgets(buf, sizeof(buf), fp) == NULL) {
      fprintf(stderr, "Error: mm_read_header: cannot read metrics.\n");
      return -1;
    }
  }
  if (header->format == mm_coordinate &&
      sscanf(buf, "%d %d %d", &header->m, &header->n, &header->nnz) != 3 ||
      sscanf(buf, "%d %d", &header->m, &header->n) != 2) {
    fprintf(stderr, "Error: mm_read_header: cannot parse metrics.\n");
    return -1;
  }
  if (header->format == mm_array)
    header->nnz = header->m * header->n;
  return 0;
}

int
mm_read_element(FILE *fp, double *v)
{
  char buf[1024];

  if (fgets(buf, sizeof(buf), fp) == NULL) {
    fprintf(stderr, "Error: mm_read_element: cannot read data.\n");
    return -1;
  }
  if (sscanf(buf, "%le", v) != 1) {
    fprintf(stderr, "Error: mm_read_element: cannot parse data.\n");
    return -1;
  }
  return 0;
}

int
mm_read_index(FILE *fp, int *i, int *j)
{
  char buf[1024];

  if (fgets(buf, sizeof(buf), fp) == NULL) {
    fprintf(stderr, "Error: mm_read_index: cannot read data.\n");
    return -1;
  }
  if (sscanf(buf, "%d %d", i, j) != 2) {
    fprintf(stderr, "Error: mm_read_index: cannot parse data.\n");
    return -1;
  }
  return 0;
}

int
mm_read_indexed_element(FILE *fp, int *i , int *j, double *v)
{
  char buf[1024];

  if (fgets(buf, sizeof(buf), fp) == NULL) {
    fprintf(stderr, "Error: mm_read_indexed_element: cannot read data\n");
    return -1;
  }
  if (sscanf(buf, "%d %d %le", i, j, v) != 3) {
    fprintf(stderr, "Error: mm_read_indexed_element: cannot parse data\n");
    return -1;
  }
  return 0;
}
