/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_read.h,v 1.4 2007/10/31 06:15:43 kajiyama Exp $
 */

#ifndef MM_READ_H
#define MM_READ_H 1

typedef struct {
  enum { mm_coordinate, mm_array } format;
  enum { mm_integer, mm_real, mm_complex, mm_pattern } field;
  enum { mm_general, mm_symmetric, mm_skew_symmetric, mm_hermitian } symmetry;
  int m, n, nnz;
} mm_header;

#ifdef __cplusplus
extern "C" {
#endif

/* mm_read.c */
int mm_read_header(FILE *fp, mm_header *header);
int mm_read_element(FILE *fp, double *v);
int mm_read_index(FILE *fp, int *i, int *j);
int mm_read_indexed_element(FILE *fp, int *i , int *j, double *v);

/* mm_read_array.c */
int mm_read_array(FILE *fp, mm_header *header, double **array_ptr);

/* mm_read_band.c */
int mm_read_band(FILE *fp, mm_header *header, 
		 double **array_ptr, int *kl, int *ku, int flag);

/* mm_read_crs.c */
int mm_read_crs(FILE *fp, mm_header *header,
		double **value_ptr, int **row_ptr, int **index_ptr);

#ifdef __cplusplus
}
#endif

#endif /* MM_READ_H */
