/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_read_array.c,v 1.3 2007/10/31 06:15:43 kajiyama Exp $
 */

#include <stdio.h>
#include <malloc.h>

#include "mm_read.h"

#define should_not_reach_here  fprintf(stderr, \
  "*** should not reach here (%s, line %d)\n", __FILE__, __LINE__)

int
mm_read_array(FILE *fp, mm_header *header, double **array_ptr)
	      
{
  double *array, v;
  int i, j;

  if (header->format != mm_array) {
    fprintf(stderr, "Error: mm_read_array: header->format != mm_array\n");
    return -1;
  }
  if (header->field != mm_real) {
    fprintf(stderr, "Error: mm_read_array: header->field != mm_real\n");
    return -1;
  }
  array = (double *)malloc(sizeof(double) * header->m * header->n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
  for (j = 0; j < header->n; j++)
    for (i = 0; i < header->m; i++)
      array[i * header->m + j] = 0;
#endif
  for (j = 0; j < header->n; j++) {
    for (i = 0; i < header->m; i++) {
      if (mm_read_element(fp, &v) < 0)
	break;
      if (i == j || header->symmetry == mm_general) {
	array[i + header->m * j] = v;
      } else if (header->symmetry == mm_symmetric) {
	array[i + header->m * j] = v;
	array[j + header->m * i] = v;
      } else { /* header->symmetry == mm_skew_symmetric */
	array[i + header->m * j] = v;
	array[j + header->m * i] = -v;
      }
    }
  }
  *array_ptr = array;
  return 0;
}
