/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_read_band.c,v 1.4 2007/10/31 06:15:43 kajiyama Exp $
 */

#include <stdio.h>
#include <malloc.h>

#include "mm_read.h"

#define should_not_reach_here  fprintf(stderr, \
  "*** should not reach here (%s, line %d)\n", __FILE__, __LINE__)

int
mm_read_band(FILE *fp, mm_header *header,
	     double **array_ptr, int *kl_ptr, int *ku_ptr, int flag)
{
  int i, j, kl, ku, n, lda, offset;
  double *array, v;
  long pos;

  if (header->format != mm_coordinate) {
    fprintf(stderr, "Error: mm_read_band: header->format != mm_coordinate\n");
    return -1;
  }
  if (header->field != mm_real) {
    fprintf(stderr, "Error: mm_read_band: header->field != mm_real\n");
    return -1;
  }
  /* first pass */
  pos = ftell(fp);
  kl = ku = 0;
  for (n = 0; n < header->nnz; n++) {
    if (mm_read_index(fp, &i, &j) < 0)
      break;
    if (header->symmetry == mm_general) {
      if (i > j) {
	if (kl < i - j) kl = i - j;
      } else {
	if (ku < j - i) ku = j - i;
      }
    } else { /* mm_symmetric, mm_skew_symmetric */
      if (kl < i - j) kl = ku = i - j;
    }
  }
  if (flag == 0) {
    lda = kl + ku + 1; /* leading dimension (column-major) */
    offset = 0;
  } else {
    lda = kl * 2 + ku + 1; /* additional 'kl' rows for LU factorization */
    offset = kl;
  }

  /* second pass */
  fseek(fp, pos, SEEK_SET);
#if 0 /* ENABLE_NUMA */
  array = (double *)malloc(sizeof(double) * lda * header->n);
#pragma omp parallel for
  for (i = 0; i < header->n; i++) {
    int k;
    for (k = 0; k < lda; k++)
      array[i * lda + k] = 0;
  }
  printf("array initialized for NUMA\n");
#else
  array = (double *)calloc(lda * header->n, sizeof(double));
#endif
  for (n = 0; n < header->nnz; n++) {
    if (mm_read_indexed_element(fp, &i, &j, &v) < 0)
      break;
    i--; j--; /* 0 origin */
    if (i == j || header->symmetry == mm_general) {
      array[lda * j + offset + ku + i - j] = v;
    } else if (header->symmetry == mm_symmetric) {
      array[lda * j + offset + ku + i - j] = v;
      array[lda * i + offset + ku + j - i] = v;
    } else { /* header->symmetry == mm_skew_symmetric */
      array[lda * j + offset + ku + i - j] = v;
      array[lda * i + offset + ku + j - i] = -v;
    }
  }
  *array_ptr = array;
  *kl_ptr = kl;
  *ku_ptr = ku;
  return 0;
}
