/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: mm_read_crs.c,v 1.4 2007/10/31 06:15:43 kajiyama Exp $
 */

#include <stdio.h>
#include <malloc.h>

#include "mm_read.h"

int
mm_read_crs(FILE *fp, mm_header *header,
	    double **value_ptr, int **row_ptr, int **index_ptr)
{
  int i, j, k, n, nnz;
  double *value, v;
  int *row, *index, *buf;
  long pos;

  if (header->format != mm_coordinate) {
    fprintf(stderr, "Error: mm_read_crs: header->format != mm_coordinate\n");
    return -1;
  }
  if (header->field != mm_real) {
    fprintf(stderr, "Error: mm_read_crs: header->field != mm_real\n");
    return -1;
  }
  /* first pass */
  pos = ftell(fp);
  buf = (int *)calloc(header->m, sizeof(int));
  for (n = 0; n < header->nnz; n++) {
    if (mm_read_index(fp, &i, &j) < 0)
      break;
    i--; j--; /* 0 origin */
    if (i == j || header->symmetry == mm_general) {
      buf[i]++;
    } else { /* mm_symmetric, mm_skew_symmetric */
      buf[i]++;
      buf[j]++;
    }
  }
  row = (int *)malloc(sizeof(int) * (header->m + 1));
  nnz = 0;
  for (n = 0; n < header->m; n++) {
    row[n] = nnz;
    nnz += buf[n];
  }
  row[header->m] = nnz;
  /* second pass */
  fseek(fp, pos, SEEK_SET);
  value = (double *)malloc(sizeof(double) * nnz);
  index = (int *)malloc(sizeof(int) * nnz);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i, j)
  for (i = 0; i < header->m; i++) {
    for (j = row[i]; j < row[i+1]; j++) {
      value[j] = 0.0;
      index[j] = 0;
    }
  }
#endif
  for (n = 0; n < header->m; n++)
    buf[n] = row[n];
  for (n = 0; n < header->nnz; n++) {
    if (mm_read_indexed_element(fp, &i, &j, &v) < 0)
      break;
    i--; j--; /* 0 origin */
    if (i == j || header->symmetry == mm_general) {
      k = buf[i]++; index[k] = j; value[k] = v;
    } else if (header->symmetry == mm_symmetric) {
      k = buf[i]++; index[k] = j; value[k] = v;
      k = buf[j]++; index[k] = i; value[k] = v;
    } else { /* header->symmetry == mm_skew_symmetric */
      k = buf[i]++; index[k] = j; value[k] = v;
      k = buf[j]++; index[k] = i; value[k] = -v;
    }
  }
  *value_ptr = value;
  *row_ptr = row;
  *index_ptr = index;
  return 0;
}
