#!/usr/bin/env python
#
# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: demo3.py,v 1.4 2007/10/31 06:19:46 kajiyama Exp $

# Python version of demo3.c

import sys
import math
import time

from silc import *

if len(sys.argv) > 1:
    N = int(sys.argv[1])
else:
    N = 1000
NNZ = 3 * N - 2

object = Envelope()

# create a tridiagonal matrix in CRS format
value = []
b = []
x = []
row = []
index = []
for i in range(N):
    row.append(len(value))
    if i == 0:
        value.append( 2.0); index.append(i)
        value.append(-1.0); index.append(i + 1)
        b.append(-1.0)
        x.append( 0.0)
    elif i < N - 1:
        value.append(-1.0); index.append(i - 1)
        value.append( 2.0); index.append(i)
        value.append(-1.0); index.append(i + 1)
        b.append(0.0)
        x.append(0.0)
    else:
        value.append(-1.0); index.append(i - 1)
        value.append( 2.0); index.append(i)
        b.append(-1.0)
        x.append( 0.0)
        row.append(len(value))

t = time.time()

# initialize SILC
SILC_INIT()

# put matrix A
object.v = value
object.type = SILC_MATRIX_TYPE
object.precision = SILC_DOUBLE
object.format = SILC_FORMAT_CRS
object.m = object.n = N
object.nnz = NNZ
object.row = row
object.index = index
SILC_PUT("A", object)

# put vector b
object.v = b
object.type = SILC_COLUMN_VECTOR_TYPE
object.precision = SILC_DOUBLE
object.length = N
SILC_PUT("b", object)

# solve linear equations
SILC_EXEC("x = A \\ b")

# get vector x
SILC_GET(object, "x")
x = object.v

# finalize SILC
SILC_FINALIZE()

t = time.time() - t
print "%lfs" % t

# check solution
norm = 0.0
for i in range(N):
    d = b[i]
    for j in range(row[i], row[i+1]):
        d = d - value[j] * x[index[j]]
    norm = norm + d * d
print "||b-Ax|| = %e" % math.sqrt(norm)
