# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: mm.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

class Header:
    def __init__(self, format, field, symmetry, m, n, nnz=None):
        self.format = format
        self.field = field
        self.symmetry = symmetry
        self.m = m
        self.n = n
        self.nnz = nnz

def read_header(file):
    line = file.readline()
    if not line:
        raise ValueError, "unexpected end of file"
    comment = line.split()
    assert comment[0] == "%%MatrixMarket"
    assert comment[1] == "matrix"
    assert comment[2] in ["coordinate", "array"]
    assert comment[3] in ["integer", "real", "complex"]
    assert comment[4] in ["general", "symmetric", "skew-symmetric",
                          "hermitian"]
    while line[0] == "%":
        line = file.readline()
        if not line:
            raise ValueError, "unexpected end of file"
    metrics = line.split()
    if len(metrics) == 3:
        m, n, nnz = int(metrics[0]), int(metrics[1]), int(metrics[2])
    elif len(metrics) == 2:
        m, n, nnz = int(metrics[0]), int(metrics[1]), None
    else:
        raise ValueError, "cannot parse metrics"
    return Header(comment[2], comment[3], comment[4], m, n, nnz)

def read_crs(file, header):
    if header.format != "coordinate":
        raise ValueError, "unsupported format: " + header.format
    if header.field == "real":
        def parse(line):
            line = line.split()
            return int(line[0]), int(line[1]), float(line[2])
    elif header.field == "complex":
        def parse(line):
            line = line.split()
            return int(line[0]), int(line[1]), complex(float(line[2]),
                                                       float(line[3]))
    else:        
        raise ValueError, "unsupported field: " + header.field
    buffer = []
    while 1:
        line = file.readline()
        if not line:
            break
        i, j, v = parse(line)
        if i == j or header.symmetry == "general":
            buffer.append((i, j, v))
        elif header.symmetry == "symmetric":
            buffer.append((i, j, v))
            buffer.append((j, i, v))
        elif header.symmetry == "skew-symmetric":
            buffer.append((i, j, v))
            buffer.append((j, i, -v))
        elif header.symmetry == "hermitian":
            buffer.append((i, j, v))
            buffer.append((j, i, v.conjugate()))
        else:
            raise RuntimeError, "should not reach here"
    buffer.sort()
    value, row, index = [], [], []
    last = 0
    for i, j, v in buffer:
        if last != i:
            row.append(len(value))
            last = i
        value.append(v)
        index.append(j-1)
    row.append(len(value))
    return value, row, index
