# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: silctest.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

from silc import *

import unittest

class TestCase(unittest.TestCase):
    def setUp(self):
        self.put_method = {
            SILC_SCALAR_TYPE:        self.put_scalar,
            SILC_ROW_VECTOR_TYPE:    self.put_row_vector,
            SILC_COLUMN_VECTOR_TYPE: self.put_column_vector,
            SILC_MATRIX_TYPE:        self.put_matrix}
        self.client = Client()
        self.client.INIT()
        self.client.EXEC("prefer dense")
    def put_scalar(self, name, value, precision):
        obj = Envelope()
        obj.type = SILC_SCALAR_TYPE
        obj.precision = precision
        obj.v = value
        self.client.PUT(name, obj)
    def put_row_vector(self, name, value, precision):
        obj = Envelope()
        obj.type = SILC_ROW_VECTOR_TYPE
        obj.precision = precision
        obj.length = len(value)
        obj.v = value
        self.client.PUT(name, obj)
    def put_column_vector(self, name, value, precision):
        obj = Envelope()
        obj.type = SILC_COLUMN_VECTOR_TYPE
        obj.precision = precision
        obj.length = len(value)
        obj.v = value
        self.client.PUT(name, obj)
    def put_matrix(self, name, m, n, value, precision):
        obj = Envelope()
        obj.type = SILC_MATRIX_TYPE
        obj.format = SILC_FORMAT_DENSE
        obj.precision = precision
        obj.m, obj.n = m, n
        obj.v = value
        self.client.PUT(name, obj)
    def put_matrix_crs(self, name, m, n, value, row, index, precision):
        obj = Envelope()
        obj.type = SILC_MATRIX_TYPE
        obj.format = SILC_FORMAT_CRS
        obj.precision = precision
        obj.m, obj.n = m, n
        obj.nnz = len(value)
        obj.v = value
        obj.row = row
        obj.index = index
        self.client.PUT(name, obj)
    def put_matrix_band(self, name, m, n, l, u, value, precision):
        obj = Envelope()
        obj.type = SILC_MATRIX_TYPE
        obj.format = SILC_FORMAT_BAND
        obj.precision = precision
        obj.m, obj.n = m, n
        obj.l, obj.u = l, u
        obj.v = value
        self.client.PUT(name, obj)
    def execute(self, statement):
        self.client.EXEC(statement)
    def evaluate(self, expression):
        self.client.EXEC("_=" + expression)
        return self.get("_")
    def get(self, name):
        obj = Envelope()
        self.client.GET(obj, name)
        if obj.type == SILC_MATRIX_TYPE and obj.format == SILC_FORMAT_CRS:
            for i in range(obj.m):
                start, end = obj.row[i], obj.row[i+1]
                buffer = []
                for j in range(start, end):
                    buffer.append((obj.index[j], obj.v[j]))
                buffer.sort()
                for j in range(start, end):
                    obj.index[j], obj.v[j] = buffer[j - start]
        return obj
    def tearDown(self):
        self.client.FINALIZE()

def main():
    unittest.main()
