# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_dense.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

from silc import *

import silctest

class MatrixMatrixAdditionTestCase(silctest.TestCase):
    def put_all(self, precision):
        value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        value = [[4, 3], [2, 1]]
        self.put_matrix("B", 2, 2, value, precision)
    def do_add(self, precision):
        self.put_all(precision)
        return self.evaluate("A + B").v
    def do_sub(self, precision):
        self.put_all(precision)
        return self.evaluate("A - B").v
    def runTest(self):
        assert self.do_add(SILC_INT) == [[5, 5], [5, 5]]
        assert self.do_add(SILC_LONG) == [[5L, 5L], [5L, 5L]]
        assert self.do_add(SILC_FLOAT) == \
               self.do_add(SILC_DOUBLE) == [[5.0, 5.0], [5.0, 5.0]]
        assert self.do_add(SILC_COMPLEX) == \
               self.do_add(SILC_DOUBLE_COMPLEX) == \
               [[(5+0j), (5+0j)], [(5+0j), (5+0j)]]
        assert self.do_sub(SILC_INT) == [[-3, -1], [1, 3]]
        assert self.do_sub(SILC_LONG) == [[-3L, -1L], [1L, 3L]]
        assert self.do_sub(SILC_FLOAT) == \
               self.do_sub(SILC_DOUBLE) == [[-3.0, -1.0], [1.0, 3.0]]
        assert self.do_sub(SILC_COMPLEX) == \
               self.do_sub(SILC_DOUBLE_COMPLEX) == \
               [[(-3+0j), (-1+0j)], [(1+0j), (3+0j)]]

class MatrixVectorProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        value = [1, 1]
        self.put_column_vector("b", value, precision)
        answer = self.evaluate("A * b")
        assert answer.type == SILC_COLUMN_VECTOR_TYPE
        return answer.v
    def runTest(self):
        assert self.do_test(SILC_INT) == [3, 7]
        assert self.do_test(SILC_LONG) == [3L, 7L]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [3.0, 7.0]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == [(3+0j), (7+0j)]

class MatrixVectorProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        value = [1, 1]
        self.put_row_vector("b", value, precision)
        answer = self.evaluate("b * A")
        assert answer.type == SILC_ROW_VECTOR_TYPE
        return answer.v
    def runTest(self):
        assert self.do_test(SILC_INT) == [4, 6]
        assert self.do_test(SILC_LONG) == [4L, 6L]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [4.0, 6.0]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == [(4+0j), (6+0j)]

class MatrixMatrixProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        return self.evaluate("A * A").v
    def runTest(self):
        assert self.do_test(SILC_INT) == [[7, 10], [15, 22]]
        assert self.do_test(SILC_LONG) == [[7L, 10L], [15L, 22L]]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [[7.0, 10.0], [15.0, 22.0]]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == \
               [[(7+0j), (10+0j)], [(15+0j), (22+0j)]]

class VectorVectorProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value = [2, 4]
        self.put_column_vector("c", value, precision)
        self.put_row_vector("r", value, precision)
        return self.evaluate("c * r").v
    def runTest(self):
        assert self.do_test(SILC_INT) == [[4, 8], [8, 16]]
        assert self.do_test(SILC_LONG) == [[4L, 8L], [8L, 16L]]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [[4.0, 8.0], [8.0, 16.0]]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == \
               [[(4+0j), (8+0j)], [(8+0j), (16+0j)]]

class MatrixScalarProductTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        value = 2
        self.put_scalar("a", value, precision)
        value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        assert self.evaluate("a * A").v == self.evaluate("A * a").v == answer
    def runTest(self):
        self.do_test(SILC_INT, [[2, 4], [6, 8]])
        self.do_test(SILC_LONG, [[2L, 4L], [6L, 8L]])
        self.do_test(SILC_FLOAT, [[2.0, 4.0], [6.0, 8.0]])
        self.do_test(SILC_DOUBLE, [[2.0, 4.0], [6.0, 8.0]])
        self.do_test(SILC_COMPLEX, [[(2+0j), (4+0j)], [(6+0j), (8+0j)]])
        self.do_test(SILC_DOUBLE_COMPLEX, [[(2+0j), (4+0j)], [(6+0j), (8+0j)]])

class NegateTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            value = [[1+1j, 2+1j], [3+1j, 4+1j]]
        else:
            value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        assert self.evaluate("-A").v == answer
    def runTest(self):
        self.do_test(SILC_INT, [[-1, -2], [-3, -4]])
        self.do_test(SILC_LONG, [[-1L, -2L], [-3L, -4L]])
        self.do_test(SILC_FLOAT, [[-1.0, -2.0], [-3.0, -4.0]])
        self.do_test(SILC_DOUBLE, [[-1.0, -2.0], [-3.0, -4.0]])
        self.do_test(SILC_COMPLEX,
                     [[(-1+1j), (-2+1j)], [(-3+1j), (-4+1j)]])
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [[(-1+1j), (-2+1j)], [(-3+1j), (-4+1j)]])

class ComplexConjugateTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            value = [[1+1j, 2+1j], [3+1j, 4+1j]]
        else:
            value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        assert self.evaluate("A~").v == answer
    def runTest(self):
        self.do_test(SILC_INT, [[1, 2], [3, 4]])
        self.do_test(SILC_LONG, [[1L, 2L], [3L, 4L]])
        self.do_test(SILC_FLOAT, [[1.0, 2.0], [3.0, 4.0]])
        self.do_test(SILC_DOUBLE, [[1.0, 2.0], [3.0, 4.0]])
        self.do_test(SILC_COMPLEX, [[(1-1j), (2-1j)], [(3-1j), (4-1j)]])
        self.do_test(SILC_DOUBLE_COMPLEX, [[(1-1j), (2-1j)], [(3-1j), (4-1j)]])

class TransposeTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            value = [[1+1j, 2+1j], [3+1j, 4+1j]]
        else:
            value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        assert self.evaluate("A~'").v == answer
    def runTest(self):
        self.do_test(SILC_INT, [[1, 3], [2, 4]])
        self.do_test(SILC_LONG, [[1L, 3L], [2L, 4L]])
        self.do_test(SILC_FLOAT, [[1.0, 3.0], [2.0, 4.0]])
        self.do_test(SILC_DOUBLE, [[1.0, 3.0], [2.0, 4.0]])
        self.do_test(SILC_COMPLEX, [[(1+1j), (3+1j)], [(2+1j), (4+1j)]])
        self.do_test(SILC_DOUBLE_COMPLEX, [[(1+1j), (3+1j)], [(2+1j), (4+1j)]])

class ConjugateTransposeTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            value = [[1+1j, 2+1j], [3+1j, 4+1j]]
        else:
            value = [[1, 2], [3, 4]]
        self.put_matrix("A", 2, 2, value, precision)
        assert self.evaluate("A'").v == answer
    def runTest(self):
        self.do_test(SILC_INT, [[1, 3], [2, 4]])
        self.do_test(SILC_LONG, [[1L, 3L], [2L, 4L]])
        self.do_test(SILC_FLOAT, [[1.0, 3.0], [2.0, 4.0]])
        self.do_test(SILC_DOUBLE, [[1.0, 3.0], [2.0, 4.0]])
        self.do_test(SILC_COMPLEX, [[(1-1j), (3-1j)], [(2-1j), (4-1j)]])
        self.do_test(SILC_DOUBLE_COMPLEX, [[(1-1j), (3-1j)], [(2-1j), (4-1j)]])

class FuncMinTestCase(silctest.TestCase):
    def do_test(self, precision, value):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            answer = []
            for j in range(3):
                v = value[0][j]
                m = v.real ** 2 + v.imag ** 2
                for i in range(1, 3):
                    v2 = value[i][j]
                    m2 = v2.real ** 2 + v2.imag ** 2
                    if m > m2:
                        v = v2
                        m = m2
                answer.append(v)
        else:
            answer = []
            for j in range(3):
                v = value[0][j]
                for i in range(1, 3):
                    v2 = value[i][j]
                    if v > v2:
                        v = v2
                answer.append(v)
        self.put_matrix("A", 3, 3, value, precision)
        assert self.evaluate("min(A)").v == answer
    def runTest(self):
        value = [[1, -1, 3],
                 [0, -2, 6],
                 [3, -3, 9]]
        self.do_test(SILC_INT, value)
        self.do_test(SILC_LONG, value)
        self.do_test(SILC_FLOAT,
                     [[float(v) for v in row] for row in value])
        self.do_test(SILC_DOUBLE,
                     [[float(v) for v in row] for row in value])
        self.do_test(SILC_COMPLEX,
                     [[complex(v, 2) for v in row] for row in value])
        self.do_test(SILC_DOUBLE_COMPLEX, 
                     [[complex(v, 2) for v in row] for row in value])

if __name__ == "__main__":
    silctest.main()
