# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_leq.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

import silctest

from silc import *
precision_table = [(int, SILC_INT),
                   (int, SILC_LONG),
                   (float, SILC_FLOAT),
                   (float, SILC_DOUBLE),
                   (complex, SILC_COMPLEX),
                   (complex, SILC_DOUBLE_COMPLEX)]

class LAPACKTestCase(silctest.TestCase):
    def runTest(self):
        matrix = [[ 2.0, -1.0,  0.0,  0.0],
                  [-1.0,  2.0, -1.0,  0.0],
                  [ 0.0, -1.0,  2.0, -1.0],
                  [ 0.0,  0.0, -1.0,  2.0]]
        vector = [-1.0, 0.0, 0.0, -1.0]
        self.execute("prefer blasmodule")
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                continue
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                answer = [-1.0, -1.0, -1.0, -1.0]
            else:
                answer = [-1.0+0j, -1.0+0j, -1.0+0j, -1.0+0j]
            self.put_matrix("a", 4, 4, matrix, precision)
            self.put_row_vector("r", vector, precision)
            self.put_column_vector("c", vector, precision)
            self.execute("x = 0; x = a \\ c")
            obj = self.get("x")
            for i in range(4):
                assert abs(obj.v[i] - answer[i]) < 1e-6
            self.execute("x = 0; x = r / a")
            obj = self.get("x")
            for i in range(4):
                assert abs(obj.v[i] - answer[i]) < 1e-6

if __name__ == "__main__":
    silctest.main()
