# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_matrix_format.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

import silctest

from silc import *
precision_table = [(int, SILC_INT),
                   (int, SILC_LONG),
                   (float, SILC_FLOAT),
                   (float, SILC_DOUBLE),
                   (complex, SILC_COMPLEX),
                   (complex, SILC_DOUBLE_COMPLEX)]

class FormatConversionTestCase(silctest.TestCase):
    def test_dense_band(self):
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                dense = [[1, 1, 1, 1],
                         [1, 1, 1, 1],
                         [1, 1, 1, 1],
                         [1, 1, 1, 1]]
                band = [[0, 1, 2, 3],
                        [1, 2, 3, 4],
                        [1, 2, 3, 0]]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                dense = [[1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0]]
                band = [[0.0, 1.0, 2.0, 3.0],
                        [1.0, 2.0, 3.0, 4.0],
                        [1.0, 2.0, 3.0, 0.0]]
            else:
                dense = [[1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j]]
                band = [[0+0j, 1+1j, 2+2j, 3+3j],
                        [1+1j, 2+2j, 3+3j, 4+4j],
                        [1+1j, 2+2j, 3+3j, 0+0j]]
            self.put_matrix("a", 4, 4, dense, precision)
            self.put_matrix_band("b", 4, 4, 1, 1, band, precision)
            self.execute("x = 0; x = a + b")
            obj = self.get("x")
            answer = [x[:] for x in dense]
            for i in range(3):
                answer[i][i+1] += band[0][i+1]
            for i in range(4):
                answer[i][i]   += band[1][i]
            for i in range(3):
                answer[i+1][i] += band[2][i]
            assert obj.v == answer
    def test_dense_crs(self):
        row = [0, 1, 2, 3, 4]
        index = [0, 1, 2, 3]
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                dense = [[1, 1, 1, 1],
                         [1, 1, 1, 1],
                         [1, 1, 1, 1],
                         [1, 1, 1, 1]]
                value = [1, 2, 3, 4]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                dense = [[1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0],
                         [1.0, 1.0, 1.0, 1.0]]
                value = [1.0, 2.0, 3.0, 4.0]
            else:
                dense = [[1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j],
                         [1+1j, 1+1j, 1+1j, 1+1j]]
                value = [1+1j, 2+2j, 3+3j, 4+4j]
            self.put_matrix("a", 4, 4, dense, precision)
            self.put_matrix_crs("b", 4, 4, value, row, index, precision)
            self.execute("x = 0; x = a + b")
            obj = self.get("x")
            answer = [x[:] for x in dense]
            for i in range(4):
                for j in range(row[i], row[i+1]):
                    answer[i][index[j]] = answer[i][index[j]] + value[j]
            assert obj.v == answer
    def test_crs_band(self):
        row = [0, 1, 2, 3, 4]
        index = [0, 1, 2, 3]
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                value = [1, 2, 3, 4]
                band = [[0, 1, 1, 1],
                        [1, 1, 1, 1],
                        [1, 1, 1, 0]]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                value = [1.0, 2.0, 3.0, 4.0]
                band = [[0.0, 1.0, 1.0, 1.0],
                        [1.0, 1.0, 1.0, 1.0],
                        [1.0, 1.0, 1.0, 0.0]]
            else:
                value = [1+1j, 2+2j, 3+3j, 4+4j]
                band = [[0+0j, 1+1j, 1+1j, 1+1j],
                        [1+1j, 1+1j, 1+1j, 1+1j],
                        [1+1j, 1+1j, 1+1j, 0+0j]]
            self.put_matrix_crs("a", 4, 4, value, row, index, precision)
            self.put_matrix_band("b", 4, 4, 1, 1, band, precision)
            self.execute("x = 0; x = a + b")
            obj = self.get("x")
            answer = [value[0]] * 10
            for i in range(4):
                answer[i*3] = answer[i*3] + value[i]
            assert obj.v == answer

if __name__ == "__main__":
    silctest.main()
