# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_put_matrix.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

import silctest

from silc import *
precision_table = [(int, SILC_INT),
                   (int, SILC_LONG),
                   (float, SILC_FLOAT),
                   (float, SILC_DOUBLE),
                   (complex, SILC_COMPLEX),
                   (complex, SILC_DOUBLE_COMPLEX)]

class PutMatrixTestCase(silctest.TestCase):
    def test_matrix_to_scalar(self):
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                matrix = [[1]]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                matrix = [[1.0]]
            else:
                matrix = [[1+1j]]
            self.put_matrix("x", 1, 1, matrix, precision)
            obj = self.get("x")
            assert obj.type == SILC_SCALAR_TYPE
            assert obj.precision == precision
            assert obj.v == matrix[0][0]
    def test_matrix_to_row_vector(self):
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                matrix = [[1, 2, 3, 4]]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                matrix = [[1.0, 2.0, 3.0, 4.0]]
            else:
                matrix = [[1+1j, 2+2j, 3+3j, 4+4j]]
            self.put_matrix("x", 1, 4, matrix, precision)
            obj = self.get("x")
            assert obj.type == SILC_ROW_VECTOR_TYPE
            assert obj.precision == precision
            assert obj.v == matrix[0]
    def test_matrix_to_column_vector(self):
        for conv, precision in precision_table:
            if precision in [SILC_INT, SILC_LONG]:
                matrix = [[1], [2], [3], [4]]
            elif precision in [SILC_FLOAT, SILC_DOUBLE]:
                matrix = [[1.0], [2.0], [3.0], [4.0]]
            else:
                matrix = [[1+1j], [2+2j], [3+3j], [4+4j]]
            self.put_matrix("x", 4, 1, matrix, precision)
            obj = self.get("x")
            assert obj.type == SILC_COLUMN_VECTOR_TYPE
            assert obj.precision == precision
            assert obj.v == matrix[0] + matrix[1] + matrix[2] + matrix[3]

if __name__ == "__main__":
    silctest.main()
