# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_sparse_crs.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

from silc import *

import silctest

def CRS(m, n, data):
    assert len(data) == m * n
    value, row, index = [], [0], []
    for i in range(m):
        for j in range(n):
            v = data[i * n + j]
            if v == 0:
                continue
            value.append(v)
            index.append(j)
        row.append(len(value))
    #for i in range(m):
    #    for j in range(row[i], row[i+1]):
    #        print "(%d, %d) = %s" % (i+1, index[j], repr(value[j]))
    return value, row, index

class MatrixVectorProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value, row, index = CRS(4, 4, [1, 0, 0, 0,
                                       2, 3, 0, 0,
                                       0, 4, 5, 0,
                                       6, 0, 7, 8])
        self.put_matrix_crs("A", 4, 4, value, row, index, precision)
        value = [1, 1, 1, 1]
        self.put_column_vector("b", value, precision)
        answer = self.evaluate("A * b")
        assert answer.type == SILC_COLUMN_VECTOR_TYPE
        return answer.v
    def runTest(self):
        assert self.do_test(SILC_INT) == [1, 5, 9, 21]
        assert self.do_test(SILC_LONG) == [1L, 5L, 9L, 21L]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [1.0, 5.0, 9.0, 21.0]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == \
               [(1+0j), (5+0j), (9+0j), (21+0j)]

class VectorMatrixProductTestCase(silctest.TestCase):
    def do_test(self, precision):
        value, row, index = CRS(4, 4, [1, 0, 0, 0,
                                       2, 3, 0, 0,
                                       0, 4, 5, 0,
                                       6, 0, 7, 8])
        self.put_matrix_crs("A", 4, 4, value, row, index, precision)
        value = [1, 1, 1, 1]
        self.put_row_vector("b", value, precision)
        answer = self.evaluate("b * A")
        assert answer.type == SILC_ROW_VECTOR_TYPE
        return answer.v
    def runTest(self):
        assert self.do_test(SILC_INT) == [9, 7, 12, 8]
        assert self.do_test(SILC_LONG) == [9L, 7L, 12L, 8L]
        assert self.do_test(SILC_FLOAT) == \
               self.do_test(SILC_DOUBLE) == [9.0, 7.0, 12.0, 8.0]
        assert self.do_test(SILC_COMPLEX) == \
               self.do_test(SILC_DOUBLE_COMPLEX) == \
               [(9+0j), (7+0j), (12+0j), (8+0j)]               

class MatrixScalarProductTestCase(silctest.TestCase):
    def do_test(self, precision, answer):
        value = 2
        self.put_scalar("a", value, precision)
        value, row, index = CRS(3, 3, [1, 0, 0, 2, 3, 0, 0, 4, 5])
        self.put_matrix_crs("A", 3, 3, value, row, index, precision)
        return self.evaluate("a * A").v == self.evaluate("A * a").v == answer
    def runTest(self):
        assert self.do_test(SILC_INT, [2, 4, 6, 8, 10])
        assert self.do_test(SILC_LONG, [2L, 4L, 6L, 8L, 10L])
        assert self.do_test(SILC_FLOAT, [2.0, 4.0, 6.0, 8.0, 10.0])
        assert self.do_test(SILC_DOUBLE, [2.0, 4.0, 6.0, 8.0, 10.0])
        assert self.do_test(SILC_COMPLEX,
                            [(2+0j), (4+0j), (6+0j), (8+0j), (10+0j)])
        assert self.do_test(SILC_DOUBLE_COMPLEX,
                            [(2+0j), (4+0j), (6+0j), (8+0j), (10+0j)])

class MatrixMatrixAdditionTestCase(silctest.TestCase):
    def do_add(self, precision, value, row, index):
        out = self.evaluate("A + B")
        assert out.v == value and out.row == row and out.index == index
    def do_sub(self, precision, value, row, index):
        out = self.evaluate("A - B")
        assert out.v == value and out.row == row and out.index == index
    def runTest(self):
        A_value, A_row, A_index = CRS(4, 4, [ 1,  4,  4,  4,
                                              0,  1,  0,  0,
                                              0,  0,  1,  0,
                                              0,  0,  0,  1])
        B_value, B_row, B_index = CRS(4, 4, [-2,  0,  0,  0,
                                              1, -2,  0,  0,
                                              1,  0, -2,  0,
                                              1,  0,  0, -2])
        value_add = [-1, 4, 4, 4, 1, -1, 1, -1, 1, -1]
        value_sub = [3, 4, 4, 4, -1, 3, -1, 3, -1, 3]
        row = [0, 4, 6, 8, 10]
        index = [0, 1, 2, 3, 0, 1, 0, 2, 0, 3]
        for precision, conv in [(SILC_INT, int),
                                (SILC_LONG, long),
                                (SILC_FLOAT, float),
                                (SILC_DOUBLE, float),
                                (SILC_COMPLEX, complex),
                                (SILC_DOUBLE_COMPLEX, complex)]:
            self.put_matrix_crs("A", 4, 4, A_value, A_row, A_index, precision)
            self.put_matrix_crs("B", 4, 4, B_value, B_row, B_index, precision)
            self.do_add(precision, [conv(a) for a in value_add], row, index)
            self.do_sub(precision, [conv(a) for a in value_sub], row, index)

class MatrixMatrixMultiplicationTestCase(silctest.TestCase):
    def do_test(self,
                A_m, A_n, A_value, A_row, A_index,
                B_m, B_n, B_value, B_row, B_index,
                value, row, index):
        for precision, conv in [(SILC_INT, int),
                                (SILC_LONG, long),
                                (SILC_FLOAT, float),
                                (SILC_DOUBLE, float),
                                (SILC_COMPLEX, complex),
                                (SILC_DOUBLE_COMPLEX, complex)]:
            self.put_matrix_crs("A", A_m, A_n, A_value, A_row, A_index,
                                precision)
            self.put_matrix_crs("B", B_m, B_n, B_value, B_row, B_index,
                                precision)
            out = self.evaluate("A * B")
            assert out.v == [conv(a) for a in value] and out.row == row and \
                   out.index == index
    def do_problem1(self):
        A_value, A_row, A_index = CRS(4, 4, [2, 0, 0, 0,
                                             0, 2, 0, 0,
                                             0, 2, 2, 0,
                                             2, 0, 0, 2])
        B_value, B_row, B_index = CRS(4, 4, [2, 3, 0, 0,
                                             0, 4, 5, 0,
                                             0, 0, 6, 7,
                                             0, 0, 0, 8])
        value = [4, 6, 8, 10, 8, 22, 14, 4, 6, 16]
        row = [0, 2, 4, 7, 10]
        index = [0, 1, 1, 2, 1, 2, 3, 0, 1, 3]
        self.do_test(4, 4, A_value, A_row, A_index,
                     4, 4, B_value, B_row, B_index,
                     value, row, index)
    def do_problem2(self):
        A_value, A_row, A_index = CRS(4, 3, [2, 0, 0,
                                             0, 2, 0,
                                             0, 2, 2,
                                             2, 0, 0])
        B_value, B_row, B_index = CRS(3, 4, [2, 3, 0, 0,
                                             0, 4, 5, 0,
                                             0, 0, 6, 7])
        value = [4, 6, 8, 10, 8, 22, 14, 4, 6]
        row = [0, 2, 4, 7, 9]
        index = [0, 1, 1, 2, 1, 2, 3, 0, 1]
        self.do_test(4, 3, A_value, A_row, A_index,
                     3, 4, B_value, B_row, B_index,
                     value, row, index)
    def runTest(self):
        self.do_problem1()
        self.do_problem2()

class NegateTestCase(silctest.TestCase):
    def do_test(self, precision, value):
        A_value, A_row, A_index = CRS(3, 3, [1, 0, 2,
                                             0, 3, 4,
                                             0, 0, 5])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [a + 1j for a in A_value]
        self.put_matrix_crs("A", 3, 3, A_value, A_row, A_index, precision)
        obj = self.evaluate("-A")
        assert obj.v == value and obj.row == A_row and obj.index == A_index
    def runTest(self):
        self.do_test(SILC_INT, [-1, -2, -3, -4, -5])
        self.do_test(SILC_LONG, [-1, -2, -3, -4, -5])
        self.do_test(SILC_FLOAT, [-1.0, -2.0, -3.0, -4.0, -5.0])
        self.do_test(SILC_DOUBLE, [-1.0, -2.0, -3.0, -4.0, -5.0])
        self.do_test(SILC_COMPLEX,
                     [(-1+1j), (-2+1j), (-3+1j), (-4+1j), (-5+1j)])
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [(-1+1j), (-2+1j), (-3+1j), (-4+1j), (-5+1j)])

class ComplexConjugateTestCase(silctest.TestCase):
    def do_test(self, precision, value):
        A_value, A_row, A_index = CRS(3, 3, [1, 0, 2,
                                             0, 3, 4,
                                             0, 0, 5])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [a + 1j for a in A_value]
        self.put_matrix_crs("A", 3, 3, A_value, A_row, A_index, precision)
        obj = self.evaluate("A~")
        assert obj.v == value and obj.row == A_row and obj.index == A_index
    def runTest(self):
        self.do_test(SILC_INT, [1, 2, 3, 4, 5])
        self.do_test(SILC_LONG, [1, 2, 3, 4, 5])
        self.do_test(SILC_FLOAT, [1.0, 2.0, 3.0, 4.0, 5.0])
        self.do_test(SILC_DOUBLE, [1.0, 2.0, 3.0, 4.0, 5.0])
        self.do_test(SILC_COMPLEX,
                     [(1-1j), (2-1j), (3-1j), (4-1j), (5-1j)])
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [(1-1j), (2-1j), (3-1j), (4-1j), (5-1j)])

class TransposeTestCase(silctest.TestCase):
    def do_test(self, precision, value, row, index):
        A_value, A_row, A_index = CRS(3, 3, [1, 0, 2,
                                             0, 3, 4,
                                             0, 0, 5])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [a + 1j for a in A_value]
        self.put_matrix_crs("A", 3, 3, A_value, A_row, A_index, precision)
        obj = self.evaluate("A~'")
        assert obj.v == value and obj.row == row and obj.index == index
    def runTest(self):
        row = [0, 1, 2, 5]
        index = [0, 1, 0, 1, 2]
        self.do_test(SILC_INT, [1, 3, 2, 4, 5], row, index)
        self.do_test(SILC_LONG, [1, 3, 2, 4, 5], row, index)
        self.do_test(SILC_FLOAT, [1.0, 3.0, 2.0, 4.0, 5.0], row, index)
        self.do_test(SILC_DOUBLE, [1.0, 3.0, 2.0, 4.0, 5.0], row, index)
        self.do_test(SILC_COMPLEX,
                     [(1+1j), (3+1j), (2+1j), (4+1j), (5+1j)], row, index)
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [(1+1j), (3+1j), (2+1j), (4+1j), (5+1j)], row, index)

class ConjugateTransposeTestCase(silctest.TestCase):
    def do_test(self, precision, value, row, index):
        A_value, A_row, A_index = CRS(3, 3, [1, 0, 2,
                                             0, 3, 4,
                                             0, 0, 5])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [a + 1j for a in A_value]
        self.put_matrix_crs("A", 3, 3, A_value, A_row, A_index, precision)
        obj = self.evaluate("A'")
        assert obj.v == value and obj.row == row and obj.index == index
    def runTest(self):
        row = [0, 1, 2, 5]
        index = [0, 1, 0, 1, 2]
        self.do_test(SILC_INT, [1, 3, 2, 4, 5], row, index)
        self.do_test(SILC_LONG, [1, 3, 2, 4, 5], row, index)
        self.do_test(SILC_FLOAT, [1.0, 3.0, 2.0, 4.0, 5.0], row, index)
        self.do_test(SILC_DOUBLE, [1.0, 3.0, 2.0, 4.0, 5.0], row, index)
        self.do_test(SILC_COMPLEX,
                     [(1-1j), (3-1j), (2-1j), (4-1j), (5-1j)], row, index)
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [(1-1j), (3-1j), (2-1j), (4-1j), (5-1j)], row, index)

class FuncDiagTestCase(silctest.TestCase):
    def do_test(self, precision, value):
        self.put_column_vector("v", value, precision)
        obj = self.evaluate("diag(v)")
        assert obj.m == obj.n == 5 and obj.v == value and \
               obj.row == [0, 1, 2, 3, 4, 5] and \
               obj.index == [0, 1, 2, 3, 4]
        obj = self.evaluate("diag(v, 1)")
        assert obj.m == obj.n == 6 and obj.v == value and \
               obj.row == [0, 1, 2, 3, 4, 5, 5] and \
               obj.index == [1, 2, 3, 4, 5]
        obj = self.evaluate("diag(v, -1)")
        assert obj.m == obj.n == 6 and obj.v == value and \
               obj.row == [0, 0, 1, 2, 3, 4, 5] and \
               obj.index == [0, 1, 2, 3, 4]
    def runTest(self):
        self.do_test(SILC_INT, [1, 3, 2, 4, 5])
        self.do_test(SILC_LONG, [1, 3, 2, 4, 5])
        self.do_test(SILC_FLOAT, [1.0, 3.0, 2.0, 4.0, 5.0])
        self.do_test(SILC_DOUBLE, [1.0, 3.0, 2.0, 4.0, 5.0])
        self.do_test(SILC_COMPLEX,
                     [(1-1j), (3-1j), (2-1j), (4-1j), (5-1j)])
        self.do_test(SILC_DOUBLE_COMPLEX,
                     [(1-1j), (3-1j), (2-1j), (4-1j), (5-1j)])

class FuncSparseTestCase(silctest.TestCase):
    def do_test(self, precision, value):
        self.put_column_vector("val", value, precision)
        A_value, A_row, A_index = CRS(4, 4, [11,  0,  0,  0,
                                             21, 22,  0,  0,
                                              0, 32, 33,  0,
                                             41,  0,  0, 44])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [complex(v, 1) for v in A_value]
        obj = self.evaluate("sparse(row, col, val, 4, 4)")
        assert obj.v == A_value and obj.row == A_row and obj.index == A_index
    def runTest(self):
        self.execute("row = {1, 2, 2, 3, 3, 4, 4}")
        self.execute("col = {1, 1, 2, 2, 3, 1, 4}")
        value = [11, 21, 22, 32, 33, 41, 44]
        self.do_test(SILC_INT, value)
        self.do_test(SILC_LONG, value)
        self.do_test(SILC_FLOAT, [float(v) for v in value])
        self.do_test(SILC_DOUBLE, [float(v) for v in value])
        self.do_test(SILC_COMPLEX, [complex(v, 1) for v in value])
        self.do_test(SILC_DOUBLE_COMPLEX, [complex(v, 1) for v in value])

class FuncSparseTestCase2(silctest.TestCase):
    def do_test(self, precision, value):
        self.put_column_vector("val", value, precision)
        A_value, A_row, A_index = CRS(5, 5, [1, 0, 0, 0, 0,
                                             0, 0, 0, 0, 0,
                                             0, 0, 2, 0, 0,
                                             0, 0, 0, 0, 0,
                                             0, 0, 0, 0, 3])
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            A_value = [complex(v, 1) for v in A_value]
        obj = self.evaluate("sparse(row, col, val, 5, 5)")
        assert obj.v == A_value and obj.row == A_row and obj.index == A_index
    def runTest(self):
        self.execute("row = {1, 2, 3, 4, 5}")
        self.execute("col = {1, 2, 3, 4, 5}")
        value = [1, 0, 2, 0, 3]
        self.do_test(SILC_INT, value)
        self.do_test(SILC_LONG, value)
        self.do_test(SILC_FLOAT, [float(v) for v in value])
        self.do_test(SILC_DOUBLE, [float(v) for v in value])
        self.do_test(SILC_COMPLEX, [complex(v, 1) for v in value])
        self.do_test(SILC_DOUBLE_COMPLEX, [complex(v, 1) for v in value])

class DenseMatrixAdditionTestCase(silctest.TestCase):
    def runTest(self):
        A = [[1, 1, 1],
             [1, 1, 1],
             [1, 1, 1]]
        value, row, index = CRS(3, 3, [1,  2,  3,
                                       2,  2,  0,
                                       3,  0,  3])
        answer = [[2, 3, 4],
                  [3, 3, 1],
                  [4, 1, 4]]
        print self.__class__.__name__ + ":",
        print "check if silc_crs_add_dense_matrix is used."
        for precision, conv in [(SILC_INT, int),
                                (SILC_LONG, long),
                                (SILC_FLOAT, float),
                                (SILC_DOUBLE, float),
                                (SILC_COMPLEX, complex),
                                (SILC_DOUBLE_COMPLEX, complex)]:
            self.put_matrix("A", 3, 3, A, precision)
            self.put_matrix_crs("B", 3, 3, value, row, index, precision)
            out = self.evaluate("A + B")
            assert out.v == [[conv(a) for a in r] for r in answer]
            out = self.evaluate("B + A")
            assert out.v == [[conv(a) for a in r] for r in answer]

if __name__ == "__main__":
    silctest.main()
