# Copyright (C) 2002-2007 The SSI Project. All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:
# 
#     * Redistributions of source code must retain the above copyright
#       notice, this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright
#       notice, this list of conditions and the following disclaimer in the
#       documentation and/or other materials provided with the distribution.
#     * Neither the name of the project nor the names of its contributors 
#       may be used to endorse or promote products derived from this software 
#       without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
# INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
#  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
# THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id: test_subscript.py,v 1.4 2007/10/31 06:19:47 kajiyama Exp $

import silctest

from silc import *
precision_table = [(int, SILC_INT),
                   (int, SILC_LONG),
                   (float, SILC_FLOAT),
                   (float, SILC_DOUBLE),
                   (complex, SILC_COMPLEX),
                   (complex, SILC_DOUBLE_COMPLEX)]

class SimpleAssignmentVectorScalarTestCase(silctest.TestCase):
    def runTest(self):
        for conv, precision in precision_table:
            for vector_type in [SILC_COLUMN_VECTOR_TYPE, SILC_ROW_VECTOR_TYPE]:
                if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                    vector = [1+1j, 2+1j, 3+1j]
                    scalar = 4+1j
                    tests = [("a",     scalar),
                             ("-a",   -scalar),
                             ("a~",    scalar.conjugate()),
                             ("-a~",  -scalar.conjugate()),
                             ("a~'",   scalar),
                             ("-a~'", -scalar),
                             ("a'",    scalar.conjugate()),
                             ("-a'",  -scalar.conjugate())]
                else:
                    if precision in [SILC_FLOAT, SILC_DOUBLE]:
                        vector = [1.0, 2.0, 3.0]
                        scalar = 4.0
                    else:
                        vector = [1, 2, 3]
                        scalar = 4
                    tests = [("a",   scalar), ("-a",   -scalar),
                             ("a~",  scalar), ("-a~",  -scalar),
                             ("a'",  scalar), ("-a'",  -scalar),
                             ("a'~", scalar), ("-a'~", -scalar)]
                put_vector = self.put_method[vector_type]
                put_vector("b", vector, precision)
                self.put_scalar("a", scalar, precision)
                for i in range(3):
                    for expr, value in tests:
                        answer = vector[:]
                        answer[i] = value
                        self.execute("bb=b; bb[%d]=%s" % (i+1, expr))
                        obj = self.get("bb")
                        assert obj.v == answer

class SimpleAssignmentVectorVectorTestCase(silctest.TestCase):
    def runTest(self):
        types = [SILC_COLUMN_VECTOR_TYPE, SILC_ROW_VECTOR_TYPE]
        type_combos = [(x, y) for x in types for y in types]
        for conv, precision in precision_table:
            if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                vector1 = [1+1j, 2+1j, 3+1j, 4+1j, 5+1j, 6+1j]
                vector2 = [11-1j, 12-1j, 13-1j]
            else:
                if precision in [SILC_FLOAT, SILC_DOUBLE]:
                    vector1 = [1.0, 2.0, 3.0, 4.0, 5.0, 6.0]
                    vector2 = [11.0, 12.0, 13.0]
                else:
                    vector1 = [1, 2, 3, 4, 5, 6]
                    vector2 = [11, 12, 13]
            for v1_type, v2_type in type_combos:
                if v1_type == v2_type:
                    tests = [("b",   0), ("-b",   1), ("b~", 4), ("-b~", 5)]
                else:
                    tests = [("b'~", 2), ("-b'~", 3), ("b'", 6), ("-b'", 7)]
                put_vector = self.put_method[v1_type]
                put_vector("a", vector1, precision)
                put_vector = self.put_method[v2_type]
                put_vector("b", vector2, precision)
                for i in range(3):
                    for expr, flags in tests:
                        answer = vector1[:]
                        for j in range(3):
                            answer[i+j] = self.modify(flags, vector2[j])
                        self.execute("aa=a")
                        self.execute("aa[%d:%d]=%s" % (i+1, i+3, expr))
                        obj = self.get("aa")
                        assert obj.v == answer
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class ComplexAssignmentVectorVectorTestCase(silctest.TestCase):
    def runTest(self):
        types = [SILC_COLUMN_VECTOR_TYPE, SILC_ROW_VECTOR_TYPE]
        type_combos = [(x, y) for x in types for y in types]
        for conv, precision in precision_table:
            for v1_size, v2_size in [(3, 6), (6, 3)]:
                if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                    vector1 = [(i+1)+1j for i in range(v1_size)]
                    vector2 = [(6-i)-1j for i in range(v2_size)]
                else:
                    if precision in [SILC_FLOAT, SILC_DOUBLE]:
                        vector1 = [float(i+1) for i in range(v1_size)]
                        vector2 = [float(6-i) for i in range(v2_size)]
                    else:
                        vector1 = [i+1 for i in range(v1_size)]
                        vector2 = [6-i for i in range(v2_size)]
                for v1_type, v2_type in type_combos:
                    if v1_type == v2_type:
                        if v1_size > v2_size:
                            tests = [("aa[%d:%d]= b[:]",  0),
                                     ("aa[%d:%d]=-b[:]",  1),
                                     ("aa[%d:%d]= b[:]~", 4),
                                     ("aa[%d:%d]=-b[:]~", 5)]
                        else:
                            tests = [("aa[:]= b[%d:%d]",  0),
                                     ("aa[:]=-b[%d:%d]",  1),
                                     ("aa[:]= b[%d:%d]~", 4),
                                     ("aa[:]=-b[%d:%d]~", 5)]
                    else:
                        if v1_size > v2_size:
                            tests = [("aa[%d:%d]= b[:]'~", 2),
                                     ("aa[%d:%d]=-b[:]'~", 3),
                                     ("aa[%d:%d]= b[:]'",  6),
                                     ("aa[%d:%d]=-b[:]'",  7)]
                        else:
                            tests = [("aa[:]= b[%d:%d]'~", 2),
                                     ("aa[:]=-b[%d:%d]'~", 3),
                                     ("aa[:]= b[%d:%d]'",  6),
                                     ("aa[:]=-b[%d:%d]'",  7)]
                    put_vector = self.put_method[v1_type]
                    put_vector("a", vector1, precision)
                    put_vector = self.put_method[v2_type]
                    put_vector("b", vector2, precision)
                    for expr, flags in tests:
                        for i in range(3):
                            ans = vector1[:]
                            if v1_size > v2_size:
                                for j in range(3):
                                    ans[i+j] = self.modify(flags, vector2[j])
                            else:
                                for j in range(3):
                                    ans[j] = self.modify(flags, vector2[i+j])
                            self.execute("aa=a")
                            self.execute(expr % (i+1, i+3))
                            obj = self.get("aa")
                            assert obj.v == ans
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class SimpleAssignmentMatrixScalarTestCase(silctest.TestCase):
    def runTest(self):
        for conv, precision in precision_table:
            if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                matrix = [[1+1j, 2+1j], [3+1j, 4+1j]]
                scalar = 5+1j
                tests = [("a",     scalar),
                         ("-a",   -scalar),
                         ("a~",    scalar.conjugate()),
                         ("-a~",  -scalar.conjugate()),
                         ("a~'",   scalar),
                         ("-a~'", -scalar),
                         ("a'",    scalar.conjugate()),
                         ("-a'",  -scalar.conjugate())]
            else:
                if precision in [SILC_FLOAT, SILC_DOUBLE]:
                    matrix = [[1.0, 2.0], [3.0, 4.0]]
                    scalar = 5.0
                else:
                    matrix = [[1, 2], [3, 4]]
                    scalar = 5
                tests = [("a",   scalar), ("-a",   -scalar),
                         ("a~",  scalar), ("-a~",  -scalar),
                         ("a'",  scalar), ("-a'",  -scalar),
                         ("a'~", scalar), ("-a'~", -scalar)]
            self.put_matrix("A", 2, 2, matrix, precision)
            self.put_scalar("a", scalar, precision)
            for expr, value in tests:
                for i in range(2):
                    answer = [row[:] for row in matrix]
                    answer[i][i] = value
                    self.execute("AA=A; AA[%d,%d]=%s" % (i+1, i+1, expr))
                    obj = self.get("AA")
                    assert obj.v == answer

class SimpleAssignmentMatrixVectorTestCase(silctest.TestCase):
    def runTest(self):
        for conv, precision in precision_table:
            for vector_type in [SILC_COLUMN_VECTOR_TYPE, SILC_ROW_VECTOR_TYPE]:
                if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                    matrix = [[1+1j, 2+1j, 3+1j],
                              [4+1j, 5+1j, 6+1j],
                              [7+1j, 8+1j, 9+1j]]
                    vector = [-1+1j, -2+2j, -3+3j]
                else:
                    if precision in [SILC_FLOAT, SILC_DOUBLE]:
                        matrix = [[1.0, 2.0, 3.0],
                                  [4.0, 5.0, 6.0],
                                  [7.0, 8.0, 9.0]]
                        vector = [-1.0, -2.0, -3.0]
                    else:
                        matrix = [[1, 2, 3],
                                  [4, 5, 6],
                                  [7, 8, 9]]
                        vector = [-1, -2, -3]
                tests = [("a",  0), ("-a",  1), ("a'~", 2), ("-a'~", 3),
                         ("a~", 4), ("-a~", 5), ("a'",  6), ("-a'",  7)]
                self.put_matrix("A", 3, 3, matrix, precision)
                put_vector = self.put_method[vector_type]
                put_vector("a", vector, precision)
                for expr, flags in tests:
                    for i in range(3):
                        answer = [row[:] for row in matrix]
                        if vector_type is SILC_COLUMN_VECTOR_TYPE:
                            vertical = (flags & 2 == 0)
                        else:
                            vertical = (flags & 2 != 0)
                        self.execute("AA=A")
                        if vertical:
                            self.execute("AA[:,%d]=%s" % (i+1, expr))
                            for index, v in enumerate(vector):
                                answer[index][i] = self.modify(flags, v)
                        else:
                            self.execute("AA[%d,:]=%s" % (i+1, expr))
                            for index, v in enumerate(vector):
                                answer[i][index] = self.modify(flags, v)
                        obj = self.get("AA")
                        assert obj.v == answer
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class SimpleAssignmentMatrixMatrixTestCase(silctest.TestCase):
    def runTest(self):
        for conv, precision in precision_table:
            if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
                matrix1 = [[ 1+1j,  2+1j,  3+1j,  4+1j],
                           [ 5+1j,  6+1j,  7+1j,  8+1j],
                           [ 9+1j, 10+1j, 11+1j, 12+1j],
                           [13+1j, 14+1j, 15+1j, 16+1j]]
                matrix2 = [[-1+1j, -2+2j, -3+3j, -4+4j],
                           [-5+5j, -6+6j, -7+7j, -8+8j]]
            else:
                if precision in [SILC_FLOAT, SILC_DOUBLE]:
                    matrix1 = [[ 1.0,  2.0,  3.0,  4.0],
                               [ 5.0,  6.0,  7.0,  8.0],
                               [ 9.0, 10.0, 11.0, 12.0],
                               [13.0, 14.0, 15.0, 16.0]]
                    matrix2 = [[-1.0, -2.0, -3.0, -4.0],
                               [-5.0, -6.0, -7.0, -8.0]]
                else:
                    matrix1 = [[ 1,  2 , 3,  4],
                               [ 5,  6 , 7,  8],
                               [ 9, 10, 11, 12],
                               [13, 14, 15, 16]]
                    matrix2 = [[-1, -2, -3, -4],
                               [-5, -6, -7, -8]]
            tests = [("B",  0), ("-B",  1), ("B'~", 2), ("-B'~", 3),
                     ("B~", 4), ("-B~", 5), ("B'",  6), ("-B'",  7)]
            self.put_matrix("A", 4, 4, matrix1, precision)
            self.put_matrix("B", 2, 4, matrix2, precision)
            for expr, flags in tests:
                for i in range(3):
                    answer = [row[:] for row in matrix1]
                    self.execute("AA=A")
                    if flags & 2:
                        self.execute("AA[:,%d:%d]=%s" % (i+1, i+2, expr))
                        for m, row in enumerate(matrix2):
                            for n, v in enumerate(row):
                                answer[n][m+i] = self.modify(flags, v)
                    else:
                        self.execute("AA[%d:%d,:]=%s" % (i+1, i+2, expr))
                        for m, row in enumerate(matrix2):
                            for n, v in enumerate(row):
                                answer[m+i][n] = self.modify(flags, v)
                    obj = self.get("AA")
                    assert obj.v == answer
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class ComplexAssignmentVectorMatrixTestCase(silctest.TestCase):
    def put_test_data(self, vector_type, precision):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            vector = [(i+1)-1j for i in range(6)]
            matrix = [[(i*6+j+1)+1j for j in range(6)] for i in range(6)]
        else:
            if precision in [SILC_FLOAT, SILC_DOUBLE]:
                vector = [float(i+1) for i in range(6)]
                matrix = [[float(i*6+j+1) for j in range(6)] for i in range(6)]
            else:
                vector = [i+1 for i in range(6)]
                matrix = [[(i*6+j+1) for j in range(6)] for i in range(6)]
        put_vector = self.put_method[vector_type]
        put_vector("b", vector, precision)
        self.put_matrix("A", 6, 6, matrix, precision)
        return vector, matrix
    def test_all_copy(self):
        for vector_type, tests in [(SILC_ROW_VECTOR_TYPE,
                                    [("bb[:]= A[%d,:]",   0),
                                     ("bb[:]=-A[%d,:]",   1),
                                     ("bb[:]= A[:,%d]'~", 2),
                                     ("bb[:]=-A[:,%d]'~", 3),
                                     ("bb[:]= A[%d,:]~",  4),
                                     ("bb[:]=-A[%d,:]~",  5),
                                     ("bb[:]= A[:,%d]'",  6),
                                     ("bb[:]=-A[:,%d]'",  7)]),
                                   (SILC_COLUMN_VECTOR_TYPE,
                                    [("bb[:]= A[:,%d]",   0),
                                     ("bb[:]=-A[:,%d]",   1),
                                     ("bb[:]= A[%d,:]'~", 2),
                                     ("bb[:]=-A[%d,:]'~", 3),
                                     ("bb[:]= A[:,%d]~",  4),
                                     ("bb[:]=-A[:,%d]~",  5),
                                     ("bb[:]= A[%d,:]'",  6),
                                     ("bb[:]=-A[%d,:]'",  7)])]:
            for conv, precision in precision_table:
                vector, matrix = self.put_test_data(vector_type, precision)
                for expr, flags in tests:
                    for i in range(3):
                        self.execute("bb=b")
                        self.execute(expr % (i+1))
                        answer = vector[:]
                        trans = (flags & 2 == 2)
                        if (vector_type is SILC_COLUMN_VECTOR_TYPE and trans or
                            vector_type is SILC_ROW_VECTOR_TYPE and not trans):
                            for j in range(6):
                                answer[j] = self.modify(flags, matrix[i][j])
                        else:
                            for j in range(6):
                                answer[j] = self.modify(flags, matrix[j][i])
                        obj = self.get("bb")
                        assert obj.v == answer
    def test_partial_copy(self):
        for vector_type, tests in [(SILC_ROW_VECTOR_TYPE,
                                    [("bb[%d:%d]= A[%d,%d:%d]",   0),
                                     ("bb[%d:%d]=-A[%d,%d:%d]",   1),
                                     ("bb[%d:%d]= A[%d:%d,%d]'~", 2),
                                     ("bb[%d:%d]=-A[%d:%d,%d]'~", 3),
                                     ("bb[%d:%d]= A[%d,%d:%d]~",  4),
                                     ("bb[%d:%d]=-A[%d,%d:%d]~",  5),
                                     ("bb[%d:%d]= A[%d:%d,%d]'",  6),
                                     ("bb[%d:%d]=-A[%d:%d,%d]'",  7)]),
                                   (SILC_COLUMN_VECTOR_TYPE,
                                    [("bb[%d:%d]= A[%d:%d,%d]",   0),
                                     ("bb[%d:%d]=-A[%d:%d,%d]",   1),
                                     ("bb[%d:%d]= A[%d,%d:%d]'~", 2),
                                     ("bb[%d:%d]=-A[%d,%d:%d]'~", 3),
                                     ("bb[%d:%d]= A[%d:%d,%d]~",  4),
                                     ("bb[%d:%d]=-A[%d:%d,%d]~",  5),
                                     ("bb[%d:%d]= A[%d,%d:%d]'",  6),
                                     ("bb[%d:%d]=-A[%d,%d:%d]'",  7)])]:
            for conv, precision in precision_table:
                vector, matrix = self.put_test_data(vector_type, precision)
                for expr, flags in tests:
                    for i in range(3):
                        self.execute("bb=b")
                        answer = vector[:]
                        trans = (flags & 2 == 2)
                        if (vector_type is SILC_COLUMN_VECTOR_TYPE and trans or
                            vector_type is SILC_ROW_VECTOR_TYPE and not trans):
                            self.execute(expr % (i+1,i+3,i+1,i+1,i+3))
                            for j in range(i,i+3):
                                answer[j] = self.modify(flags, matrix[i][j])
                        else:
                            self.execute(expr % (i+1,i+3,i+1,i+3,i+1))
                            for j in range(i,i+3):
                                answer[j] = self.modify(flags, matrix[j][i])
                        obj = self.get("bb")
                        assert obj.v == answer
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class ComplexAssignmentMatrixMatrixTestCase(silctest.TestCase):
    def put_test_matrices(self, precision):
        if precision in [SILC_COMPLEX, SILC_DOUBLE_COMPLEX]:
            matrix1 = [[1+1j, 2+1j, 3+1j],
                       [4+1j, 5+1j, 6+1j],
                       [7+1j, 8+1j, 9+1j]]
            matrix2 = [[11-1j, 12-1j, 13-1j],
                       [14-1j, 15-1j, 16-1j],
                       [17-1j, 18-1j, 19-1j]]
        else:
            if precision in [SILC_FLOAT, SILC_DOUBLE]:
                matrix1 = [[1.0, 2.0, 3.0],
                           [4.0, 5.0, 6.0],
                           [7.0, 8.0, 9.0]]
                matrix2 = [[11.0, 12.0, 13.0],
                           [14.0, 15.0, 16.0],
                           [17.0, 18.0, 19.0]]
            else:
                matrix1 = [[1, 2, 3],
                           [4, 5, 6],
                           [7, 8, 9]]
                matrix2 = [[11, 12, 13],
                           [14, 15, 16],
                           [17, 18, 19]]
        self.put_matrix("A", 3, 3, matrix1, precision)
        self.put_matrix("B", 3, 3, matrix2, precision)
        return matrix1, matrix2
    def test_copy_row(self):
        for conv, precision in precision_table:
            matrix1, matrix2 = self.put_test_matrices(precision)
            tests = [("AA[%d,:]= B[%d,:]",   0),
                     ("AA[%d,:]=-B[%d,:]",   1),
                     ("AA[%d,:]= B[:,%d]'~", 2),
                     ("AA[%d,:]=-B[:,%d]'~", 3),
                     ("AA[%d,:]= B[%d,:]~",  4),
                     ("AA[%d,:]=-B[%d,:]~",  5),
                     ("AA[%d,:]= B[:,%d]'",  6),
                     ("AA[%d,:]=-B[:,%d]'",  7)]
            for expr, flags in tests:
                for i in range(3):
                    self.execute("AA=A")
                    self.execute(expr % (i+1, i+1))
                    answer = [row[:] for row in matrix1]
                    if flags & 2:
                        for j in range(3):
                            answer[i][j] = self.modify(flags, matrix2[j][i])
                    else:
                        for j in range(3):
                            answer[i][j] = self.modify(flags, matrix2[i][j])
                    obj = self.get("AA")
                    assert obj.v == answer
    def test_copy_column(self):
        for conv, precision in precision_table:
            matrix1, matrix2 = self.put_test_matrices(precision)
            tests = [("AA[:,%d]= B[:,%d]",   0),
                     ("AA[:,%d]=-B[:,%d]",   1),
                     ("AA[:,%d]= B[%d,:]'~", 2),
                     ("AA[:,%d]=-B[%d,:]'~", 3),
                     ("AA[:,%d]= B[:,%d]~",  4),
                     ("AA[:,%d]=-B[:,%d]~",  5),
                     ("AA[:,%d]= B[%d,:]'",  6),
                     ("AA[:,%d]=-B[%d,:]'",  7)]
            for expr, flags in tests:
                for i in range(3):
                    self.execute("AA=A")
                    self.execute(expr % (i+1, i+1))
                    answer = [row[:] for row in matrix1]
                    if flags & 2:
                        for j in range(3):
                            answer[j][i] = self.modify(flags, matrix2[i][j])
                    else:
                        for j in range(3):
                            answer[j][i] = self.modify(flags, matrix2[j][i])
                    obj = self.get("AA")
                    assert obj.v == answer
    def test_copy_rectangle(self):
        for conv, precision in precision_table:
            matrix1, matrix2 = self.put_test_matrices(precision)
            tests = [("AA[%d:%d,%d:%d]= B[%d:%d,%d:%d]",   0),
                     ("AA[%d:%d,%d:%d]=-B[%d:%d,%d:%d]",   1),
                     ("AA[%d:%d,%d:%d]= B[%d:%d,%d:%d]'~", 2),
                     ("AA[%d:%d,%d:%d]=-B[%d:%d,%d:%d]'~", 3),
                     ("AA[%d:%d,%d:%d]= B[%d:%d,%d:%d]~",  4),
                     ("AA[%d:%d,%d:%d]=-B[%d:%d,%d:%d]~",  5),
                     ("AA[%d:%d,%d:%d]= B[%d:%d,%d:%d]'",  6),
                     ("AA[%d:%d,%d:%d]=-B[%d:%d,%d:%d]'",  7)]
            for expr, flags in tests:
                for p in range(2):
                    for q in range(2):
                        self.execute("AA=A")
                        self.execute(expr % (p+1, p+2, p+1, p+2,
                                             q+1, q+2, q+1, q+2))
                        answer = [row[:] for row in matrix1]
                        if flags & 2:
                            for j in range(2):
                                for k in range(2):
                                    answer[p+j][p+k] = \
                                        self.modify(flags, matrix2[q+k][q+j])
                        else:
                            for j in range(2):
                                for k in range(2):
                                    answer[p+j][p+k] = \
                                        self.modify(flags, matrix2[q+j][q+k])
                        obj = self.get("AA")
                        assert obj.v == answer
    def modify(self, flags, scalar):
        if type(scalar) is complex and flags & 4:
            scalar = scalar.conjugate()
        if flags & 1:
            scalar = -scalar
        return scalar

class OverlappingAssignmentTestCase(silctest.TestCase):
    def runTest(self):
        matrix = [[1.0, 2.0, 3.0], [4.0, 5.0, 6.0], [7.0, 8.0, 9.0]]
        self.put_matrix("A", 3, 3, matrix, SILC_DOUBLE)
        self.execute("A[1:2,1:2]=A[2:3,2:3]")
        obj = self.get("A")
        assert obj.v == [[5.0, 6.0, 3.0], [8.0, 9.0, 6.0], [7.0, 8.0, 9.0]]

if __name__ == "__main__":
    silctest.main()
