/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: silc_cg.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>

#include "client.h"

#define EPSILON 1.0e-10

extern double wtime(void);

int
silc_cg(int n, int nnz, double *value, int *index, int *row,
	double *b, double *x)
{
  silc_envelope_t object;
  int i;
  double nrm2;
  double t, elapsed;

  /* connect to SILC server */
  SILC_INIT();

  /* put matrix A */
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_CRS;
  object.precision = SILC_DOUBLE;
  object.m = object.n = n;
  object.nnz = nnz;
  object.v = value;
  object.row = row;
  object.index = index;
  SILC_PUT("A", &object);

  /* put vector b */
  object.type = SILC_COLUMN_VECTOR_TYPE;
  object.precision = SILC_DOUBLE;
  object.length = n;
  object.v = b;
  SILC_PUT("b", &object);

  t = wtime();

  SILC_EXEC("rho_old = 1.0");
  SILC_EXEC("n = length(b)");
  SILC_EXEC("p = zeros(n, 1)");
  SILC_EXEC("x = zeros(n, 1)");
  SILC_EXEC("r = b");
  SILC_EXEC("bnrm2 = 1.0 / norm2(b)");
  for (i = 0; i < n; i++) { 
    SILC_EXEC("rho = r' * r");
    SILC_EXEC("beta = rho / rho_old");
    SILC_EXEC("p = r + beta * p");
    SILC_EXEC("q = A * p");
    SILC_EXEC("alpha = rho / (p' * q)");
    SILC_EXEC("r = r - alpha * q");
    SILC_EXEC("nrm2 = norm2(r) * bnrm2");
    SILC_EXEC("x = x + alpha * p");
    /* check convergence */
    object.v = &nrm2;
    SILC_GET(&object, "nrm2"); 
#ifdef DEBUG
    printf("iter: %d, res: %e\n", i+1, nrm2);
#endif
    if (nrm2 <= EPSILON)
      break;
    SILC_EXEC("rho_old = rho");
  }

  elapsed = wtime() - t;
  printf("CG: %lf sec.\n", elapsed);

  /* get vector x */
  object.v = x;
  SILC_GET(&object, "x");

  /* close connection */
  SILC_FINALIZE();

  return i; /* number of iterations */
}

int main(int argc, char *argv[])
{
  double *value, *b, *x;
  int N, NNZ, *index, *row, i, n;
  double t, elapsed;

  if (argc == 2)
    N = atol(argv[1]);
  else
    N = 1000;
  NNZ = N * 3 - 2;  /* tridiagonal matrix */

  value = (double *)malloc(sizeof(double) * NNZ);
  b = (double *)malloc(sizeof(double) * N);
  x = (double *)malloc(sizeof(double) * N);
  row = (int *)malloc(sizeof(int) * (N + 1));
  index = (int *)malloc(sizeof(int) * NNZ);

  for (i = 0; i < N; i++) {
    n = row[i] = (i == 0) ? 0 : i * 3 - 1;
    if (i == 0) {
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = -1.0;
      x[i] =  0.0;
    } else if (i < N - 1) {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      value[n] = -1.0; index[n++] = i + 1;
      b[i] = 0.0;
      x[i] = 0.0;
    } else {
      value[n] = -1.0; index[n++] = i - 1;
      value[n] =  2.0; index[n++] = i;
      b[i] = -1.0;
      x[i] =  0.0;
      row[N] = n;
    }
  }

  t = wtime();
  silc_cg(N, NNZ, value, index, row, b, x);
  elapsed = wtime() - t;
  printf("Total: %lf sec.\n", elapsed);
}
