/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: test_band.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include "client.h"

int
main(int argc, char *argv[])
{
  silc_envelope_t object;
  double *value, c;
  int i, j, n, kl, ku, ld;
  double t, elapsed, wtime(void);

  if (argc != 4) {
    printf("Usage: %s [n] [kl] [ku]\n", argv[0]);
    return -1;
  }
  n = atoi(argv[1]);
  kl = atoi(argv[2]);
  ku = atoi(argv[3]);

  ld = kl + ku + 1;
  value = (double *)malloc(sizeof(double) * ld * n);
  for (j = 0; j < n; j++)
    for (i = 0; i < ld; i++)
      value[i + j * ld] = (double)(i + 1);

  t = wtime();

  SILC_INIT();

  object.v = value;
  object.type = SILC_MATRIX_TYPE;
  object.format = SILC_FORMAT_BAND;
  object.precision = SILC_DOUBLE;
  object.m = object.n = n;
  object.l = kl;
  object.u = ku;
  SILC_PUT("A", &object);

  SILC_EXEC("prefer sparse_band");
  SILC_EXEC("c = 1.0 / rcond(A)");

  object.v = &c;
  SILC_GET(&object, "c");

  SILC_FINALIZE();

  free(value);

  elapsed = wtime() - t;

  printf("an estimate for the condition number of A = %.6e\n", c);
  printf("%lf sec.\n", elapsed);
  return 0;
}
