/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: test_band_sa.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

/* lapackdriver.o */
void call_dgbtrf(int m, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, int *info);
void call_dgbcon(char norm, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, double anorm, double *rcond, double *work,
		 int *iwork, int *info);

#define _max(x, y) (((x) > (y)) ? (x) : (y))
#define _min(x, y) (((x) < (y)) ? (x) : (y))

int
main(int argc, char *argv[])
{
  double *value, *v, *p, *q, sum, norm, rcond;
  int i, j, n, kl, ku, ld, lda, end, *ipiv, info;
  double t, elapsed, wtime(void);

  if (argc != 4) {
    printf("Usage: %s [n] [kl] [ku]\n", argv[0]);
    return -1;
  }
  n = atoi(argv[1]);
  kl = atoi(argv[2]);
  ku = atoi(argv[3]);

  ld = kl + ku + 1;
  value = (double *)malloc(sizeof(double) * ld * n);
  for (j = 0; j < n; j++)
    for (i = 0; i < ld; i++)
      value[i + j * ld] = (double)(i + 1);

  t = wtime();

  lda = kl * 2 + ku + 1;
#if 1
  v = (double *)malloc(sizeof(double) * lda * n);
  p = v + kl;
  q = value;
  for (i = 0; i < n; i++) {
    memcpy(p, q, sizeof(double) * ld);
    p += lda;
    q += ld;
  }
#else
  v = (double *)calloc(lda * n, sizeof(double));
  for (j = 0; j < n; j++) {
    end = _min(n, j + kl + 1);
    for (i = _max(0, j - ku); i < end; i++)
      v[kl+ku+i-j+j*lda] = value[ku+i-j+j*ld];
  }
#endif
  norm = 0.0;
  for (j = 0; j < n; j++) {
    sum = 0.0;
    end = _min(n, j + kl + 1);
    for (i = _max(0, j - ku); i < end; i++)
      sum += fabs(value[ku+i-j+j*ld]);
    if (norm < sum)
      norm = sum;
  }
#ifdef DEBUG
  printf("anorm = %le\n", norm);
#endif
  ipiv = (int *)malloc(sizeof(int) * n);
#if 0
  {
    FILE *fp = fopen("dump-standalone-before","w");
    for (j = 0; j < n; j++)
      for (i = 0; i < lda; i++)
        fprintf(fp, "%.6e\n", v[i + j * lda]);
    fclose(fp);
  }
#endif
  call_dgbtrf(n, n, kl, ku, v, lda, ipiv, &info);
#if 0
  {
    FILE *fp = fopen("dump-standalone","w");
    for (j = 0; j < n; j++)
      for (i = 0; i < lda; i++)
        fprintf(fp, "%.6e\n", v[i + j * lda]);
    fclose(fp);
  }
#endif
  if (info == 0) {
    double *work;
    int *iwork;
    work = (double *)malloc(sizeof(double) * 4 * n);
    iwork = (int *)malloc(sizeof(int) * n);
    call_dgbcon('1', n, kl, ku, v, lda, ipiv, norm, &rcond, work, iwork,
		&info);
    free(work);
    free(iwork);
  }
  free(v);
  free(ipiv);
  free(value);
  if (info != 0)
    return -1;

  elapsed = wtime() - t;

  printf("an estimate for the condition number of A = %.6e\n", 1.0 / rcond);
  printf("%lf sec.\n", elapsed);
  return 0;
}
