/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: test_dense_sa.c,v 1.4 2007/10/31 06:22:07 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

/* blasdriver.o */
void
call_dgemv(char trans, int m, int n, double alpha, double *a, int lda,
	   double *x, int incx, double beta, double *y, int incy);

/* lapackdriver.o */
void
call_dgetrf(int m, int n, double *a, int lda, int *ipiv, int *info);
void
call_dgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info);

int main(int argc, char *argv[])
{
  double *A, *b, *x, *p;
  int i, j, *ipiv, info;
  double dot;
  long N;
  double t, elapsed, wtime(void);

  if (argc == 2)
    N = atol(argv[1]);
  else
    N = 1000;

  A = (double *)malloc(sizeof(double) * N * N);
  b = (double *)malloc(sizeof(double) * N);
  x = (double *)malloc(sizeof(double) * N);

  /* create matrix A and vector b */
  for (i = j = 0; i < N * N; i++) {
    A[j] = 1.0 / (i + 1);
    j = (j + N + 1) % (N * N);
  }
  for (i = 0; i < N; i++) {
    b[i] = 0;
    for (j = 0; j < N; j++)
      b[i] += A[j * N + i];
  }

  t = wtime();

  /* solve "Ax = b" */
  ipiv = (int *)malloc(sizeof(int) * N);
  p = (double *)malloc(sizeof(double) * N * N);
  memcpy(p, A, sizeof(double) * N * N);
  memcpy(x, b, sizeof(double) * N);
  call_dgetrf(N, N, p, N, ipiv, &info);
  if (info) {
    fprintf(stderr, "DGETRF failed (info = %d)\n", info);
    return -1;
  }
  call_dgetrs('N', N, 1, p, N, ipiv, x, N, &info);
  if (info) {
    fprintf(stderr, "DGETRS failed (info = %d)\n", info);
    return -1;
  }
  free(ipiv);
  free(p);

  elapsed = wtime() - t;
  printf("%8lf sec.\n", elapsed);

#ifdef DEBUG
  /* check solution */
  p = (double *)malloc(sizeof(double) * N);
  memcpy(p, b, sizeof(double) * N);
  /* p = b - A * x */
  call_dgemv('N', N, N, -1.0, A, N, x, 1, 1.0, p, 1);
  dot = 0.0;
  for (i = 0; i < N; i++)
    dot += p[i] * p[i];
  printf("||b-Ax|| = %le\n", sqrt(dot));
  free(p);
#endif

  return 0;
}
