/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: blas_dumb.c,v 1.4 2007/10/31 06:23:11 kajiyama Exp $
 */

/* [sdcz]copy */

void
call_scopy(int n, float *x, int incx, float *y, int incy)
{
  void scopy_(int *, float *, int *, float *, int *);
  scopy_(&n, x, &incx, y, &incy);
}

void
call_dcopy(int n, double *x, int incx, double *y, int incy)
{
  void dcopy_(int *, double *, int *, double *, int *);
  dcopy_(&n, x, &incx, y, &incy);
}

void
call_ccopy(int n, float *x, int incx, float *y, int incy)
{
  void ccopy_(int *, float *, int *, float *, int *);
  ccopy_(&n, x, &incx, y, &incy);
}

void
call_zcopy(int n, double *x, int incx, double *y, int incy)
{
  void zcopy_(int *, double *, int *, double *, int *);
  zcopy_(&n, x, &incx, y, &incy);
}

/* [sdcz]axpy (compute y := alpha * x + y) */

void
call_saxpy(int n, float alpha, float *x, int incx, float *y, int incy)
{
  void saxpy_(int *, float *, float *, int *, float *, int *);
  saxpy_(&n, &alpha, x, &incx, y, &incy);
}

void
call_daxpy(int n, double alpha, double *x, int incx, double *y, int incy)
{
  void daxpy_(int *, double *, double *, int *, double *, int *);
  daxpy_(&n, &alpha, x, &incx, y, &incy);
}

void
call_caxpy(int n, float alpha[2], float *x, int incx, float *y, int incy)
{
  void caxpy_(int *, float *, float *, int *, float *, int *);
  caxpy_(&n, alpha, x, &incx, y, &incy);
}

void
call_zaxpy(int n, double alpha[2], double *x, int incx, double *y, int incy)
{
  void zaxpy_(int *, double *, double *, int *, double *, int *);
  zaxpy_(&n, alpha, x, &incx, y, &incy);
}

/* general matrix-vector product */

void
call_sgemv(char trans, int m, int n, float alpha, float *a, int lda,
	   float *x, int incx, float beta, float *y, int incy)
{
  void sgemv_(char *, int *, int *, float *, float *, int *,
	      float *, int *, float *, float *, int *);
  sgemv_(&trans, &m, &n, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_dgemv(char trans, int m, int n, double alpha, double *a, int lda,
	   double *x, int incx, double beta, double *y, int incy)
{
  void dgemv_(char *, int *, int *, double *, double *, int *,
	      double *, int *, double *, double *, int *);
  dgemv_(&trans, &m, &n, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_cgemv(char trans, int m, int n, float *alpha, float *a, int lda,
	   float *x, int incx, float *beta, float *y, int incy)
{
  void cgemv_(char *, int *, int *, float *, float *, int *,
	      float *, int *, float *, float *, int *);
  cgemv_(&trans, &m, &n, alpha, a, &lda, x, &incx, beta, y, &incy);
}

void
call_zgemv(char trans, int m, int n, double *alpha, double *a, int lda,
	   double *x, int incx, double *beta, double *y, int incy)
{
  void zgemv_(char *, int *, int *, double *, double *, int *,
	      double *, int *, double *, double *, int *);
  zgemv_(&trans, &m, &n, alpha, a, &lda, x, &incx, beta, y, &incy);
}

/* general band matrix-vector product */

void
call_sgbmv(char trans, int m, int n, int kl, int ku, float alpha, float *a,
	   int lda, float *x, int incx, float beta, float *y, int incy)
{
  void sgbmv_(char *, int *, int *, int *, int *, float *, float *a,
	      int *, float *, int *, float *, float *, int *);
  sgbmv_(&trans, &m, &n, &kl, &ku, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_dgbmv(char trans, int m, int n, int kl, int ku, double alpha, double *a,
	   int lda, double *x, int incx, double beta, double *y, int incy)
{
  void dgbmv_(char *, int *, int *, int *, int *, double *, double *a,
	      int *, double *, int *, double *, double *, int *);
  dgbmv_(&trans, &m, &n, &kl, &ku, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_cgbmv(char trans, int m, int n, int kl, int ku, float *alpha, float *a,
	   int lda, float *x, int incx, float *beta, float *y, int incy)
{
  void cgbmv_(char *, int *, int *, int *, int *, float *, float *a,
	      int *, float *, int *, float *, float *, int *);
  cgbmv_(&trans, &m, &n, &kl, &ku, alpha, a, &lda, x, &incx, beta, y, &incy);
}

void
call_zgbmv(char trans, int m, int n, int kl, int ku, double *alpha, double *a,
	   int lda, double *x, int incx, double *beta, double *y, int incy)
{
  void zgbmv_(char *, int *, int *, int *, int *, double *, double *a,
	      int *, double *, int *, double *, double *, int *);
  zgbmv_(&trans, &m, &n, &kl, &ku, alpha, a, &lda, x, &incx, beta, y, &incy);
}

/* dot product */

float
call_sdot(int n, float *x, int incx, float *y, int incy)
{
  float sdot_(int *, float *, int *, float *, int *);
  return sdot_(&n, x, &incx, y, &incy);
}

double
call_ddot(int n, double *x, int incx, double *y, int incy)
{
  double ddot_(int *, double *, int *, double *, int *);
  return ddot_(&n, x, &incx, y, &incy);
}
