/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: blas_intelmkl.c,v 1.4 2007/10/31 06:23:11 kajiyama Exp $
 */

#include <mkl_blas.h>

int _MKL_BLAS_dlnu; /* XXX: undefined symbol that libmkl_i2p.so requires */

/* [sdcz]copy */

void
call_scopy(int n, float *x, int incx, float *y, int incy)
{
  SCOPY(&n, x, &incx, y, &incy);
}

void
call_dcopy(int n, double *x, int incx, double *y, int incy)
{
  DCOPY(&n, x, &incx, y, &incy);
}

void
call_ccopy(int n, float *x, int incx, float *y, int incy)
{
  CCOPY(&n, (MKL_Complex8 *)x, &incx, (MKL_Complex8 *)y, &incy);
}

void
call_zcopy(int n, double *x, int incx, double *y, int incy)
{
  ZCOPY(&n, (MKL_Complex16 *)x, &incx, (MKL_Complex16 *)y, &incy);
}

/* [sdcz]axpy (compute y := alpha * x + y) */

void
call_saxpy(int n, float alpha, float *x, int incx, float *y, int incy)
{
  SAXPY(&n, &alpha, x, &incx, y, &incy);
}

void
call_daxpy(int n, double alpha, double *x, int incx, double *y, int incy)
{
  DAXPY(&n, &alpha, x, &incx, y, &incy);
}

void
call_caxpy(int n, float alpha[2], float *x, int incx, float *y, int incy)
{
  CAXPY(&n, (MKL_Complex8 *)alpha, (MKL_Complex8 *)x, &incx,
	(MKL_Complex8 *)y, &incy);
}

void
call_zaxpy(int n, double alpha[2], double *x, int incx, double *y, int incy)
{
  ZAXPY(&n, (MKL_Complex16 *)alpha, (MKL_Complex16 *)x, &incx,
	(MKL_Complex16 *)y, &incy);
}

/* general matrix-vector product */

void
call_sgemv(char trans, int m, int n, float alpha, float *a, int lda,
	   float *x, int incx, float beta, float *y, int incy)
{
  SGEMV(&trans, &m, &n, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_dgemv(char trans, int m, int n, double alpha, double *a, int lda,
	   double *x, int incx, double beta, double *y, int incy)
{
  DGEMV(&trans, &m, &n, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_cgemv(char trans, int m, int n, float alpha[2], float *a, int lda,
	   float *x, int incx, float beta[2], float *y, int incy)
{
  CGEMV(&trans, &m, &n, (MKL_Complex8 *)alpha, (MKL_Complex8 *)a, &lda,
	(MKL_Complex8 *)x, &incx, (MKL_Complex8 *)beta,
	(MKL_Complex8 *)y, &incy);
}

void
call_zgemv(char trans, int m, int n, double alpha[2], double *a, int lda,
	   double *x, int incx, double beta[2], double *y, int incy)
{
  ZGEMV(&trans, &m, &n, (MKL_Complex16 *)alpha, (MKL_Complex16 *)a, &lda,
	(MKL_Complex16 *)x, &incx, (MKL_Complex16 *)beta,
	(MKL_Complex16 *)y, &incy);
}

/* general band matrix-vector product */

void
call_sgbmv(char trans, int m, int n, int kl, int ku, float alpha, float *a,
	   int lda, float *x, int incx, float beta, float *y, int incy)
{
  SGBMV(&trans, &m, &n, &kl, &ku, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_dgbmv(char trans, int m, int n, int kl, int ku, double alpha, double *a,
	   int lda, double *x, int incx, double beta, double *y, int incy)
{
  DGBMV(&trans, &m, &n, &kl, &ku, &alpha, a, &lda, x, &incx, &beta, y, &incy);
}

void
call_cgbmv(char trans, int m, int n, int kl, int ku, float *alpha, float *a,
	   int lda, float *x, int incx, float *beta, float *y, int incy)
{
  CGBMV(&trans, &m, &n, &kl, &ku, (MKL_Complex8 *)alpha,
	(MKL_Complex8 *)a, &lda, (MKL_Complex8 *)x, &incx,
	(MKL_Complex8 *)beta, (MKL_Complex8 *)y, &incy);
}

void
call_zgbmv(char trans, int m, int n, int kl, int ku, double *alpha, double *a,
	   int lda, double *x, int incx, double *beta, double *y, int incy)
{
  ZGBMV(&trans, &m, &n, &kl, &ku, (MKL_Complex16 *)alpha,
	(MKL_Complex16 *)a, &lda, (MKL_Complex16 *)x, &incx,
	(MKL_Complex16 *)beta, (MKL_Complex16 *)y, &incy);
}

/* dot product */

float
call_sdot(int n, float *x, int incx, float *y, int incy)
{
  return SDOT(&n, x, &incx, y, &incy);
}

double
call_ddot(int n, double *x, int incx, double *y, int incy)
{
  return DDOT(&n, x, &incx, y, &incy);
}
