/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: lapack_dumb.c,v 1.4 2007/10/31 06:23:11 kajiyama Exp $
 */

/* general matrix */

void
call_sgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  void sgetrf_(int *, int *, float *, int *, int *, int *);
  sgetrf_(&m, &n, a, &lda, ipiv, info);
}

void
call_dgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  void dgetrf_(int *, int *, double *, int *, int *, int *);
  dgetrf_(&m, &n, a, &lda, ipiv, info);
}

void
call_cgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  void cgetrf_(int *, int *, float *, int *, int *, int *);
  cgetrf_(&m, &n, a, &lda, ipiv, info);
}

void
call_zgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  void zgetrf_(int *, int *, double *, int *, int *, int *);
  zgetrf_(&m, &n, a, &lda, ipiv, info);
}

void
call_sgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  void sgetrs_(char *, int *, int *, float *, int *, int *,
	       float *, int *, int *);
  sgetrs_(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_dgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  void dgetrs_(char *, int *, int *, double *, int *, int *,
	       double *, int *, int *);
  dgetrs_(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_cgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  void cgetrs_(char *, int *, int *, float *, int *, int *,
	       float *, int *, int *);
  cgetrs_(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_zgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  void zgetrs_(char *, int *, int *, double *, int *, int *,
	       double *, int *, int *);
  zgetrs_(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_sgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, int *iwork, int *info)
{
  void sgecon_(char *, int *, float *, int *, float *,
	       float *, float *, int *, int *);
  sgecon_(&norm, &n, a, &lda, &anorm, rcond, work, iwork, info);
}

void
call_dgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, int *iwork, int *info)
{
  void dgecon_(char *, int *, double *, int *, double *,
	       double *, double *, int *, int *);
  dgecon_(&norm, &n, a, &lda, &anorm, rcond, work, iwork, info);
}

void
call_cgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, float *rwork, int *info)
{
  void cgecon_(char *, int *, float *, int *, float *,
	       float *, float *, float *, int *);
  cgecon_(&norm, &n, a, &lda, &anorm, rcond, work, rwork, info);
}

void
call_zgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, double *rwork, int *info)
{
  void zgecon_(char *, int *, double *, int *, double *,
	       double *, double *, double *, int *);
  zgecon_(&norm, &n, a, &lda, &anorm, rcond, work, rwork, info);
}

/* general band matrix */

void
call_sgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  void sgbtrf_(int *, int *, int *, int *, float *, int *, int *,
	       int *);
  sgbtrf_(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_dgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  void dgbtrf_(int *, int *, int *, int *, double *, int *, int *,
	       int *);
  dgbtrf_(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_cgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  void cgbtrf_(int *, int *, int *, int *, float *, int *, int *,
	       int *);
  cgbtrf_(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_zgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  void zgbtrf_(int *, int *, int *, int *, double *, int *, int *,
	       int *);
  zgbtrf_(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_sgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  void sgbtrs_(char *, int *, int *, int *, int *, float *,
	       int *, int *, float *, int *, int *);
  sgbtrs_(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_dgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  void dgbtrs_(char *, int *, int *, int *, int *, double *,
	       int *, int *, double *, int *, int *);
  dgbtrs_(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_cgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  void cgbtrs_(char *, int *, int *, int *, int *, float *,
	       int *, int *, float *, int *, int *);
  cgbtrs_(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_zgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  void zgbtrs_(char *, int *, int *, int *, int *, double *,
	       int *, int *, double *, int *, int *);
  zgbtrs_(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_sgbcon(char norm, int n, int kl, int ku, float *a, int lda,
	    int *ipiv, float anorm, float *rcond, float *work,
	    int *iwork, int *info)
{
  void sgbcon_(char *, int *, int *, int *, float *, int *, int *, float *,
	       float *, float *, int *, int *);
  sgbcon_(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, iwork,
	  info);
}

void
call_dgbcon(char norm, int n, int kl, int ku, double *a, int lda,
	    int *ipiv, double anorm, double *rcond, double *work,
	    int *iwork, int *info)
{
  void dgbcon_(char *, int *, int *, int *, double *, int *, int *, double *,
	       double *, double *, int *, int *);
  dgbcon_(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, iwork,
	  info);
}

void
call_cgbcon(char norm, int n, int kl, int ku, float *a, int lda,
	    int *ipiv, float anorm, float *rcond, float *work,
	    float *rwork, int *info)
{
  void cgbcon_(char *, int *, int *, int *, float *, int *, int *, float *,
	       float *, float *, float *, int *);
  cgbcon_(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, rwork,
	  info);
}

void
call_zgbcon(char norm, int n, int kl, int ku, double *a, int lda,
	    int *ipiv, double anorm, double *rcond, double *work,
	    double *rwork, int *info)
{
  void zgbcon_(char *, int *, int *, int *, double *, int *, int *, double *,
	       double *, double *, double *, int *);
  zgbcon_(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, rwork,
	  info);
}

/* singular value decomposition (SVD) */

void
call_sgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, int *info)
{
  void sgesvd_(char *, char *, int *, int *, float *, int *, float *,
	       float *, int *, float *, int *, float *, int *, int *);
  sgesvd_(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	  work, &lwork, info);
}

void
call_dgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, int *info)
{
  void dgesvd_(char *, char *, int *, int *, double *, int *, double *,
	       double *, int *, double *, int *, double *, int *, int *);
  dgesvd_(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	  work, &lwork, info);
}

void
call_cgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, float *rwork, int *info)
{
  void cgesvd_(char *, char *, int *, int *, float *, int *, float *,
	       float *, int *, float *, int *, float *, int *, float *,
	       int *);
  cgesvd_(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	  work, &lwork, rwork, info);
}

void
call_zgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, double *rwork, int *info)
{
  void zgesvd_(char *, char *, int *, int *, double *, int *, double *,
	       double *, int *, double *, int *, double *, int *, double *,
	       int *);
  zgesvd_(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	  work, &lwork, rwork, info);
}
