/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: lapack_intelmkl.c,v 1.4 2007/10/31 06:23:11 kajiyama Exp $
 */

#include <mkl_lapack.h>

int s_copy; /* XXX: undefined symbol that libmkl_lapack{32,64}.so require */

/* general matrix */

void
call_sgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  SGETRF(&m, &n, a, &lda, ipiv, info);
}

void
call_dgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  DGETRF(&m, &n, a, &lda, ipiv, info);
}

void
call_cgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  CGETRF(&m, &n, (MKL_Complex8 *)a, &lda, ipiv, info);
}

void
call_zgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  ZGETRF(&m, &n, (MKL_Complex16 *)a, &lda, ipiv, info);
}

void
call_sgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  SGETRS(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_dgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  DGETRS(&trans, &n, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_cgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  CGETRS(&trans, &n, &nrhs, (MKL_Complex8 *)a, &lda, ipiv,
	 (MKL_Complex8 *)b, &ldb, info);
}

void
call_zgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  ZGETRS(&trans, &n, &nrhs, (MKL_Complex16 *)a, &lda, ipiv,
	 (MKL_Complex16 *)b, &ldb, info);
}

void
call_sgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, int *iwork, int *info)
{
  SGECON(&norm, &n, a, &lda, &anorm, rcond, work, iwork, info);
}

void
call_dgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, int *iwork, int *info)
{
  DGECON(&norm, &n, a, &lda, &anorm, rcond, work, iwork, info);
}

void
call_cgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, float *rwork, int *info)
{
  CGECON(&norm, &n, (MKL_Complex8 *)a, &lda, &anorm, rcond,
	 (MKL_Complex8 *)work, rwork, info);
}

void
call_zgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, double *rwork, int *info)
{
  ZGECON(&norm, &n, (MKL_Complex16 *)a, &lda, &anorm, rcond,
	 (MKL_Complex16 *)work, rwork, info);
}

/* general band matrix */

void
call_sgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  SGBTRF(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_dgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  DGBTRF(&m, &n, &kl, &ku, a, &lda, ipiv, info);
}

void
call_cgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  CGBTRF(&m, &n, &kl, &ku, (MKL_Complex8 *)a, &lda, ipiv, info);
}

void
call_zgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  ZGBTRF(&m, &n, &kl, &ku, (MKL_Complex16 *)a, &lda, ipiv, info);
}

void
call_sgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  SGBTRS(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_dgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  DGBTRS(&trans, &n, &kl, &ku, &nrhs, a, &lda, ipiv, b, &ldb, info);
}

void
call_cgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  CGBTRS(&trans, &n, &kl, &ku, &nrhs, (MKL_Complex8 *)a, &lda, ipiv,
	 (MKL_Complex8 *)b, &ldb, info);
}

void
call_zgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  ZGBTRS(&trans, &n, &kl, &ku, &nrhs, (MKL_Complex16 *)a, &lda, ipiv,
	 (MKL_Complex16 *)b, &ldb, info);
}

void
call_sgbcon(char norm, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    float anorm, float *rcond, float *work, int *iwork, int *info)
{
  SGBCON(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, iwork, info);
}

void
call_dgbcon(char norm, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    double anorm, double *rcond, double *work, int *iwork, int *info)
{
  DGBCON(&norm, &n, &kl, &ku, a, &lda, ipiv, &anorm, rcond, work, iwork, info);
}

void
call_cgbcon(char norm, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    float anorm, float *rcond, float *work, float *rwork, int *info)
{
  CGBCON(&norm, &n, &kl, &ku, (MKL_Complex8 *)a, &lda, ipiv, &anorm, rcond,
	 (MKL_Complex8 *)work, rwork, info);
}

void
call_zgbcon(char norm, int n, int kl, int ku, double *a, int lda, int *ipiv,
	double anorm, double *rcond, double *work, double *rwork, int *info)
{
  ZGBCON(&norm, &n, &kl, &ku, (MKL_Complex16 *)a, &lda, ipiv, &anorm, rcond,
	 (MKL_Complex16 *)work, rwork, info);
}

/* singular value decomposition (SVD) */

void
call_sgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, int *info)
{
  SGESVD(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	 work, &lwork, info);
}

void
call_dgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, int *info)
{
  DGESVD(&jobu, &jobvt, &m, &n, a, &lda, s, u, &ldu, vt, &ldvt,
	 work, &lwork, info);
}

void
call_cgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, float *rwork, int *info)
{
  CGESVD(&jobu, &jobvt, &m, &n, (MKL_Complex8 *)a, &lda, s,
	 (MKL_Complex8 *)u, &ldu, (MKL_Complex8 *)vt, &ldvt,
	 (MKL_Complex8 *)work, &lwork, rwork, info);
}

void
call_zgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, double *rwork, int *info)
{
  ZGESVD(&jobu, &jobvt, &m, &n, (MKL_Complex16 *)a, &lda, s,
	 (MKL_Complex16 *)u, &ldu, (MKL_Complex16 *)vt, &ldvt,
	 (MKL_Complex16 *)work, &lwork, rwork, info);
}
