/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: lapack_sunperf.c,v 1.4 2007/10/31 06:23:11 kajiyama Exp $
 */

#include <sunperf.h>

/* general matrix */

void
call_sgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  sgetrf(m, n, a, lda, ipiv, info);
}

void
call_dgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  dgetrf(m, n, a, lda, ipiv, info);
}

void
call_cgetrf(int m, int n, float *a, int lda, int *ipiv, int *info)
{
  cgetrf(m, n, (complex *)a, lda, ipiv, info);
}

void
call_zgetrf(int m, int n, double *a, int lda, int *ipiv, int *info)
{
  zgetrf(m, n, (doublecomplex *)a, lda, ipiv, info);
}

void
call_sgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  sgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info);
}

void
call_dgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  dgetrs(trans, n, nrhs, a, lda, ipiv, b, ldb, info);
}

void
call_cgetrs(char trans, int n, int nrhs, float *a, int lda, int *ipiv,
	    float *b, int ldb, int *info)
{
  cgetrs(trans, n, nrhs, (complex *)a, lda, ipiv, (complex *)b, ldb, info);
}

void
call_zgetrs(char trans, int n, int nrhs, double *a, int lda, int *ipiv,
	    double *b, int ldb, int *info)
{
  zgetrs(trans, n, nrhs, (doublecomplex *)a, lda, ipiv,
	 (doublecomplex *)b, ldb, info);
}

void
call_sgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, int *iwork, int *info)
{
  sgecon(norm, n, a, lda, anorm, rcond, info);
}

void
call_dgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, int *iwork, int *info)
{
  dgecon(norm, n, a, lda, anorm, rcond, info);
}

void
call_cgecon(char norm, int n, float *a, int lda, float anorm,
	    float *rcond, float *work, float *rwork, int *info)
{
  cgecon(norm, n, (complex *)a, lda, anorm, rcond, info);
}

void
call_zgecon(char norm, int n, double *a, int lda, double anorm,
	    double *rcond, double *work, double *rwork, int *info)
{
  zgecon(norm, n, (doublecomplex *)a, lda, anorm, rcond, info);
}

/* general band matrix */

void
call_sgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  sgbtrf(m, n, kl, ku, a, lda, ipiv, info);
}

void
call_dgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  dgbtrf(m, n, kl, ku, a, lda, ipiv, info);
}

void
call_cgbtrf(int m, int n, int kl, int ku, float *a, int lda, int *ipiv,
	    int *info)
{
  cgbtrf(m, n, kl, ku, (complex *)a, lda, ipiv, info);
}

void
call_zgbtrf(int m, int n, int kl, int ku, double *a, int lda, int *ipiv,
	    int *info)
{
  zgbtrf(m, n, kl, ku, (doublecomplex *)a, lda, ipiv, info);
}

void
call_sgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  sgbtrs(trans, n, kl, ku, nrhs, a, lda, ipiv, b, ldb, info);
}

void
call_dgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  dgbtrs(trans, n, kl, ku, nrhs, a, lda, ipiv, b, ldb, info);
}

void
call_cgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
	    int lda, int *ipiv, float *b, int ldb, int *info)
{
  cgbtrs(trans, n, kl, ku, nrhs, (complex *)a, lda, ipiv,
	 (complex *)b, ldb, info);
}

void
call_zgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
	    int lda, int *ipiv, double *b, int ldb, int *info)
{
  zgbtrs(trans, n, kl, ku, nrhs, (doublecomplex *)a, lda, ipiv,
	 (doublecomplex *)b, ldb, info);
}

void
call_sgbcon(char norm, int n, int kl, int ku, float *a, int lda,
	    int *ipiv, float anorm, float *rcond, float *work,
	    int *iwork, int *info)
{
  sgbcon(norm, n, kl, ku, a, lda, ipiv, anorm, rcond, info);
}

void
call_dgbcon(char norm, int n, int kl, int ku, double *a, int lda,
	    int *ipiv, double anorm, double *rcond, double *work,
	    int *iwork, int *info)
{
  dgbcon(norm, n, kl, ku, a, lda, ipiv, anorm, rcond, info);
}

void
call_cgbcon(char norm, int n, int kl, int ku, float *a, int lda,
	    int *ipiv, float anorm, float *rcond, float *work,
	    float *rwork, int *info)
{
  cgbcon(norm, n, kl, ku, (complex *)a, lda, ipiv, anorm, rcond, info);
}

void
call_zgbcon(char norm, int n, int kl, int ku, double *a, int lda,
	    int *ipiv, double anorm, double *rcond, double *work,
	    double *rwork, int *info)
{
  zgbcon(norm, n, kl, ku, (doublecomplex *)a, lda, ipiv, anorm, rcond, info);
}

/* singular value decomposition (SVD) */

void
call_sgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, int *info)
{
  sgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, info);
}

void
call_dgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, int *info)
{
  dgesvd(jobu, jobvt, m, n, a, lda, s, u, ldu, vt, ldvt, info);
}

void
call_cgesvd(char jobu, char jobvt, int m, int n, float *a, int lda,
	    float *s, float *u, int ldu, float *vt, int ldvt,
	    float *work, int lwork, float *rwork, int *info)
{
  cgesvd(jobu, jobvt, m, n, (complex *)a, lda, s, (complex *)u,
	 ldu, (complex *)vt, ldvt, info);
}

void
call_zgesvd(char jobu, char jobvt, int m, int n, double *a, int lda,
	    double *s, double *u, int ldu, double *vt, int ldvt,
	    double *work, int lwork, double *rwork, int *info)
{
  zgesvd(jobu, jobvt, m, n, (doublecomplex *)a, lda, s, (doublecomplex *)u,
	 ldu, (doublecomplex *)vt, ldvt, info);
}
