/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_cg.c,v 1.4 2007/10/31 06:23:13 kajiyama Exp $
 */

#include <stdio.h>
#include <math.h>
#include "ssi_leqlib.h"

/* epsilon = 1.0e-10 */

int ssi_cg(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[],
		   SSI_SCALAR params[], int options[],
		   SSI_MATRIX *A, SSI_PRECON *M)
{
	SSI_SCALAR *r,*z,*p,*q;
	SSI_SCALAR alpha, beta, rho, rho_old;
	SSI_SCALAR bnrm2, nrm2, epsilon, dot_pq;
	int iter,maxiter,n,output;

#if 0
	int i, j;
	printf("ssi_cg()\n");
	for (i = 0; i < A->n; i++)
	  for (j = A->row[i]; j < A->row[i+1]; j++)
	    printf("A[%d,%d] = %8.2f\n", i, A->index[j], A->value[j]);
	for (i = 0; i < A->n; i++)
	  printf("b[%d] = %8.2f\n", i, b[i]);
	printf("maxiter = %d\n", options[SSI_OPTIONS_MAXITER]);
	printf("epsilon = %e\n", params[SSI_PARAMS_RESID]);
#endif
	n = A->n;
	maxiter = options[SSI_OPTIONS_MAXITER];
	output  = options[SSI_OPTIONS_OUTPUT];
	epsilon = params[SSI_PARAMS_RESID];
	r = work;
	z = &work[n];
	p = &work[n*2];
	q = &work[n*3];
	rho_old = (SSI_SCALAR)1.0;
	beta = (SSI_SCALAR)0.0;

//#pragma omp parallel shared(A,M,b,x,r,z,p,q,nrm2,rho,rho_old,alpha,beta,bnrm2,iter,maxiter,epsilon) private(n)
	// Initial Residual
	ssi_copy(n,b,r);
	if( ssi_dot(n,x,x) != 0.0 )
//	if( ssi_nrm2(n,x) != 0.0 )
	{
		ssi_matvec(-1,A,x,1,r); // r = r - Ax
		if( ssi_nrm2(n,r) <= epsilon ) return SSIRET_SUCCESS;
	}
	bnrm2 = sqrt(ssi_dot(n,b,b));
//	bnrm2 = ssi_nrm2(n,b);
	if( bnrm2 == 0.0 )
	{
		bnrm2 = 1.0;
	}
	else
	{
		bnrm2 = 1.0 / bnrm2;
	}

//	ssi_copy(n,z,p);
	ssi_setv(n,0.0,p);
	
	
	for( iter=1; iter<=maxiter; iter++ )
	{
		/* z = M^-1 * r */
//		ssi_psolve(M,r,z);
		ssi_copy(n,r,z);

		/* rho = <r,z> */
		rho = ssi_dot(n,r,z);

		/* beta = rho / rho_old */
		beta = rho / rho_old;

		/* p = z + beta*p       */
		ssi_xpay(n,z,beta,p);
//		ssi_axpy(n,beta,p,z);
//		ssi_copy(n,z,p);
		
		/* q = Ap */
		ssi_matvec(1,A,p,0,q);
		
		/* dot_pq = <p,q> */
		dot_pq = ssi_dot(n,p,q);
		/* breakdown check */
		/* */
		
		/* alpha = rho / dot_pq */
		alpha = rho / dot_pq;
		
		/* x = x + alpha*p */
		ssi_axpy(n,alpha,p,x);
		
		/* r = r - alpha*q */
		ssi_axpy(n,-alpha,q,r);

		/* convergence check */
		/* */
		nrm2 = sqrt(ssi_dot(n,r,r)) * bnrm2;
//		nrm2 = ssi_nrm2(n,r) / bnrm2;

		if( output )
		{
			printf("iter: %5d  residual = %e\n", iter, nrm2);
		}
		
		if( epsilon >= nrm2 )
		{
			options[SSI_OPTIONS_MAXITER] = iter;
			return SSIRET_SUCCESS;
		}
		
		rho_old = rho;
//		ssi_psolve(M,r,z);
//		ssi_copy(n,z,p);
		
		
	}
	return SSIRET_MAXITER;
}
