/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_copy.c,v 1.4 2007/10/31 06:23:13 kajiyama Exp $
 */

#include "ssi_leqlib.h"

int ssi_copy(int n, SSI_SCALAR *vx, SSI_SCALAR *vy)
{
	int i;

//	if( 0>=n ) return SSIRET_DIMENSION_LE_ZERO;
#pragma omp parallel for private(i)
//#pragma omp for private(i)
	for(i=0; i<n; i++)
	{
		vy[i] = vx[i];
	}
	return SSIRET_SUCCESS;
//	return omp_get_max_threads();
}

int ssi_copy_(int n, SSI_SCALAR *dx, int incx, SSI_SCALAR *dy, int incy)
{
    int i, m, ix, iy, mp1;

	/*     copies a vector, x, to a vector, y. */
	/*     uses unrolled loops for increments equal to one. */
	/*     jack dongarra, linpack, 3/11/78. */

	/* Function Body */
    if (n <= 0) 
	{
		return 0;
    }
    if( incx == 1 && incy == 1 )
	{
		/* code for both increments equal to 1 */
	    m = n % 7;
	    if (m != 0) 
		{
			for (i = 0; i < m; ++i) 
			{
				dy[i] = dx[i];
		    }
		    if (n < 7) return 0;
	    }
	    mp1 = m ;
	    for (i = mp1; i < n; i += 7) 
		{
			dy[i] = dx[i];
			dy[i + 1] = dx[i + 1];
			dy[i + 2] = dx[i + 2];
			dy[i + 3] = dx[i + 3];
			dy[i + 4] = dx[i + 4];
			dy[i + 5] = dx[i + 5];
			dy[i + 6] = dx[i + 6];
	    }
	}
	else
	{
/*        code for unequal increments or equal increments */
/*          not equal to 1 */

	    ix = 0;
	    iy = 0;
	    if (incx < 0)
		{
			ix = (-n + 1) * incx + 1;
	    }
	    if (incy < 0)
		{
			iy = (-n + 1) * incy + 1;
	    }
	    for (i = 0; i < n; ++i)
		{
			dy[iy] = dx[ix];
			ix += incx;
			iy += incy;
	    }
	}
	return SSIRET_SUCCESS;
}
