/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_dot.c,v 1.4 2007/10/31 06:23:13 kajiyama Exp $
 */

#include "ssi_leqlib.h"

#ifdef _OPENMP
#include <omp.h>
#endif

SSI_SCALAR ssi_dot(int n, SSI_SCALAR *vx, SSI_SCALAR *vy)
{
	int i;
	SSI_SCALAR dot = (SSI_SCALAR)0;
	SSI_SCALAR dots[128];

//	if( 0>=n ) return dot;
#if defined(_sgi) && defined(_OPENMP)
#pragma omp parallel private(dot)
	{
	  dot = 0;
#pragma omp for
	  for(i=0; i<n; i++)
	    {
	      dot += vx[i]*vy[i];
	    }
	  dots[omp_get_thread_num()] = dot;
	}
	dot = 0;
	for (i = 0; i < omp_get_max_threads(); i++)
	  dot += dots[i];
#else
#pragma omp parallel for private(i) reduction(+:dot)
	for(i=0; i<n; i++)
	{
		dot += vx[i]*vy[i];
	}
#endif
	return dot;
}

/*
SSI_SCALAR ssi_dot(int n, SSI_SCALAR *vx, SSI_SCALAR *vy)
{
	int i;
	SSI_SCALAR dot1 = (SSI_SCALAR)0;
	SSI_SCALAR dot = (SSI_SCALAR)0;

	if( 0>=n ) return dot;
//#pragma omp parallel for private(i) reduction(+:dot)
#pragma omp for private(i,dot1)
	for(i=0; i<n; i++)
	{
		dot1 += vx[i]*vy[i];
	}
#pragma omp critical
{
	dot += dot1;
}

	return dot;
}
*/
