/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_gs.c,v 1.4 2007/10/31 06:23:13 kajiyama Exp $
 */

#include <math.h>
#include "ssi_leqlib.h"

int ssi_gs(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[],
		   SSI_SCALAR params[], int options[],
		   SSI_MATRIX *A, SSI_PRECON *M)
{
	SSI_SCALAR ux,lx,dd;
	SSI_SCALAR xx,bb,xb;
	SSI_SCALAR bnrm2, nrm2;
	SSI_SCALAR tol, rhotol;
	int iter,maxiter,n,output;
	int i,j;
	int base;

	n = A->n;
	base = A->array_base;
	maxiter = options[SSI_OPTIONS_MAXITER];
	output  = options[SSI_OPTIONS_OUTPUT];
	tol = params[SSI_PARAMS_RESID];

//	rhotol = getbreak();
	rhotol = 1.0e-18;

	bb = 0.0;
	for(i=0;i<n;i++)
	{
		bb += b[i]*b[i];
		work[i] = b[i];
	}
	bb = sqrt(bb);

	for( iter=1; iter<=maxiter; iter++ )
	{
		xx = 0.0;
//		bb = 0.0;
		for(i=0;i<n;i++)
		{
			lx = b[i];
			ux = 0.0;
			for(j=0;j<i;j++)
			{
				lx -= A->value[j*n+i] * x[j];
			}
			for(j=i+1;j<n;j++)
			{
				ux -= A->value[j*n+i] * x[j];
			}
			dd = A->value[i*n+i];

//				xb = x[i];
//				x[i] = (lx + ux) / dd;
//				xb -= x[i];
//				xx += xb*xb;
//				bb += x[i]*x[i];
			work[i] += ux - dd*x[i];
			xx      += work[i] * work[i];
			x[i]     = (lx + ux) / dd;
			work[i]  = lx;
		}
		/* convergence check */
		
//		bb = sqrt(bb);
		/* test zero */
//		if( bb < rhotol )
//		{
//			return;
//		}
		xx = sqrt(xx);
		nrm2 = xx/bb;

		if( output )
		{
			printf("iter: %5d  residual = %e\n", iter, nrm2);
		}

		if( tol >= nrm2 )
		{
			options[SSI_OPTIONS_MAXITER] = iter;
			return SSIRET_SUCCESS;
		}
	}

	return SSIRET_MAXITER;
}
