/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_leqlib.h,v 1.4 2007/10/31 06:23:13 kajiyama Exp $
 */

#ifndef __SSI_LEQLIB_H__
#define __SSI_LEQLIB_H__
#include <stdio.h>
//#include <mpi.h>
#ifdef WIN32
//#include "blaswrap.h"
#endif
//#include "f2c.h"
//#include "clapack.h"

#define _max(a,b) ((a) >= (b) ? (a) : (b))
#define _min(a,b) ((a) <= (b) ? (a) : (b))
typedef double					SSI_SCALAR;
typedef FILE					*SSI_INPUT_FILE;
typedef FILE					*SSI_OUTPUT_FILE;

/*        */
/* Matrix */
/*        */
typedef struct
{
	int			matrix_type;
	int			array_base;
	int			matrix_base;
	int			np;
	int			n;
	int			nnz;
	int			ndz;
	int			bn;
	int			bnr;
	int			bnc;
	int			bnnz;
	int			nnd;
	int			maxnzr;
	SSI_SCALAR	*value;
	int			*ptr;
	int			*row;
	int			*col;
	int			*index;
	int			*block_index;
	int			*block_ptr;
} SSI_MAT;

typedef struct
{
	int			issplit;
	SSI_MAT		A;
	SSI_MAT		L;
	SSI_MAT		U;
	SSI_SCALAR	*D;
} SSI_MATRIX3;

struct SSI_MATRIX_STRUCT
{
	int			matrix_type;
	int			array_base;
	int			matrix_base;
	int			np;
	int			n;
	int			nnz;
	int			ndz;
	int			bn;
	int			bnr;
	int			bnc;
	int			bnnz;
	int			nnd;
	int			maxnzr;
	SSI_SCALAR	*data;
	SSI_SCALAR	*value;
	int			*ptr;
	int			*row;
	int			*col;
	int			*index;
	int			*block_index;
	int			*block_ptr;
};
typedef struct SSI_MATRIX_STRUCT SSI_MATRIX;

typedef struct
{
	int			matrix_type;
	int			array_base;
	int			matrix_base;
	int			n;
	int			bn;
	int			nnz;
	int			ndz;
	int			bnr;
	int			bnc;
	int			bnnz;
	int			nnd;
	int			maxnzr;
} SSI_MATRIX_PARAMS;

/*                */
/* Preconditioner */
/*                */
struct SSI_PRECON_STRUCT
{
	int			precon_type;
	SSI_MATRIX	*Pmat;
	SSI_MATRIX	*Amat;
	SSI_SCALAR	*Pb;
};
typedef struct SSI_PRECON_STRUCT SSI_PRECON;

typedef struct
{
	int			myrank;
	int			comm;
	int			neibpetot;
	int			*neibpe;
	int			*index_export;
	int			*node_export;
	int			*index_import;
	int			*node_import;
  //	MPI_Request *req1,*req2;
  //	MPI_Status	*sta1,*sta2;
} SSI_COMMTABLE;

typedef struct
{
	char		filename[1024];
	int			fileformat;
	int			matrix_type_in;
	int			matrix_type_out;
	int			is_matrix_split;
	int			is_array_zero_cut;
	int			is_solution;
	int			is_rhs;
	int			array_base_ptr;
	int			array_base_index;
	int			precision;
	int			param_len;
	int			array_len;
	int			param_order[9];
	int			array_order[9];
} SSI_MATRIX_INOUT;

typedef struct
{
	int row;
	int col;
	int ptr;
} SSI_MATRIX_SORT_DATA;
typedef struct
{
	int nnzr;
	int row;
} SSI_MATRIX_SORT_DATA_PERM;

#define SSI_MATRIX_TYPE_DEFAULT		0
#define SSI_MATRIX_TYPE_DENSE		0
#define SSI_MATRIX_TYPE_DNS			0
#define SSI_MATRIX_TYPE_COO			1
#define SSI_MATRIX_TYPE_CRS			2
#define SSI_MATRIX_TYPE_CSR			2
#define SSI_MATRIX_TYPE_CCS			3
#define SSI_MATRIX_TYPE_CSC			3
#define SSI_MATRIX_TYPE_MSR			4
#define SSI_MATRIX_TYPE_DIA			5
#define SSI_MATRIX_TYPE_CDS			5
#define SSI_MATRIX_TYPE_ELL			6
#define SSI_MATRIX_TYPE_JDS			7
#define SSI_MATRIX_TYPE_JAD			7
#define SSI_MATRIX_TYPE_BSR			8
#define SSI_MATRIX_TYPE_BSC			9
#define SSI_MATRIX_TYPE_VBR			10

#define SSI_PRECON_TYPE_NONE		0
#define SSI_PRECON_TYPE_JACOBI		1
#define SSI_PRECON_TYPE_ILU			2
#define SSI_PRECON_TYPE_SSOR		3
#define SSI_PRECON_TYPE_IS			4
#define SSI_PRECON_TYPE_ITER		5

#define SSI_MATRIX_ARRAY_BASE_0		0
#define SSI_MATRIX_ARRAY_BASE_1		1

#define SSI_MATRIX_SORT_ROWCOL		0
#define SSI_MATRIX_SORT_COLROW		1

#define SSI_MATRIX_INPUT_MATRIXMARKET	1
#define SSI_MATRIX_INPUT_MM				1
#define SSI_MATRIX_INPUT_HARWELLBOEING	2
#define SSI_MATRIX_INPUT_HB				2
//#define SSI_MATRIX_INPUT_CRSLU			3
#define SSI_MATRIX_INPUT_SSI			3
#define SSI_MATRIX_INPUT_FREE			4
#define SSI_MATRIX_OUTPUT_SSI			0

#define SSI_MATRIX_INOUT_N				0
#define SSI_MATRIX_INOUT_NNZ			1
#define SSI_MATRIX_INOUT_NDZ			2
#define SSI_MATRIX_INOUT_NND			3
#define SSI_MATRIX_INOUT_MAXNZR			4
#define SSI_MATRIX_INOUT_BN				5
#define SSI_MATRIX_INOUT_BNNZ			6
#define SSI_MATRIX_INOUT_BNR			7
#define SSI_MATRIX_INOUT_BNC			8
#define SSI_MATRIX_INOUT_NNZL			9
#define SSI_MATRIX_INOUT_NNZU			10
#define SSI_MATRIX_INOUT_ROW			0
#define SSI_MATRIX_INOUT_COL			1
#define SSI_MATRIX_INOUT_PTR			2
#define SSI_MATRIX_INOUT_INDEX			3
#define SSI_MATRIX_INOUT_BLOCK_PTR		4
#define SSI_MATRIX_INOUT_BLOCK_INDEX	5
#define SSI_MATRIX_INOUT_VALUE			6
#define SSI_MATRIX_INOUT_SOLUTION		7
#define SSI_MATRIX_INOUT_RHS			8
#define SSI_MATRIX_INOUT_DIAG			9
#define SSI_MATRIX_INOUT_LROW			10
#define SSI_MATRIX_INOUT_LCOL			11
#define SSI_MATRIX_INOUT_LINDEX			12
#define SSI_MATRIX_INOUT_LVALUE			13
#define SSI_MATRIX_INOUT_UROW			14
#define SSI_MATRIX_INOUT_UCOL			15
#define SSI_MATRIX_INOUT_UINDEX			16
#define SSI_MATRIX_INOUT_UVALUE			17


#define SSI_PARAMS_LEN				2
#define SSI_PARAMS_RESID			0
#define SSI_PARAMS_W				1

#define SSI_OPTIONS_LEN				6
#define SSI_OPTIONS_MAXITER			0
#define SSI_OPTIONS_RESTART			1
#define SSI_OPTIONS_ELL				1
#define SSI_OPTIONS_OUTPUT			2
#define SSI_OPTIONS_PRECON			3
#define SSI_OPTIONS_SCALE			4
#define SSI_OPTIONS_SOLVER			5

#define SSI_OPTIONS_OUTPUT_FORMAT	0

#define SSI_MATRIX_GDDM_YES			0
#define SSI_MATRIX_GDDM_NO			1
#define SSI_MATRIX_GDDM_UNKNOWN		2

#define SSI_SOLVER_CG				0
#define SSI_SOLVER_BICG				1
#define SSI_SOLVER_CGS				2
#define SSI_SOLVER_BICGSTAB			3
#define SSI_SOLVER_BICGSTABL		4
#define SSI_SOLVER_GPBICG			5
#define SSI_SOLVER_ORTHOMIN			6
#define SSI_SOLVER_GMRES			7
#define SSI_SOLVER_QMR				8
#define SSI_SOLVER_JACOBI			9
#define SSI_SOLVER_GS				10
#define SSI_SOLVER_SOR				11

#ifdef __cplusplus
extern "C"
{
#endif

	/****************************/
	/* BLAS LAPACK              */
	/****************************/
	extern int ssi_asum(int n, SSI_SCALAR *vx, SSI_SCALAR *asum);
	extern int ssi_axpy(int n, SSI_SCALAR alpha, SSI_SCALAR *vx, SSI_SCALAR *vy);
	extern int ssi_xpay(int n, SSI_SCALAR *vx, SSI_SCALAR alpha, SSI_SCALAR *vy);
	extern int ssi_axpyz(int n, SSI_SCALAR alpha, SSI_SCALAR *vx, SSI_SCALAR *vy, SSI_SCALAR *vz);
	extern int ssi_copy(int n, SSI_SCALAR *vx, SSI_SCALAR *vy);
	extern SSI_SCALAR ssi_dot(int n, SSI_SCALAR *vx, SSI_SCALAR *vy);
	extern SSI_SCALAR ssi_nrm2(int n, SSI_SCALAR *vx);
	extern int ssi_setv(int n, SSI_SCALAR alpha, SSI_SCALAR *vx);
	extern int ssi_scal(int n, SSI_SCALAR alpha, SSI_SCALAR *vx);
	extern int ssi_swap(int n, SSI_SCALAR *vx, SSI_SCALAR *vy);
	extern void ssi_rot(int n, SSI_SCALAR dx[], int incx, SSI_SCALAR dy[], int incy, SSI_SCALAR c, SSI_SCALAR s);
	extern void ssi_rotg(SSI_SCALAR *da, SSI_SCALAR *db, SSI_SCALAR *c, SSI_SCALAR *s);

	extern int dcopy_(int *n, SSI_SCALAR *dx, int *incx, SSI_SCALAR *dy, int *incy);
	extern int ssi_copy_(int n, SSI_SCALAR *dx, int incx, SSI_SCALAR *dy, int incy);
	extern SSI_SCALAR sign(SSI_SCALAR a, SSI_SCALAR b);
	extern void ssi_gemv(char trans, int m, int n, SSI_SCALAR alpha,
			   SSI_SCALAR a[], int lda, SSI_SCALAR x[], int incx,
			   SSI_SCALAR beta, SSI_SCALAR y[], int incy);
	extern void ssi_trsv(char uplo, char trans, char diag, int n, 
			   SSI_SCALAR a[], int lda, SSI_SCALAR x[], int incx);

	/****************************/
	/* Vector Create            */
	/****************************/
	extern void ssi_vector_print(int n, SSI_SCALAR x[], char *str);
	extern void ssi_vectori_print(int n, int x[], char *str);
	extern int ssi_vector_create(void **vec, int size, int iszero);
//	extern int ssi_vector_create(int n, SSI_SCALAR *vec, int iszero);
//	extern int ssi_vector_create(int n, void **vec, int size, int iszero);
	extern void ssi_vector_destroy(void *vec);
	/****************************/
	/* Matrix Create            */
	/****************************/
//	extern int ssi_matrix_create(SSI_MATRIX *Amat);
//	extern int ssi_matrix_destroy(SSI_MATRIX Amat);
	extern int ssi_matrix_create(SSI_MATRIX_PARAMS params, SSI_MATRIX **Amat);
//		extern int ssi_matrix_create_dense(int row, int col, SSI_MATRIX **Amat);
	extern int ssi_matrix_destroy(SSI_MATRIX *Amat);
	/****************************/
	/* Matrix Print             */
	/****************************/
	extern int ssi_matrix_print(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_dense(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_crs(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_ccs(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_msr(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_coo(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_dia(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_ell(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_jds(SSI_MATRIX *Amat);
		extern int ssi_matrix_print_vbr(SSI_MATRIX *Amat);
	/****************************/
	/* Matrix Convert           */
	/****************************/
	extern int ssi_matrix_convert(SSI_MATRIX Ain, SSI_MATRIX *Aout, int convert_matrix_type);
		/***************/
		/* COO ==> dst */
		/***************/
		extern int ssi_matrix_convert_coo2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_coo2ccs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_coo2msr(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		/***************/
		/* CRS ==> dst */
		/***************/
		extern int ssi_matrix_convert_crs2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2dns(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2coo(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2ccs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2msr(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2dia(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2ell(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2jds(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2vbr(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2bsr(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_crs2bsc(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		/***************/
		/* src ==> CRS */
		/***************/
		extern int ssi_matrix_convert_dns2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_ccs2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_msr2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_dia2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_ell2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_jds2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_vbr2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_bsr2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
		extern int ssi_matrix_convert_bsc2crs(SSI_MATRIX Ain, SSI_MATRIX *Aout);
	/****************************/
	/* Matrix Input             */
	/****************************/
	extern int ssi_matrix_input(SSI_MATRIX *A, SSI_SCALAR *b, SSI_SCALAR *x, SSI_MATRIX_INOUT minout, int matrix_type);
		extern int ssi_matrix_input_mm(SSI_MATRIX *A, int matrix_type, int matrix_array_base, SSI_INPUT_FILE file);
//		extern int ssi_matrix_input_crslu(SSI_MATRIX **A, SSI_SCALAR *b[], SSI_SCALAR *x[], int matrix_type, int matrix_array_base, SSI_INPUT_FILE file);
		extern int ssi_matrix_input_ssi(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT minout, int matrix_type, SSI_INPUT_FILE file);
			extern int ssi_matrix_input_ssi_crs(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT minout,SSI_INPUT_FILE file);
			extern int ssi_matrix_input_ssi_crs2(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT minout,SSI_INPUT_FILE file,int nnzl, int nnzu);
	/****************************/
	/* Matrix Output            */
	/****************************/
	extern int ssi_matrix_inout_init(SSI_MATRIX_INOUT *minout);
	extern int ssi_matrix_output(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT inout, int options[], char *path);
		extern int ssi_matrix_output_ssi(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT inout, int options[], SSI_OUTPUT_FILE file);
			extern int ssi_matrix_output_ssi_crs(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_MATRIX_INOUT inout, int options[], SSI_OUTPUT_FILE file);
	/****************************/
	/* Matrix Vector Product    */
	/****************************/
	extern void ssi_matvec(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);	//ȥ٥ȥѡy=alpha*A*x+beta*y
		extern void ssi_matvec_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvec_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
	extern void ssi_matvect(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);	//ȥ٥ȥѡy=alpha*A*x+beta*y
		extern void ssi_matvect_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matvect_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
	extern void ssi_matveca(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);	//ȥ٥ȥѡy=alpha*A*x+beta*y
		extern void ssi_matveca_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
		extern void ssi_matveca_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[]);
	extern void ssi_matdia(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_dns(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_crs(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_ccs(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_msr(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_dia(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_ell(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_jds(SSI_MATRIX *A, SSI_SCALAR x[]);
		extern void ssi_matdia_vbr(SSI_MATRIX *A, SSI_SCALAR x[]);
	/****************************/
	/* Get Diagonal             */
	/****************************/
	extern void ssi_get_diagonal(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_dns(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_crs(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_ccs(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_msr(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_dia(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_ell(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_jds(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_get_diagonal_vbr(SSI_MATRIX *A, SSI_SCALAR d[]);
	/****************************/
	/* Scaling diagonal         */
	/****************************/
	extern int ssi_scaling_diagonal(SSI_MATRIX *A, SSI_SCALAR b[]);
		extern void ssi_scaling_diagonal_dns(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_crs(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_ccs(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_msr(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_dia(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_ell(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_jds(SSI_MATRIX *A, SSI_SCALAR d[]);
		extern void ssi_scaling_diagonal_vbr(SSI_MATRIX *A, SSI_SCALAR d[]);
	/****************************/
	/* Matrix check             */
	/****************************/
		extern int ssi_matrix_check_zerodiag(SSI_MATRIX *Amat);
		extern int ssi_matrix_sort(SSI_MATRIX *Amat, SSI_MATRIX_SORT_DATA **data, int sort_order);
	extern int ssi_matrix_check_gddm(SSI_MATRIX *A, SSI_SCALAR d[], int options[], int *gddm);
	/****************************/
	/* preconditioner           */
	/****************************/
	extern void ssi_psolve(SSI_PRECON *Precon, SSI_SCALAR b[], SSI_SCALAR x[]);
	extern void ssi_psolvet(SSI_PRECON *Precon, SSI_SCALAR b[], SSI_SCALAR x[]);
	extern void ssi_psolveq(SSI_PRECON *Precon, SSI_SCALAR b[], SSI_SCALAR x[], char t);
	extern void ssi_psolveqt(SSI_PRECON *Precon, SSI_SCALAR b[], SSI_SCALAR x[], char t);
	extern int ssi_precon_jacobi(SSI_PRECON *precon, SSI_MATRIX *A);
	extern int ssi_precon_create(SSI_MATRIX *Amat, SSI_SCALAR b[], int precon_type, SSI_PRECON **precon);
	extern void ssi_precon_destroy(SSI_PRECON *precon);
		/****************************/
		/* preconditioner ILU       */
		/****************************/
		extern int ssi_precon_ilu(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_dns(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_crs(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_ccs(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_msr(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_dia(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_ell(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_jds(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_ilu_vbr(SSI_PRECON *precon, SSI_MATRIX *A);
		/****************************/
		/* preconditioner I+S       */
		/****************************/
		extern int ssi_precon_is(SSI_PRECON *precon, SSI_MATRIX *A, SSI_SCALAR b[]);
			extern int ssi_precon_is_dns(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_crs(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_ccs(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_msr(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_dia(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_ell(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_jds(SSI_PRECON *precon, SSI_MATRIX *A);
			extern int ssi_precon_is_vbr(SSI_PRECON *precon, SSI_MATRIX *A);
				extern int ssi_precon_is_crs_make(SSI_PRECON *precon, SSI_MATRIX *A, SSI_SCALAR b[], int nnz);
	/****************************/
	/* Solvers                  */
	/****************************/
	extern int ssi_solver(SSI_MATRIX *A, SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR params[], int options[]);
		extern int ssi_cg(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_bicgstab(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_bicg(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_gmres(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_cgs(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_gpbicg(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_bicgstabl(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_orthomin(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_qmr(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_jacobi(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_gs(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
		extern int ssi_sor(SSI_SCALAR b[], SSI_SCALAR x[], SSI_SCALAR work[], SSI_SCALAR params[], int options[], SSI_MATRIX *A, SSI_PRECON *M);
	/****************************/
	/* Utilities                */
	/****************************/
	extern int ssi_send_recv(int n, SSI_COMMTABLE *commtable, SSI_SCALAR ws[], SSI_SCALAR wr[], SSI_SCALAR x[]);
	extern double ssi_get_flops(int options[], int iter, double times, int n, int nnz);
	extern double ssi_get_iter_flops(double axpy_flops, double inner_flops, double matvec_flops, int iter, int n, int nnz, int solvers, int restart);
	extern SSI_SCALAR getbreak(void);
	extern SSI_OUTPUT_FILE ssi_output_file(char *path);
	extern SSI_INPUT_FILE ssi_input_file(char *path);
	extern double ssi_cputime(void);


#ifdef __cplusplus
}
#endif

#define SSI_TRUE								1
#define SSI_FALSE								0
#define SSIRET_FAILS							-1
#define SSIRET_SUCCESS							0
#define SSIRET_DIMENSION_LE_ZERO				1
#define SSIRET_OUT_OF_MEMORY					2
#define SSIRET_BREAKDOWN						3
#define SSIRET_SOLVERS_UNKNOWN					4
#define SSIRET_MAXITER							5
#define SSIRET_MAXITER_LE_ZERO					6



#define ssi_symv	dsymv_
#define ssi_getrs	dgetrs_
#define ssi_getrf	dgetrf_
#define ssi_lacpy	dlacpy_
#define ssi_laset	dlaset_
#define ssi_potrs	dpotrs_
#define ssi_potrf	dpotrf_
//#define ssi_symv	DSYMV
//#define ssi_getrs	DGETRS
//#define ssi_getrf	DGETRF
//#define ssi_lacpy	DLACPY
//#define ssi_laset	DLASET


#endif
