/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_matrix.c,v 1.4 2007/10/31 06:23:14 kajiyama Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ssi_leqlib.h"

int ssi_matrix_create(SSI_MATRIX_PARAMS params, SSI_MATRIX **Amat)
{
	int err;
	int len,ilen;
	int np,n,nnz,ndz,nnd,maxnzr,bnnz,bnr,bnc,bn;
	int array_base,matrix_type,matrix_base;

	*Amat = NULL;

	matrix_type = params.matrix_type;
	array_base  = params.array_base;
	matrix_base = params.matrix_base;
//	np	        = params.np;
	n	        = params.n;
	nnz         = params.nnz;
	ndz         = params.ndz;
	nnd         = params.nnd;
	maxnzr      = params.maxnzr;
	bn          = params.bn;
	bnr         = params.bnr;
	bnc         = params.bnc;
	bnnz        = params.bnnz;

	if( 0>=np ) return SSIRET_DIMENSION_LE_ZERO;
	if( 0>=n ) return SSIRET_DIMENSION_LE_ZERO;
//	if( 0>array_base ) return SSIRET_FAILS;
//	if( 0>matrix_base ) return SSIRET_FAILS;

	switch( matrix_type )
	{
	case SSI_MATRIX_TYPE_DENSE:
		len = np*np*sizeof(SSI_SCALAR);
		break;
	case SSI_MATRIX_TYPE_CRS:
	case SSI_MATRIX_TYPE_CCS:
		if( 0>=nnz ) return SSIRET_FAILS;
		len  = nnz*sizeof(SSI_SCALAR);
		ilen = (nnz+np+1)*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_MSR:
		if( 0>=nnz ) return SSIRET_FAILS;
		if( 0>ndz /*|| 0<ndz*/) return SSIRET_FAILS;
		len  = (nnz+ndz+1)*sizeof(SSI_SCALAR);
		ilen = (nnz+ndz+1)*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_COO:
		if( 0>=nnz ) return SSIRET_FAILS;
		len  = nnz*sizeof(SSI_SCALAR);
		ilen = 2*nnz*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_DIA:
		if( 0>=nnd ) return SSIRET_FAILS;
		len  = np*nnd*sizeof(SSI_SCALAR);
		ilen = nnd*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_JDS:
		if( 0>=nnz || 0>=maxnzr ) return SSIRET_FAILS;
		len  = nnz*sizeof(SSI_SCALAR);
		ilen = (nnz+n+maxnzr+1)*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_ELL:
		if( 0>=maxnzr ) return SSIRET_FAILS;
		len  = np*maxnzr*sizeof(SSI_SCALAR);
		ilen = np*maxnzr*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_VBR:
		if( 0>=nnz || 0>=bnnz || 0>=bnr || 0>=bnc ) return SSIRET_FAILS;
		len  = nnz*sizeof(SSI_SCALAR);
		ilen = (2*bnnz+2*bnr+bnc+4)*sizeof(int);
		break;
	case SSI_MATRIX_TYPE_BSR:
	case SSI_MATRIX_TYPE_BSC:
		if( 0>=nnz || 0>=bn || bnnz>=0 ) return SSIRET_FAILS;
		if( np%bn!=0 ) return SSIRET_FAILS;
		len  = bnnz*bn*bn*sizeof(SSI_SCALAR);
		ilen = (np/bn+bnnz)*sizeof(int);
		break;
	default:
		return SSIRET_FAILS;
		break;
	}

	*Amat = malloc( sizeof(SSI_MATRIX) );
	if( NULL==*Amat ) return SSIRET_OUT_OF_MEMORY;

	(*Amat)->value       = NULL;
	(*Amat)->ptr         = NULL;
	(*Amat)->row         = NULL;
	(*Amat)->col         = NULL;
	(*Amat)->index       = NULL;
	(*Amat)->block_index = NULL;
	(*Amat)->block_ptr   = NULL;

	err = ssi_vector_create((void **)&(*Amat)->value, len, SSI_FALSE);
	if( err )
	{
		free(Amat);
		return err;
	}
	err = ssi_vector_create((void **)&(*Amat)->index, ilen, SSI_FALSE);
	if( err )
	{
		free((*Amat)->value);
		free(Amat);
		return err;
	}
	(*Amat)->matrix_type = matrix_type;
	(*Amat)->array_base  = SSI_MATRIX_ARRAY_BASE_0;
	(*Amat)->matrix_base = SSI_MATRIX_ARRAY_BASE_0;
//	(*Amat)->array_base  = array_base;
//	(*Amat)->matrix_base = matrix_base;
	(*Amat)->np          = np;
	(*Amat)->n           = n;
	(*Amat)->nnz         = nnz;
	(*Amat)->ndz         = ndz;
	(*Amat)->nnd         = nnd;
	(*Amat)->maxnzr      = maxnzr;
	(*Amat)->bn          = bn;
	(*Amat)->bnr         = bnr;
	(*Amat)->bnc         = bnc;
	(*Amat)->bnnz        = bnnz;

	switch( matrix_type )
	{
	case SSI_MATRIX_TYPE_CRS:
		(*Amat)->row   = (*Amat)->index + nnz;
		break;
	case SSI_MATRIX_TYPE_CCS:
		(*Amat)->col   = (*Amat)->index + nnz;
		break;
	case SSI_MATRIX_TYPE_COO:
		(*Amat)->row   = (*Amat)->index;
		(*Amat)->col   = &(*Amat)->row[nnz];
		break;
	case SSI_MATRIX_TYPE_JDS:
		(*Amat)->row   = (*Amat)->index + nnz;
		(*Amat)->ptr   = &(*Amat)->row[n];
		break;
	case SSI_MATRIX_TYPE_VBR:
		(*Amat)->row         = (*Amat)->index + nnz;
		(*Amat)->col         = &(*Amat)->row[bnr+1];
		(*Amat)->block_ptr   = &(*Amat)->col[bnc+1];
		(*Amat)->block_index = &(*Amat)->block_ptr[bnr+1];
		break;
	case SSI_MATRIX_TYPE_BSR:
	case SSI_MATRIX_TYPE_BSC:
		(*Amat)->block_ptr   = (*Amat)->index;
		(*Amat)->block_index = &(*Amat)->block_ptr[np/bn+1];
		break;
	}
	return SSIRET_SUCCESS;
}

int ssi_matrix_destroy(SSI_MATRIX *Amat)
{
	if( Amat )
	{
		ssi_vector_destroy(Amat->value);
		ssi_vector_destroy(Amat->index);
		free(Amat);
	}
	Amat = NULL;

	return SSIRET_SUCCESS;
}

int ssi_vector_create(void **vec, int size, int iszero)
{
	*vec = malloc( size );
	if( *vec==NULL ) return SSIRET_OUT_OF_MEMORY;

	if( iszero )
	{
//		memset(*vec, 0, size);
	}
	return SSIRET_SUCCESS;
}

void ssi_vector_destroy(void *vec)
{
	free(vec);
}
