/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_matvec.c,v 1.4 2007/10/31 06:23:14 kajiyama Exp $
 */

#include <stdio.h>
#include <math.h>
#ifdef _OPENMP
#include <omp.h>
#endif
//#include <mpi.h>
#include "ssi_leqlib.h"

void ssi_matvec(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	switch( A->matrix_type )
	{
	case SSI_MATRIX_TYPE_DENSE:
		ssi_matvec_dns(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CRS:
		ssi_matvec_crs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CCS:
		ssi_matvec_ccs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_MSR:
		ssi_matvec_msr(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_DIA:
		ssi_matvec_dia(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_ELL:
		ssi_matvec_ell(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_JAD:
		ssi_matvec_jds(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_VBR:
		ssi_matvec_vbr(alpha, A, x, beta, y);
		break;
	default:
		break;
	}
}

void ssi_matvect(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	switch( A->matrix_type )
	{
	case SSI_MATRIX_TYPE_DENSE:
		ssi_matvect_dns(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CRS:
		ssi_matvect_crs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CCS:
		ssi_matvect_ccs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_MSR:
		ssi_matvect_msr(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_DIA:
		ssi_matvect_dia(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_ELL:
		ssi_matvect_ell(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_JAD:
		ssi_matvect_jds(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_VBR:
		ssi_matvect_vbr(alpha, A, x, beta, y);
		break;
	default:
		break;
	}
}

void ssi_matveca(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	switch( A->matrix_type )
	{
	case SSI_MATRIX_TYPE_DENSE:
		ssi_matveca_dns(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CRS:
		ssi_matveca_crs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_CCS:
		ssi_matveca_ccs(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_MSR:
		ssi_matveca_msr(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_DIA:
		ssi_matveca_dia(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_ELL:
		ssi_matveca_ell(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_JAD:
		ssi_matveca_jds(alpha, A, x, beta, y);
		break;
	case SSI_MATRIX_TYPE_VBR:
		ssi_matveca_vbr(alpha, A, x, beta, y);
		break;
	default:
		break;
	}
}

void ssi_matdia(SSI_MATRIX *A, SSI_SCALAR x[])
{
	switch( A->matrix_type )
	{
	case SSI_MATRIX_TYPE_DENSE:
		ssi_matdia_dns(A, x);
		break;
	case SSI_MATRIX_TYPE_CRS:
		ssi_matdia_crs(A, x);
		break;
	case SSI_MATRIX_TYPE_CCS:
		ssi_matdia_ccs(A, x);
		break;
	case SSI_MATRIX_TYPE_MSR:
		ssi_matdia_msr(A, x);
		break;
	case SSI_MATRIX_TYPE_DIA:
		ssi_matdia_dia(A, x);
		break;
	case SSI_MATRIX_TYPE_ELL:
		ssi_matdia_ell(A, x);
		break;
	case SSI_MATRIX_TYPE_JAD:
		ssi_matdia_jds(A, x);
		break;
	case SSI_MATRIX_TYPE_VBR:
		ssi_matdia_vbr(A, x);
		break;
	default:
		break;
	}
}

void ssi_matvec_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;

	n = A->n;

	if (beta != 1.0)

#pragma omp parallel for
	  for(i=0; i<n; i++)
	    {
	      y[i] = beta * y[i];
	    }
#pragma omp parallel for private(j)
	for(i=0; i<n; i++)
	{
		for(j=0; j<n; j++)
		{
			y[i] += alpha * A->value[i*n+j] * x[j];
		}
	}
}

void ssi_matvec_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel private(i,j,js,je,t,jj)
{
//#pragma omp parallel for private(i) 
#pragma omp for
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
//#pragma omp parallel for private(i,j,js,je,t,jj)
#pragma omp for 
	for(i=is; i<ie; i++)
	{
		js = A->row[i];
		je = A->row[i+1];
		t = y[i];
		for(j=js;j<je;j++)
//		for(j=A->row[i];j<A->row[i+1];j++)
		{
			jj  = A->index[j];
			t += alpha * A->value[j] * x[jj];
		}
		y[i] = t;
	}
}
}

void ssi_matvec_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel for private(i) 
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,js,je,t,jj)
	for(i=is; i<ie; i++)
	{
		js = A->col[i];
		je = A->col[i+1];
		t = x[i];
		for(j=js;j<je;j++)
		{
			jj  = A->index[j];
			y[jj] += alpha * A->value[j] * t;
		}
	}
}

void ssi_matvec_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel for private(i)
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,js,je,t,jj)
	for(i=is; i<ie; i++)
	{
		t = y[i] + alpha * A->value[i] * x[i];
		js = A->index[i];
		je = A->index[i+1];
		for(j=js;j<je;j++)
		{
			jj = A->index[j];
			t += alpha * A->value[j] * x[jj];
		}
		y[i] = t;
	}
}

void ssi_matvec_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n,nnd;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	nnd  = A->nnd;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,is,ie,js,je,t,jj)
	for(j=0;j<nnd;j++)
	{
		jj = A->index[j];
		if( jj < 0 )
		{
			is = -jj;
			ie = n;
		}
		else
		{
			is = 0;
			ie = n-jj;
		}
		for(i=is;i<ie;i++)
		{
			y[i] += alpha * A->value[j*n + i] * x[jj+i];
		}
	}
}

void ssi_matvec_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n,maxnzr;
	int base;
	SSI_SCALAR t;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->matrix_base;
	x      = x - base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,is,ie,js,je,t,jj)
	for(j=0;j<maxnzr;j++)
	{
		for(i=0;i<n;i++)
		{
			y[i] += alpha * A->value[j*n + i] * x[A->index[j*n + i]];
		}
	}
}

void ssi_matvec_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k,is,ie,js,je,jj;
	int n,maxnzr;
	int base;
	SSI_SCALAR t;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,k,is,ie,js,je,t,jj)
	for(j=0;j<maxnzr;j++)
	{
		k = 0;
		is = A->ptr[j];
		ie = A->ptr[j+1];
		for(i=is;i<ie;i++)
		{
			y[A->row[k]] += alpha * A->value[i] * x[A->index[i]];
			k++;
		}
	}
}

void ssi_matvec_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k;
	int bi,bj;
	int bnr,bnc;
	int n;
	int base;

	n    = A->n;
	bnr  = A->bnr;
	bnc  = A->bnc;
	base = A->array_base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	k = 0;
	for(bi=0;bi<bnr;bi++)
	{
		for(bj=A->block_ptr[bi]-base;bj<A->block_ptr[bi+1]-base;bj++)
		{
			for(j=A->col[A->block_index[bj]-base]-base;j<A->col[A->block_index[bj]-base+1]-base;j++)
			{
				for(i=A->row[bi]-base;i<A->row[bi+1]-base;i++)
				{
					y[i] += alpha * A->value[k] * x[j];
					k++;
				}
			}
		}
	}
}

void ssi_matvect_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;

	n = A->n;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=0; i<n; i++)
	{
		for(j=0; j<n; j++)
		{
			y[i] += alpha * A->value[j*n+i] * x[j];
		}
	}
}

void ssi_matvect_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel for private(i) 
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=is; i<ie; i++)
	{
		js = A->row[i];
		je = A->row[i+1];
		t = x[i];
//#pragma omp parallel for private(i,j,jj)
		for(j=js;j<je;j++)
		{
			jj  = A->index[j];
			y[jj] += alpha * A->value[j] * t;
		}
	}
}

void ssi_matvect_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel for private(i) 
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,js,je,t,jj)
	for(i=is; i<ie; i++)
	{
		js = A->col[i];
		je = A->col[i+1];
		t = y[i];
		for(j=js;j<je;j++)
		{
			jj  = A->index[j];
			t += alpha * A->value[j] * x[jj];
		}
		y[i] = t;
	}
}

void ssi_matvect_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	base = A->matrix_base;
	x    = x - base;
	y    = y - base;
	is   = base;
	ie   = n+base;

#pragma omp parallel for private(i)
	for(i=is; i<ie; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,js,je,t,jj)
	for(i=is; i<ie; i++)
	{
		t = x[i];
		y[i] += alpha * A->value[i] * t;
		js = A->index[i];
		je = A->index[i+1];
		for(j=js;j<je;j++)
		{
			jj = A->index[j];
			y[jj] += alpha * A->value[j] * t;
		}
	}
}

void ssi_matvect_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n,nnd;
	int base;
	SSI_SCALAR t;

	n    = A->n;
	nnd  = A->nnd;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,is,ie,js,je,t,jj)
	for(j=0;j<nnd;j++)
	{
		jj = A->index[j];
		if( jj < 0 )
		{
			is = -jj;
			ie = n;
		}
		else
		{
			is = 0;
			ie = n-jj;
		}
		t = y[jj+i];
		for(i=is;i<ie;i++)
		{
			t += alpha * A->value[j*n + i] * x[i];
		}
		y[jj+i] = t;
	}
}

void ssi_matvect_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie,js,je,jj;
	int n,maxnzr;
	int base;
	SSI_SCALAR t;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->matrix_base;
	x      = x - base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,is,ie,js,je,t,jj)
	for(j=0;j<maxnzr;j++)
	{
		for(i=0;i<n;i++)
		{
			y[A->index[j*n + i]] += alpha * A->value[j*n + i] * x[i];
		}
	}
}

void ssi_matvect_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k,is,ie,js,je,jj;
	int n,maxnzr;
	int base;
	SSI_SCALAR t;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
#pragma omp parallel for private(i,j,k,is,ie,js,je,t,jj)
	for(j=0;j<maxnzr;j++)
	{
		k = 0;
		is = A->ptr[j];
		ie = A->ptr[j+1];
		for(i=is;i<ie;i++)
		{
			y[A->index[i]] += alpha * A->value[i] * x[A->row[k]];
			k++;
		}
	}
}

void ssi_matvect_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k;
	int bi,bj;
	int bnr,bnc;
	int n;
	int base;

	n    = A->n;
	bnr  = A->bnr;
	bnc  = A->bnc;
	base = A->array_base;

#pragma omp parallel for private(i)
	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	k = 0;
	for(bi=0;bi<bnr;bi++)
	{
		for(bj=A->block_ptr[bi]-base;bj<A->block_ptr[bi+1]-base;bj++)
		{
			for(j=A->col[A->block_index[bj]-base]-base;j<A->col[A->block_index[bj]-base+1]-base;j++)
			{
				for(i=A->row[bi]-base;i<A->row[bi+1]-base;i++)
				{
					y[j] += alpha * A->value[k] * x[i];
					k++;
				}
			}
		}
	}
}

void ssi_matveca_dns(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;

	n = A->n;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=0; i<n; i++)
	{
		for(j=0; j<n; j++)
		{
			y[i] += alpha * fabs(A->value[i*n+j]) * x[j];
		}
	}
}

void ssi_matveca_crs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=0; i<n; i++)
	{
		for(j=A->row[i]-base;j<A->row[i+1]-base;j++)
		{
			y[i] += alpha * fabs(A->value[j]) * x[A->index[j]-base];
		}
	}
}

void ssi_matveca_ccs(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=0; i<n; i++)
	{
		for(j=A->col[i]-base;j<A->col[i+1]-base;j++)
		{
			y[A->index[j]-base] += alpha * fabs(A->value[j]) * x[i];
		}
	}
}

void ssi_matveca_msr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(i=0; i<n; i++)
	{
		y[i] += alpha * A->value[i] * x[i];
		for(j=A->index[i]-base;j<A->index[i+1]-base;j++)
		{
			y[i] += alpha * fabs(A->value[j]) * x[A->index[j]-base];
		}
	}
}

void ssi_matveca_dia(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,is,ie;
	int n,nnd;
	int base;

	n    = A->n;
	nnd  = A->nnd;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(j=0;j<nnd;j++)
	{
		if( A->index[j] < 0 )
		{
			is = -A->index[j];
			ie = n;
		}
		else
		{
			is = 0;
			ie = n-A->index[j];
		}
		for(i=is;i<ie;i++)
		{
			y[i] += alpha * fabs(A->value[j*n + i]) * x[A->index[j]+i];
		}
	}
}

void ssi_matveca_ell(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j;
	int n,maxnzr;
	int base;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(j=0;j<maxnzr;j++)
	{
		for(i=0;i<n;i++)
		{
			y[i] += alpha * fabs(A->value[j*n + i]) * x[A->index[j*n + i]-base];
		}
	}
}

void ssi_matveca_jds(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k;
	int n,maxnzr;
	int base;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	for(j=0;j<maxnzr;j++)
	{
		k = 0;
		for(i=A->ptr[j]-base;i<A->ptr[j+1]-base;i++)
		{
			y[A->row[k]-base] += alpha * fabs(A->value[i]) * x[A->index[i]-base];
			k++;
		}
	}
}

void ssi_matveca_vbr(SSI_SCALAR alpha, SSI_MATRIX *A, SSI_SCALAR x[], SSI_SCALAR beta, SSI_SCALAR y[])
{
	int i,j,k;
	int bi,bj;
	int bnr,bnc;
	int n;
	int base;

	n    = A->n;
	bnr  = A->bnr;
	bnc  = A->bnc;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		y[i] = beta * y[i];
	}
	k = 0;
	for(bi=0;bi<bnr;bi++)
	{
		for(bj=A->block_ptr[bi]-base;bj<A->block_ptr[bi+1]-base;bj++)
		{
			for(j=A->col[A->block_index[bj]-base]-base;j<A->col[A->block_index[bj]-base+1]-base;j++)
			{
				for(i=A->row[bi]-base;i<A->row[bi+1]-base;i++)
				{
					y[i] += alpha * fabs(A->value[k]) * x[j];
					k++;
				}
			}
		}
	}
}

void ssi_matdia_dns(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n;

	n = A->n;

	for(i=0; i<n; i++)
	{
		for(j=0; j<n; j++)
		{
			A->value[i*n+j] *= x[j];
		}
	}
}

void ssi_matdia_crs(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		for(j=A->row[i]-base;j<A->row[i+1]-base;j++)
		{
			A->value[j] *= x[A->index[j]-base];
		}
	}
}

void ssi_matdia_ccs(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		for(j=A->col[i]-base;j<A->col[i+1]-base;j++)
		{
			A->value[j] *= x[i];
		}
	}
}

void ssi_matdia_msr(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n;
	int base;

	n    = A->n;
	base = A->array_base;

	for(i=0; i<n; i++)
	{
		A->value[i] *= x[i];
		for(j=A->index[i]-base;j<A->index[i+1]-base;j++)
		{
			A->value[j] *= x[A->index[j]-base];
		}
	}
}

void ssi_matdia_dia(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j,is,ie;
	int n,nnd;
	int base;

	n    = A->n;
	nnd  = A->nnd;
	base = A->array_base;

	for(j=0;j<nnd;j++)
	{
		if( A->index[j] < 0 )
		{
			is = -A->index[j];
			ie = n;
		}
		else
		{
			is = 0;
			ie = n-A->index[j];
		}
		for(i=is;i<ie;i++)
		{
			A->value[j*n + i] *= x[A->index[j]+i];
		}
	}
}

void ssi_matdia_ell(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n,maxnzr;
	int base;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

	for(j=0;j<maxnzr;j++)
	{
		for(i=0;i<n;i++)
		{
			A->value[j*n + i] *= x[A->index[j*n + i]-base];
		}
	}
}

void ssi_matdia_jds(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j;
	int n,maxnzr;
	int base;

	n      = A->n;
	maxnzr = A->maxnzr;
	base   = A->array_base;

	for(j=0;j<maxnzr;j++)
	{
		for(i=A->ptr[j]-base;i<A->ptr[j+1]-base;i++)
		{
			A->value[i] *= x[A->index[i]-base];
		}
	}
}

void ssi_matdia_vbr(SSI_MATRIX *A, SSI_SCALAR x[])
{
	int i,j,k;
	int bi,bj;
	int bnr,bnc;
	int n;
	int base;

	n    = A->n;
	bnr  = A->bnr;
	bnc  = A->bnc;
	base = A->array_base;

	k = 0;
	for(bi=0;bi<bnr;bi++)
	{
		for(bj=A->block_ptr[bi]-base;bj<A->block_ptr[bi+1]-base;bj++)
		{
			for(j=A->col[A->block_index[bj]-base]-base;j<A->col[A->block_index[bj]-base+1]-base;j++)
			{
				for(i=A->row[bi]-base;i<A->row[bi+1]-base;i++)
				{
					A->value[k] *= x[j];
					k++;
				}
			}
		}
	}
}

#if 0
int ssi_send_recv(int n, SSI_COMMTABLE *commtable, SSI_SCALAR ws[], SSI_SCALAR wr[], SSI_SCALAR x[])
{
	int neib,i,is,inum,neibpetot;
/*
	neibpetot = commtable->neibpetot;
	for(neib=0;neib<neibpetot;neib++)
	{
		is = commtable->index_export[neib];
		inum = commtable->index_export[neib+1] - is;
		for(i=is;i<is+inum;i++)
		{
			ws[i] = x[commtable->node_export[i]];
		}
		MPI_Isend(ws,inum,MPI_DOUBLE_PRECISION,commtable->neibpe[neib],0,commtable->comm,&commtable->req1[neib]);
	}
	for(neib=0;neib<neibpetot;neib++)
	{
		is = commtable->index_import[neib];
		inum = commtable->index_import[neib+1] - is;
		MPI_Irecv(wr,inum,MPI_DOUBLE_PRECISION,commtable->neibpe[neib],0,commtable->comm,&commtable->req2[neib]);
	}
	MPI_Waitall(neibpetot, commtable->req2, commtable->sta2);
	for(neib=0;neib<neibpetot;neib++)
	{
		is = commtable->index_import[neib];
		inum = commtable->index_import[neib+1] - is;
		for(i=is;i<is+inum;i++)
		{
			x[commtable->node_import[i]] = wr[i];
		}
	}
	MPI_Waitall(neibpetot, commtable->req1, commtable->sta1);
	*/
	return SSIRET_SUCCESS;
}
#endif
