/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: ssi_nrm2.c,v 1.4 2007/10/31 06:23:14 kajiyama Exp $
 */

#include <math.h>
#include "ssi_leqlib.h"

SSI_SCALAR ssi_nrm2(int n, SSI_SCALAR *vx)
{
	int i;
	SSI_SCALAR scale, ssq, absvxi, tmp;
	const SSI_SCALAR one  = (SSI_SCALAR)1;
	const SSI_SCALAR zero = (SSI_SCALAR)0;
	SSI_SCALAR norm = zero;

	if( 0>=n ) return zero;
	scale = zero;
	ssq   = one;
//#pragma omp parallel for private(i,absvxi,tmp,scale) reduction(+:ssq)
	for(i=0; i<n; i++)
	{
		if( zero!=vx[i] )
		{
			absvxi = fabs(vx[i]);
			if( scale < absvxi )
			{
				tmp   = scale / absvxi;
				ssq   = one + ssq*tmp*tmp;
				scale = absvxi; 
			}
			else
			{
				tmp   = absvxi / scale;
				ssq   = ssq + tmp*tmp;
			}
		}
	}
	norm = scale * sqrt(ssq);
	return norm;
}
