/* 
 *  Copyright (c) 2006 Tamito KAJIYAMA.  All rights reserved.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation, and that the name of Tamito KAJIYAMA not be
 *  used in advertising or publicity pertaining to distribution of the
 *  software without specific, written prior permission.
 *
 *  TAMITO KAJIYAMA DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 *  INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 *  EVENT SHALL TAMITO KAJIYAMA BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 *  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 *  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 *  PERFORMANCE OF THIS SOFTWARE.
 *
 *  $Id: mp_crs.h,v 1.1 2007/08/25 22:47:06 kajiyama Exp $
 */

#ifndef MP_CRS_H
#define MP_CRS_H 1

#include <stdio.h>
#include <gmp.h>

typedef struct {
  int length;
  mpf_t *value;
} mp_vector_t;

typedef struct {
  int m, n, nnz;
  mpf_t *value;
  int *row, *index;
} mp_crs_t;

mp_crs_t *mp_crs_new(int, int, int);
int mp_crs_free(mp_crs_t *);
mp_vector_t *mp_vector_new(int);
int mp_vector_free(mp_vector_t *);
int mp_vector_copy(mp_vector_t *, mp_vector_t *);
int mp_vector_set(mp_vector_t *, mpf_t);
int mp_vector_axpy(mpf_t, mp_vector_t *, mp_vector_t *);
int mp_vector_xpay(mp_vector_t *, mpf_t, mp_vector_t *);
int mp_vector_dot(mpf_t, mp_vector_t *, mp_vector_t *);
int mp_vector_norm2(mpf_t, mp_vector_t *);
int mp_crs_matvec(mpf_t, mp_crs_t *, mp_vector_t *, mpf_t, mp_vector_t *);
int mp_crs_tmatvec(mpf_t, mp_crs_t *, mp_vector_t *, mpf_t, mp_vector_t *);

#define mp_crs_value(p, i) ((p)->value[(i)])
#define mp_crs_row(p, i)   ((p)->row[(i)])
#define mp_crs_index(p, i) ((p)->index[(i)])
#define mp_crs_m(p)        ((p)->m)
#define mp_crs_n(p)        ((p)->n)
#define mp_crs_nnz(p)      ((p)->nnz)

#define mp_vector_value(p, i) ((p)->value[(i)])
#define mp_vector_length(p)   ((p)->length)

#ifdef USE_HISTORY
extern mp_vector_t *mp_residual_history;
int mp_init_residual_history(int);
int mp_free_residual_history(void);
#endif

#endif /* MP_CRS_H */
