/* 
 *  Copyright (c) 2006 Tamito KAJIYAMA.  All rights reserved.
 *
 *  Permission to use, copy, modify, and distribute this software and its
 *  documentation for any purpose and without fee is hereby granted,
 *  provided that the above copyright notice appear in all copies and that
 *  both that copyright notice and this permission notice appear in
 *  supporting documentation, and that the name of Tamito KAJIYAMA not be
 *  used in advertising or publicity pertaining to distribution of the
 *  software without specific, written prior permission.
 *
 *  TAMITO KAJIYAMA DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 *  INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 *  EVENT SHALL TAMITO KAJIYAMA BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 *  CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF
 *  USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 *  OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 *  PERFORMANCE OF THIS SOFTWARE.
 *
 *  $Id: mp_crs_test.c,v 1.1 2007/08/25 22:47:06 kajiyama Exp $
 */

#include <stdio.h>

#ifdef _OPENMP
#include <omp.h>
#endif

#include "mp_crs.h"

int
main(int argc, char *argv[])
{
  mp_crs_t *A;
  mp_vector_t *x, *y;
  mpf_t dot, alpha, beta;
  int i, j;

#ifdef _OPENMP
  printf("omp_get_max_threads() => %d\n", omp_get_max_threads());
#endif

  mpf_init(alpha);
  mpf_init(beta);

  printf("> x = {1, 2, 3, 4}\n");
  x = mp_vector_new(4);
  for (i = 0; i < mp_vector_length(x); i++) {
    mpf_set_si(mp_vector_value(x, i), i+1);
    gmp_printf("x[%d] = %Fe\n", i, mp_vector_value(x, i));
  }

  printf("> y = -x\nexpect: {-1, -2, -3, -4}\n");
  y = mp_vector_new(mp_vector_length(x));
  for (i = 0; i < mp_vector_length(y); i++) {
    mpf_neg(mp_vector_value(y, i), mp_vector_value(x, i));
    gmp_printf("y[%d] = %Fe\n", i, mp_vector_value(y, i));
  }

  printf("> y = x - 2 * y\nexpect: {3, 6, 9, 12}\n");
  mpf_set_si(alpha, -2);
  mp_vector_xpay(x, alpha, y);
  for (i = 0; i < mp_vector_length(y); i++)
    gmp_printf("y[%d] = %Fe\n", i, mp_vector_value(y, i));

  mpf_init(dot);
  mp_vector_dot(dot, x, y);
  gmp_printf("> dot = x' * y\nexpect: 90\n%Fe\n", dot);

  printf("> A = | 11          |\n"
         "      | 21 22       |\n"
         "      |    32 33    |\n"
         "      | 41    43 44 |\n");
  A = mp_crs_new(4, 4, 8);
  mpf_set_d(mp_crs_value(A, 0), 11.0); mp_crs_index(A, 0) = 0;
  mpf_set_d(mp_crs_value(A, 1), 21.0); mp_crs_index(A, 1) = 0;
  mpf_set_d(mp_crs_value(A, 2), 22.0); mp_crs_index(A, 2) = 1;
  mpf_set_d(mp_crs_value(A, 3), 32.0); mp_crs_index(A, 3) = 1;
  mpf_set_d(mp_crs_value(A, 4), 33.0); mp_crs_index(A, 4) = 2;
  mpf_set_d(mp_crs_value(A, 5), 41.0); mp_crs_index(A, 5) = 0;
  mpf_set_d(mp_crs_value(A, 6), 43.0); mp_crs_index(A, 6) = 2;
  mpf_set_d(mp_crs_value(A, 7), 44.0); mp_crs_index(A, 7) = 3;
  mp_crs_row(A, 0) = 0;
  mp_crs_row(A, 1) = 1;
  mp_crs_row(A, 2) = 3;
  mp_crs_row(A, 3) = 5;
  mp_crs_row(A, 4) = 8;
  for (i = 0; i < mp_crs_m(A); i++) {
    for (j = mp_crs_row(A, i); j < mp_crs_row(A, i+1); j++) {
      gmp_printf("A[%d,%d] = %Fe\n", i,
                 mp_crs_index(A, j),
                 mp_crs_value(A, j));
    }
  }
  mpf_set_d(alpha, 1.0);
  mpf_set_d(beta, 0.0);
  printf("> y = A * x\nexpect: {11, 65, 163, 346}\n");
  mp_crs_matvec(alpha, A, x, beta, y);
  for (i = 0; i < mp_vector_length(y); i++)
    gmp_printf("y[%d] = %Fe\n", i, mp_vector_value(y, i));
  printf("> y = A' * x\nexpect: {217, 140, 271, 176}\n");
  mp_crs_tmatvec(alpha, A, x, beta, y);
  for (i = 0; i < mp_vector_length(y); i++)
    gmp_printf("y[%d] = %Fe\n", i, mp_vector_value(y, i));
  mp_crs_free(A);
  mp_vector_free(x);
  mp_vector_free(y);
  mpf_clear(dot);
  mpf_clear(alpha);
  mpf_clear(beta);
  return 0;
}
