/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: clist.h,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

/* clist.h - compressed list */

#ifndef SSI_CLIST_H
#define SSI_CLIST_H 1

typedef enum {
  CListType = 0,
  CListPointType,
  CListRangeType
} clist_type_t;

typedef struct _clist {
  clist_type_t type;
  struct _clist *next;
  int start, end;
} clist_t;

typedef struct {
  clist_type_t type;
  struct _clist *next;
  int n;
} clist_point_t; /* inherits clist_t */

typedef struct {
  clist_type_t type;
  struct _clist *next;
  int start, end;
} clist_range_t; /* inherits clist_t */

#define CLIST_POINT(p) ((clist_point_t *)(p))
#define CLIST_RANGE(p) ((clist_range_t *)(p))

#if defined(_WIN32) && !defined(STATIC_MODULES)
#ifdef _EXPORTING
#define DECLSPEC __declspec(dllexport)
#else
#define DECLSPEC __declspec(dllimport)
#endif
#else
#define DECLSPEC
#endif

DECLSPEC clist_t *clist_new();
DECLSPEC void clist_free(clist_t *);
DECLSPEC clist_t *clist_append_point(clist_t *, int);
DECLSPEC clist_t *clist_append_range(clist_t *, int, int);
DECLSPEC void clist_set_defaults(clist_t *, int, int);
DECLSPEC void clist_reset_defaults(clist_t *);
DECLSPEC int clist_range(clist_t *, int *, int *);
DECLSPEC int clist_length(clist_t *);
DECLSPEC int clist_nth(clist_t *, int);
DECLSPEC int clist_min(clist_t *);
DECLSPEC int clist_max(clist_t *);
DECLSPEC int *clist_expand(clist_t *);
DECLSPEC clist_t *clist_compress(int *, int);
DECLSPEC clist_t *clist_sublist(clist_t *, clist_t *);
DECLSPEC clist_t *clist_duplicate(clist_t *);
DECLSPEC void clist_pprint(clist_t *);

#endif /* SSI_CLIST_H */
