%{
/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: cmd_lexer.l,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include "cmd.h"
#include "cmd_parser.h"

static char cmd_text[SILC_PROGRAM_MAX+1], *cmd_text_ptr;

void set_cmd_text(char *s)
{
  cmd_text_ptr = strcpy(cmd_text, s);
}

int get_cmd_text_ptr()
{
  return cmd_text_ptr - cmd_text;
}

#define YY_INPUT(buf,result,max_size) \
        result = ((buf[0] = *cmd_text_ptr++) == '\0') ? YY_NULL : 1;

void cmderror(const char *);

%}

%option prefix="cmd"

%%
~'                      return TOK_TRANSPOSE;
~                       return TOK_COMPLEX_CONJUGATE;
\\                      return TOK_LEQ;
\/@=                    return TOK_DIVIDE_ELEMENTWISE_EQUAL;
\/@                     return TOK_DIVIDE_ELEMENTWISE;
\/=                     return TOK_DIVIDE_EQUAL;
\/                      return TOK_DIVIDE;
\-=                     return TOK_MINUS_EQUAL;
\-                      return TOK_MINUS;
\+=                     return TOK_PLUS_EQUAL;
\+                      return TOK_PLUS;
\*@=                    return TOK_MULTIPLY_ELEMENTWISE_EQUAL;
\*@                     return TOK_MULTIPLY_ELEMENTWISE;
\*=                     return TOK_MULTIPLY_EQUAL;
\*                      return TOK_MULTIPLY;
\%=                     return TOK_REMAINDER_EQUAL;
\%                      return TOK_REMAINDER;
=                       return TOK_EQUAL;
'~                      return TOK_TRANSPOSE;
'                       return TOK_CONJUGATE_TRANSPOSE;
;;                      return TOK_DOUBLE_SEMICOLON;
prefer                  return TOK_PREFER;
rename                  return TOK_RENAME;
[0-9]+\.[0-9]*[Dd][+\-][0-9]+    |
[0-9]*\.[0-9]+[Dd][+\-][0-9]+    {
                          char *p;
                          p = strchr(yytext, 'D'); if (p) *p = 'E';
                          p = strchr(yytext, 'd'); if (p) *p = 'e';
                          cmdlval.num_real8 = atof(yytext);
                          return TOK_REAL8_LITERAL;
                        }
[0-9]+\.[0-9]*`S        |
[0-9]*\.[0-9]+`S        |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+`S  |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+`S  {
                          cmdlval.num_real4 = atof(yytext);
                          return TOK_REAL4_LITERAL;
                        }
[0-9]+\.[0-9]*`D        |
[0-9]*\.[0-9]+`D        |
[0-9]+\.[0-9]*          |
[0-9]*\.[0-9]+          |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+`D  |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+`D  |
[0-9]+\.[0-9]*[Ee][+\-][0-9]+    |
[0-9]*\.[0-9]+[Ee][+\-][0-9]+    {
                          cmdlval.num_real8 = atof(yytext);
                          return TOK_REAL8_LITERAL;
                        }
[0-9]+`D                {
                          cmdlval.num_int8 = atol(yytext);
			  return TOK_INT8_LITERAL;
                        }
[0-9]+`S                |
[0-9]+                  {
                          cmdlval.num_int4 = atoi(yytext);
			  return TOK_INT4_LITERAL;
                        }
[_A-Za-z][_A-Za-z0-9]*  {
			  if (yyleng < SILC_MAXNAMELEN) {
                            strcpy(cmdlval.string, yytext);
			    return TOK_IDENTIFIER;
			  }
			  cmderror("too long identifier");
                        }
[\[\](){}:;,]           return yytext[0];
[ \t\n]+                ; /* ignore whitespace */
.                       return TOK_EOL;
%%
