/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: complex_sqrt.c,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

#include <math.h>

void
c_sqrt(float *in, float *out)
{
  float real, imag, c;

  real = in[0]; imag = in[1];
  if ((int)imag == 0) {
    if (real < 0) {
      out[0] = 0.0;
      out[1] = sqrt(-real);
    } else {
      out[0] = sqrt(real);
      out[1] = 0.0;
    }
  } else if (real < 0) {
    c = (sqrt(real * real + imag * imag) - real) / 2;
    out[0] = imag / (2 * sqrt(c));
    out[1] = sqrt(c);
  } else {
    c = (sqrt(real * real + imag * imag) + real) / 2;
    out[0] = sqrt(c);
    out[1] = imag / (2 * sqrt(c));
  }
}

void
z_sqrt(double *in, double *out)
{
  double real, imag, c;

  real = in[0]; imag = in[1];
  if ((int)imag == 0) {
    if (real < 0) {
      out[0] = 0.0;
      out[1] = sqrt(-real);
    } else {
      out[0] = sqrt(real);
      out[1] = 0.0;
    }
  } else if (real < 0) {
    c = (sqrt(real * real + imag * imag) - real) / 2;
    out[0] = imag / (2 * sqrt(c));
    out[1] = sqrt(c);
  } else {
    c = (sqrt(real * real + imag * imag) + real) / 2;
    out[0] = sqrt(c);
    out[1] = imag / (2 * sqrt(c));
  }
}
