/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: dense_alloc.c,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "object.h"
#include "dense.h"

#define should_not_reach_here  fprintf(stderr, \
  "*** should not reach here (%s, line %d)\n", __FILE__, __LINE__)

silc_dense_t *
silc_dense_allocate(int precision, size_t m, size_t n)
{
  silc_dense_t *in;
  int i;

  in = (silc_dense_t *)malloc(sizeof(silc_dense_t));
  if (!in)
    return NULL;
  switch (precision) {
  case SILC_INT:
    in->v.i = (int32_t *)malloc(sizeof(int32_t) * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.i + m * i, 0, sizeof(int32_t) * m);
#endif
    break;
  case SILC_LONG:
    in->v.l = (int64_t *)malloc(sizeof(int64_t) * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.l + m * i, 0, sizeof(int64_t) * m);
#endif
    break;
  case SILC_FLOAT:
    in->v.s = (float *)malloc(sizeof(float) * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.s + m * i, 0, sizeof(float) * m);
#endif
    break;
  case SILC_DOUBLE:
    in->v.d = (double *)malloc(sizeof(double) * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.d + m * i, 0, sizeof(double) * m);
#endif
    break;
  case SILC_COMPLEX:
    in->v.c = (float *)malloc(sizeof(float) * 2 * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.c + 2 * m * i, 0, sizeof(float) * 2 * m);
#endif
    break;
  case SILC_DOUBLE_COMPLEX:
    in->v.z = (double *)malloc(sizeof(double) * 2 * m * n);
#ifdef ENABLE_NUMA
#pragma omp parallel for private(i)
    for (i = 0; i < n; i++)
      memset(in->v.z + 2 * m * i, 0, sizeof(double) * 2 * m);
#endif
    break;
  default:
    should_not_reach_here;
  }
  return in;
}
