/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: dl.c,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

#include <stdio.h>

#include "dl.h"

#ifdef STATIC_MODULES

void *
silc_dlopen(char *filename)
{
  return NULL;
}

void *
silc_dlsym(void *handle, char *symbol)
{
  return NULL;
}

int
silc_dlclose(void *handle)
{
  return -1;
}

const char *
silc_dlerror(void)
{
  return NULL;
}

#else /* STATIC_MODULES */

#ifdef _osx

NSModule
silc_dlopen(char *filename)
{
  NSObjectFileImage fileImage;
  NSObjectFileImageReturnCode returnCode;
  NSModule handle;

  returnCode = NSCreateObjectFileImageFromFile(filename, &fileImage);
  if(returnCode != NSObjectFileImageSuccess)
    return NULL;
  handle = NSLinkModule(fileImage, filename,
			NSLINKMODULE_OPTION_RETURN_ON_ERROR |
			NSLINKMODULE_OPTION_PRIVATE);
  NSDestroyObjectFileImage(fileImage);
  return handle;
}

void *
silc_dlsym(NSModule handle, char *symbol)
{
  NSSymbol nssym;
  char buf[SILC_MAXNAMELEN+1];

  strcpy(buf, "_");
  strcat(buf, symbol);
  nssym = NSLookupSymbolInModule(handle, buf);
  if (!nssym)
    return NULL;
  return NSAddressOfSymbol(nssym);
}

int
silc_dlclose(NSModule handle)
{
  return NSUnLinkModule(handle, FALSE);
}

const char *
silc_dlerror(void)
{
  NSLinkEditErrors c;
  int error;
  const char *filename, *message;

  NSLinkEditError(&c, &error, &filename, &message);
  return message;
}

#else /* _osx */

#ifdef _WIN32

silc_handle
silc_dlopen(char *filename)
{
  return LoadLibrary(filename);
}

void *
silc_dlsym(silc_handle handle, char *symbol)
{
  return GetProcAddress(handle, symbol);
}

int
silc_dlclose(silc_handle handle)
{
  return FreeLibrary(handle);
}

static char last_error[1024];

const char *
silc_dlerror(void)
{
  FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
                NULL, GetLastError(), MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                last_error, sizeof(last_error) / sizeof(char), NULL);
}

#else /* _WIN32 */

void *
silc_dlopen(char *filename)
{
  return dlopen(filename, RTLD_LAZY | RTLD_LOCAL);
}

void *
silc_dlsym(void *handle, char *symbol)
{
  return dlsym(handle, symbol);
}

int
silc_dlclose(void *handle)
{
  return dlclose(handle);
}

const char *
silc_dlerror(void)
{
  return dlerror();
}

#endif /* _WIN32 */

#endif /* _osx */

#endif /* STATIC_MODULES */
