/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: leq_gs.c,v 1.4 2007/10/31 05:50:17 kajiyama Exp $
 */

#include <malloc.h>
#include <string.h>
#include <math.h>

#include "module.h"
#include "dense.h"

#include "ssi_leqlib.h"

#define MODULE_NAME "leq_gs"

int
leq_gs_solve(int n_params, silc_mfunc_param_t *param)
{
  silc_object_t *A, *b, *x;
  silc_dense_t *A_in;
  size_t status;
  double *work;
  double ssi_params[SSI_PARAMS_LEN];
  int ssi_options[SSI_OPTIONS_LEN];
  SSI_MATRIX mat_A;

  A = param[0].o;
  b = param[1].o;
#ifdef DEBUG
  printf("leq_gs_solve(0x%lx, 0x%lx)\n", A, b);
#endif
  if (A->d.matrix.m != A->d.matrix.n) {
    fprintf(stderr, MODULE_NAME ": not a square matrix\n");
    return -1;
  }
  if (A->d.matrix.m != b->d.vector.length) {
    fprintf(stderr, MODULE_NAME ": matrix-vector size mismatch\n");
    return -1;
  }
  A_in = SILC_DENSE(A);
  x = silc_object_new_column_vector(SILC_DOUBLE, b->d.vector.length);
  memset(x->d.vector.v.d, 0, sizeof(double) * x->d.vector.length);
  mat_A.matrix_type = SSI_MATRIX_TYPE_DENSE;
  mat_A.array_base  = SSI_MATRIX_ARRAY_BASE_0;
  mat_A.matrix_base = SSI_MATRIX_ARRAY_BASE_0;
  mat_A.n           = A->d.matrix.m;
  mat_A.value	    = A_in->v.d;
  work = (double *)malloc(sizeof(double) * A->d.matrix.m * 6);
  ssi_params[SSI_PARAMS_RESID] = 1.0e-10;
  if (A->d.matrix.m < 100)
    ssi_options[SSI_OPTIONS_MAXITER] = 100;
  else
    ssi_options[SSI_OPTIONS_MAXITER] = A->d.matrix.m;
  ssi_options[SSI_OPTIONS_OUTPUT] = 0;
  status = ssi_gs(b->d.vector.v.d, x->d.vector.v.d, work,
		  ssi_params, ssi_options, &mat_A, NULL);
  free(work);
  if (status == SSIRET_MAXITER) {
    silc_object_free(x);
    fprintf(stderr, MODULE_NAME ": too many iterations\n");
    return -1;
  }
  param[2].o = x;
  return 0;
}

static binary_op_table_t _binary_op_table[] = {
  /* name, op, precision, x_type, x_format, y_type, y_format, out, flags */
  {leq_gs_solve, "leq_gs_solve", SILC_LEQ_OP, SILC_DOUBLE,
   SILC_MATRIX_TYPE, SILC_FORMAT_DENSE, SILC_COLUMN_VECTOR_TYPE, 0, 2, 0},
  {NULL} /* sentinel */
};

static function_table_t _function_table[] = {
  /* name, sig_text, flags */
  {leq_gs_solve, "leq_gs_solve", "vector leq_gs(matrix, vector)", 0},
  {NULL} /* sentinel */
};

void
init_leq_gs()
{
  silc_register_module(MODULE_NAME,
		       NULL,   /* _unary_op_table */
		       _binary_op_table,
		       _function_table,
		       NULL);  /* _procedure_table */
}
