%{
/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sig_lexer.l,v 1.5 2007/10/31 05:59:05 kajiyama Exp $
 */

#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include "sig.h"
#include "sig_parser.h"

char sig_text[SILC_SIGNATURE_MAX+1], *sig_text_ptr;

void set_sig_text(char *s)
{
  sig_text_ptr = strcpy(sig_text, s);
}

#define YY_INPUT(buf,result,max_size) \
        result = ((buf[0] = *sig_text_ptr++) == '\0') ? YY_NULL : 1;

void sigerror(const char *);

%}

%option prefix="sig"

%%
row			return ROW;
column			return COLUMN;
scalar			return SCALAR;
vector			return VECTOR;
matrix			return MATRIX;
inout			siglval.number = SILC_PARAM_INOUT; return IOTYPE;
in			siglval.number = SILC_PARAM_IN;    return IOTYPE;
out			siglval.number = SILC_PARAM_OUT;   return IOTYPE;
[_A-Za-z][_A-Za-z0-9]*  {
			  if (yyleng < SILC_MAXNAMELEN) {
			    strcpy(siglval.string, yytext);
			    return NAME;
			  }
			  sigerror("too long identifier");
                        }
\'[^\"\n\t]+\'		|
\"[^\"\n\t]+\"		{
			  if (yyleng < SILC_MAXNAMELEN) {
			    strncpy(siglval.string, yytext+1, yyleng-2);
			    siglval.string[yyleng-2] = '\0';
			    return FORMAT;
			  }
			  sigerror("too long string");
			}
[(,)]			return yytext[0];
[ \t\n]+		; /* ignore whitespace */
%%
