%{
/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sig_parser.y,v 1.4 2007/10/31 05:50:18 kajiyama Exp $
 */

#include <stdio.h>
#include <string.h>
#include <malloc.h>

#include "sig.h"

static signature_t *sig;
static int error;

void sigerror(const char *s)
{
  fprintf(stderr, "parser_signature: %s\n", s);
  error = -1;
}

int sigwrap(void)
{
  return 1;
} 

int siglex(void);

void add_param(int type, char *format_name, int iotype)
{
  param_t *p;
  int n;

  n = sig->n_params++;
  if (sig->param == NULL)
    sig->param = (param_t *)malloc(sizeof(param_t) * sig->n_params);
  else
    sig->param = (param_t *)realloc(sig->param,
				    sizeof(param_t) * sig->n_params);
  p = sig->param + n;
  p->type = type;
  p->iotype = iotype;
  strcpy(p->format_name, format_name);
}

%}

%union 
{
  char string[SILC_MAXNAMELEN];
  int number;
}

%token <string> NAME FORMAT
%token <number> IOTYPE

%token SCALAR ROW COLUMN VECTOR MATRIX

%type <number> iotype


%%

signature: function | procedure
;

function: SCALAR NAME '(' parameter_list ')'
{
  sig->return_type = SILC_SCALAR_TYPE;
  strcpy(sig->name, $2);
}
| ROW VECTOR NAME '(' parameter_list ')'
{
  sig->return_type = SILC_ROW_VECTOR_TYPE;
  strcpy(sig->name, $3);
}
| COLUMN VECTOR NAME '(' parameter_list ')'
{
  sig->return_type = SILC_COLUMN_VECTOR_TYPE;
  strcpy(sig->name, $3);
}
| VECTOR NAME '(' parameter_list ')'
{
  sig->return_type = SILC_COLUMN_VECTOR_TYPE;
  strcpy(sig->name, $2);
}
| MATRIX NAME '(' parameter_list ')'
{
  sig->return_type = SILC_MATRIX_TYPE;
  strcpy(sig->return_format_name, SILC_DEFAULT_FORMAT);
  strcpy(sig->name, $2);
}
| MATRIX FORMAT NAME '(' parameter_list ')'
{
  sig->return_type = SILC_MATRIX_TYPE;
  strcpy(sig->return_format_name, $2);
  strcpy(sig->name, $3);
}
;

procedure: NAME '(' parameter_list ')'
{
  sig->return_type = 0;
  strcpy(sig->name, $1);
}
;

parameter_list: /* empty */ | one_or_more_params
;

one_or_more_params: parameter | one_or_more_params ',' parameter
;

parameter: SCALAR iotype
{
  add_param(SILC_SCALAR_TYPE, "", $2);
}
| ROW VECTOR iotype
{
  add_param(SILC_ROW_VECTOR_TYPE, "", $3);
}
| COLUMN VECTOR iotype
{
  add_param(SILC_COLUMN_VECTOR_TYPE, "", $3);
}
| VECTOR iotype
{
  add_param(SILC_COLUMN_VECTOR_TYPE|SILC_ROW_VECTOR_TYPE, "", $2);
}
| MATRIX iotype
{
  add_param(SILC_MATRIX_TYPE, SILC_DEFAULT_FORMAT, $2);
}
| MATRIX FORMAT iotype
{
  add_param(SILC_MATRIX_TYPE, $2, $3);
}
;

iotype: /* empty */
{
  $$ = SILC_PARAM_IN;
}
| IOTYPE
;

%%

int parse_signature(char *s, signature_t *p)
{
  extern void set_sig_text(char *);

  set_sig_text(s);
  sig = p;
  sig->n_params = 0;
  sig->param = NULL;
  error = 0;
  yyparse();
  return error;
}
