/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sig_test.c,v 1.4 2007/10/31 05:50:18 kajiyama Exp $
 */

#include <stdio.h>

#include "sig_parser.h"

char *
matrix(char *format)
{
  static char s[SILC_MAXNAMELEN+10];

  sprintf(s, "matrix \"%s\"", format);
  return s;
}

int
main(int argc, char *argv[])
{
  signature_t s;
  int i;

  if (argc != 2) {
    fprintf(stderr, "Usage: %s <signature>\n", argv[0]);
    return -1;
  }
  if (parse_signature(argv[1], &s) < 0)
    return -1;
  printf("name: %s ", s.name);
  if (s.return_type == 0) {
    printf("(procedure)\n");
  } else {
    printf("(function)\n");
    printf("return type: %s\n",
           (s.return_type == SILC_SCALAR_TYPE) ? "scalar" :
           (s.return_type == SILC_ROW_VECTOR_TYPE) ? "row vector" :
           (s.return_type == SILC_COLUMN_VECTOR_TYPE) ? "column vector" :
           (s.return_type == SILC_MATRIX_TYPE) ? matrix(s.return_format_name) :
	   "(unknown)");
  }
  printf("%d param%s\n", s.n_params, (s.n_params < 2) ? "" : "s");
  for (i = 0; i < s.n_params; i++) {
    printf("param %d: %s (%s)\n", i+1,
           (s.param[i].type == SILC_SCALAR_TYPE) ? "scalar" :
           (s.param[i].type == SILC_ROW_VECTOR_TYPE) ? "row vector" :
           (s.param[i].type == SILC_COLUMN_VECTOR_TYPE) ? "column vector" :
           (s.param[i].type == SILC_MATRIX_TYPE) ? matrix(s.param[i].format_name) :
	   "(unknown)",
           (s.param[i].iotype == SILC_PARAM_IN) ? "in" :
           (s.param[i].iotype == SILC_PARAM_OUT) ? "out" :
           (s.param[i].iotype == SILC_PARAM_INOUT) ? "inout" :
	   "(unknown)");
  }
  printf("\n");
  return 0;
} 
