/*
 * Copyright (C) 2002-2007 The SSI Project. All rights reserved. 
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the project nor the names of its contributors 
 *       may be used to endorse or promote products derived from this software 
 *       without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE SSI PROJECT ``AS IS'' AND ANY EXPRESS 
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE SSI PROJECT BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 *  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * $Id: sparse_band_op.c,v 1.4 2007/10/31 05:50:18 kajiyama Exp $
 */

/* todo: take x_mods and y_mods into consideration */

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include <math.h>

#include <sys/types.h> /* for int32_t and int64_t */

#include "object.h"
#include "dense.h"
#include "sparse_crs.h"
#include "sparse_band.h"
#include "module.h"
#include "wtime.h"

#define should_not_reach_here  fprintf(stderr, \
  "*** should not reach here (%s, line %d)\n", __FILE__, __LINE__)

#define _max(x, y) (((x) > (y)) ? (x) : (y))
#define _min(x, y) (((x) < (y)) ? (x) : (y))

/* BLAS drivers */
void call_sgbmv(char trans, int m, int n, int kl, int ku, float alpha,
		float *a, int lda, float *x, int incx, float beta,
		float *y, int incy);
void call_dgbmv(char trans, int m, int n, int kl, int ku, double alpha,
		double *a, int lda, double *x, int incx, double beta,
		double *y, int incy);
void call_cgbmv(char trans, int m, int n, int kl, int ku, float *alpha,
		float *a, int lda, float *x, int incx, float *beta,
		float *y, int incy);
void call_zgbmv(char trans, int m, int n, int kl, int ku, double *alpha,
		double *a, int lda, double *x, int incx, double *beta,
		double *y, int incy);

/* LAPACK drivers */
void call_sgbtrf(int m, int n, int kl, int ku, float *a, int lda,
		 int *ipiv, int *info);
void call_dgbtrf(int m, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, int *info);
void call_cgbtrf(int m, int n, int kl, int ku, float *a, int lda,
		 int *ipiv, int *info);
void call_zgbtrf(int m, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, int *info);
void call_sgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
		 int lda, int *ipiv, float *b, int ldb, int *info);
void call_dgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
		 int lda, int *ipiv, double *b, int ldb, int *info);
void call_cgbtrs(char trans, int n, int kl, int ku, int nrhs, float *a,
		 int lda, int *ipiv, float *b, int ldb, int *info);
void call_zgbtrs(char trans, int n, int kl, int ku, int nrhs, double *a,
		 int lda, int *ipiv, double *b, int ldb, int *info);
void call_sgbcon(char norm, int n, int kl, int ku, float *a, int lda,
		 int *ipiv, float anorm, float *rcond, float *work,
		 int *iwork, int *info);
void call_dgbcon(char norm, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, double anorm, double *rcond, double *work,
		 int *iwork, int *info);
void call_cgbcon(char norm, int n, int kl, int ku, float *a, int lda,
		 int *ipiv, float anorm, float *rcond, float *work,
		 float *rwork, int *info);
void call_zgbcon(char norm, int n, int kl, int ku, double *a, int lda,
		 int *ipiv, double anorm, double *rcond, double *work,
		 double *rwork, int *info);

#ifdef TIME_BANDED_MV
extern double _banded_mv_time; /* server.c */
#endif

/* out: 1 */
int
silc_band_general_band_matvec(int num_params, silc_mfunc_param_t *param)
{
  silc_object_t *x, *y;
  silc_modifier_t *x_mods, *y_mods;
  silc_band_t *in;
  int lda;
  double t, elapsed;

  x = param[0].o; x_mods = param[0].m;
  y = param[1].o; y_mods = param[1].m;
#ifdef DEBUG
  printf("silc_band_general_band_matvec(0x%lx, 0x%lx)\n", x, y);
#endif
#ifdef TIME_BANDED_MV
  t = silc_wtime();
#endif
  in = SILC_BAND(x);
  lda = in->l + in->u + 1; /* leading dimension */
  switch (x->precision) {
  case SILC_FLOAT:
    {
      float *value;
      value = (float *)malloc(sizeof(float) * y->d.vector.length);
      call_sgbmv('N', x->d.matrix.m, x->d.matrix.n, in->l, in->u, 1.0,
		 in->v.s, lda, y->d.vector.v.s, 1, 0.0, value, 1);
      free(y->d.vector.v.s);
      y->d.vector.v.s = value;
    }
    break;
  case SILC_DOUBLE:
    {
      double *value;
      value = (double *)malloc(sizeof(double) * y->d.vector.length);
      call_dgbmv('N', x->d.matrix.m, x->d.matrix.n, in->l, in->u, 1.0,
		 in->v.d, lda, y->d.vector.v.d, 1, 0.0, value, 1);
      free(y->d.vector.v.d);
      y->d.vector.v.d = value;
    }
    break;
  case SILC_COMPLEX:
    {
      float *value, alpha[2] = {1.0, 0.0}, beta[2] = {0.0, 0.0};
      value = (float *)malloc(sizeof(float) * 2 * y->d.vector.length);
      call_cgbmv('N', x->d.matrix.m, x->d.matrix.n, in->l, in->u, alpha,
		 in->v.c, lda, y->d.vector.v.c, 1, beta, value, 1);
      free(y->d.vector.v.c);
      y->d.vector.v.c = value;
    }
    break;
  case SILC_DOUBLE_COMPLEX:
    {
      double *value, alpha[2] = {1.0, 0.0}, beta[2] = {0.0, 0.0};
      value = (double *)malloc(sizeof(double) * 2 * y->d.vector.length);
      call_zgbmv('N', x->d.matrix.m, x->d.matrix.n, in->l, in->u, alpha,
		 in->v.z, lda, y->d.vector.v.z, 1, beta, value, 1);
      free(y->d.vector.v.z);
      y->d.vector.v.z = value;
    }
    break;
  default:
    should_not_reach_here;
  }
#ifdef TIME_BANDED_MV
  elapsed = silc_wtime() - t;
  printf("sparse_band: general_band_matvec: %8lf sec.\n", elapsed);
  _banded_mv_time += elapsed;
#endif
  return 0;
}

static int
matrix_scalar_product(silc_mfunc_param_t *matrix,
		      silc_mfunc_param_t *scalar)
{
  silc_object_t *x, *y;
  silc_modifier_t *x_mods, *y_mods;
  silc_band_t *in;
  int i, j, k, m, n, kl, ku, ld, end;

  x = matrix->o; x_mods = matrix->m;
  y = scalar->o; y_mods = scalar->m;
#ifdef DEBUG
  printf("sparse_band: matrix_scalar_product(0x%lx, 0x%lx)\n", x, y);
#endif
  m = x->d.matrix.m;
  n = x->d.matrix.n;
  in = SILC_BAND(x);
  kl = in->l;
  ku = in->u;
  ld = kl + ku + 1; /* leading dimension */
  switch (x->precision) {
  case SILC_INT:
    for (j = 0; j < n; j++) {
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++)
	in->v.i[kl+ku+i-j+j*ld] *= y->d.scalar.i;
    }
    break;
  case SILC_LONG:
    for (j = 0; j < n; j++) {
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++)
	in->v.l[kl+ku+i-j+j*ld] *= y->d.scalar.l;
    }
    break;
  case SILC_FLOAT:
    for (j = 0; j < n; j++) {
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++)
	in->v.s[kl+ku+i-j+j*ld] *= y->d.scalar.s;
    }
    break;
  case SILC_DOUBLE:
    for (j = 0; j < n; j++) {
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++)
	in->v.d[kl+ku+i-j+j*ld] *= y->d.scalar.d;
    }
    break;
  case SILC_COMPLEX:
    for (j = 0; j < n; j++) {
      float re, im;
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++) {
	k = (kl+ku+i-j+j*ld) * 2;
	re = (in->v.c[k] * y->d.scalar.c[0] - in->v.c[k+1] * y->d.scalar.c[1]);
	im = (in->v.c[k] * y->d.scalar.c[1] + in->v.c[k+1] * y->d.scalar.c[0]);
	in->v.c[k]   = re;
	in->v.c[k+1] = im;
      }
    }
    break;
  case SILC_DOUBLE_COMPLEX:
    for (j = 0; j < n; j++) {
      double re, im;
      end = _min(m, j + kl + 1);
      for (i = _max(0, j - ku); i < end; i++) {
	k = (kl+ku+i-j+j*ld) * 2;
	re = (in->v.z[k] * y->d.scalar.z[0] - in->v.z[k+1] * y->d.scalar.z[1]);
	im = (in->v.z[k] * y->d.scalar.z[1] + in->v.z[k+1] * y->d.scalar.z[0]);
	in->v.z[k]   = re;
	in->v.z[k+1] = im;
      }
    }
    break;
  default:
    should_not_reach_here;
  }
  return 0;
}

/* out: 0 */
int
silc_band_mul_matrix_scalar(int n_params, silc_mfunc_param_t *param)
{
  return matrix_scalar_product(&param[0], &param[1]);
}

/* out: 1 */
int
silc_band_mul_scalar_matrix(int n_params, silc_mfunc_param_t *param)
{
  return matrix_scalar_product(&param[1], &param[0]);
}

#ifdef TIME_BANDED_LU
extern double _banded_lu_time; /* server.c */
#endif

/* out: 1 */
int
silc_band_general_band_lu_solve(int num_params, silc_mfunc_param_t *param)
{
  silc_object_t *x, *y;
  silc_modifier_t *x_mods, *y_mods;
  silc_band_t *in;
  int *ipiv, info, if_vector;
  size_t lda, ldx, nrhs, size, i;
  double t, elapsed;

  x = param[0].o; x_mods = param[0].m;
  y = param[1].o; y_mods = param[1].m;
#ifdef DEBUG
  printf("silc_band_general_band_lu_solve(0x%lx, 0x%lx)\n", x, y);
#endif
  if (x->type != SILC_MATRIX_TYPE ||
      x->format != silc_find_format_id(SILC_FORMAT_BAND))
    return -1;
  switch (y->type) {
  case SILC_COLUMN_VECTOR_TYPE:
    if (x->d.matrix.n != y->d.vector.length) {
      fprintf(stderr, "sparse_band: matrix-vector size mismatch\n");
      return -1;
    }
    if_vector = 1;
    nrhs = 1;
    break;
  case SILC_MATRIX_TYPE:
    if (y->format != silc_find_format_id(SILC_FORMAT_DENSE))
      return -1;
    if (x->d.matrix.n != y->d.matrix.m) {
      fprintf(stderr, "sparse_band: matrix-matrix size mismatch\n");
      return -1;
    }
    if_vector = 0;
    nrhs = y->d.matrix.n;
    break;
  default:
    should_not_reach_here;
    return -1;
  }
#ifdef TIME_BANDED_LU
  t = silc_wtime();
#endif
  in = SILC_BAND(x);
  lda = in->l * 2 + in->u + 1; /* leading dimension of ab */
  ldx = in->l + in->u + 1;
  size = _max(1, _min(x->d.matrix.m, x->d.matrix.n));
  ipiv = (int *)malloc(sizeof(int) * size);
  switch (x->precision) {
  case SILC_FLOAT:
    {
      float *ab, *p, *q;
      ab = (float *)malloc(sizeof(float) * lda * x->d.matrix.n);
      size = sizeof(float) * ldx;
      p = ab + in->l;
      q = in->v.s;
      for (i = 0; i < x->d.matrix.n; i++) {
	memcpy(p, q, size);
	p += lda;
	q += ldx;
      }
      call_sgbtrf(x->d.matrix.m, x->d.matrix.n, in->l, in->u, ab, lda, ipiv,
		  &info);
      if (info) {
	fprintf(stderr, "sparse_band: sgbtrf() failed (info = %d)\n", info);
	free(ab);
	goto onError;
      }
      call_sgbtrs('N', x->d.matrix.n, in->l, in->u, nrhs, ab, lda, ipiv,
		  if_vector ? y->d.vector.v.s : SILC_DENSE(y)->v.s,
		  if_vector ? y->d.vector.length : y->d.matrix.m, &info);
      free(ab);
      if (info) {
	fprintf(stderr, "sparse_band: sgbtrs() failed (info = %d)\n", info);
	goto onError;
      }
    }
    break;
  case SILC_DOUBLE:
    {
      double *ab, *p, *q;
      ab = (double *)malloc(sizeof(double) * lda * x->d.matrix.n);
      size = sizeof(double) * ldx;
      p = ab + in->l;
      q = in->v.d;
      for (i = 0; i < x->d.matrix.n; i++) {
	memcpy(p, q, size);
	p += lda;
	q += ldx;
      }
      call_dgbtrf(x->d.matrix.m, x->d.matrix.n, in->l, in->u, ab, lda, ipiv,
		  &info);
      if (info) {
	fprintf(stderr, "sparse_band: dgbtrf() failed (info = %d)\n", info);
	free(ab);
	goto onError;
      }
      call_dgbtrs('N', x->d.matrix.n, in->l, in->u, nrhs, ab, lda, ipiv,
		  if_vector ? y->d.vector.v.d : SILC_DENSE(y)->v.d,
		  if_vector ? y->d.vector.length : y->d.matrix.m, &info);
      free(ab);
      if (info) {
	fprintf(stderr, "sparse_band: dgbtrs() failed (info = %d)\n", info);
	goto onError;
      }
    }
    break;
  case SILC_COMPLEX:
    {
      float *ab, *p, *q;
      ab = (float *)malloc(sizeof(float) * 2 * lda * x->d.matrix.n);
      size = sizeof(float) * 2 * ldx;
      p = ab + in->l * 2;
      q = in->v.c;
      for (i = 0; i < x->d.matrix.n; i++) {
	memcpy(p, q, size);
	p += 2 * lda;
	q += 2 * ldx;
      }
      call_cgbtrf(x->d.matrix.m, x->d.matrix.n, in->l, in->u, ab, lda, ipiv,
		  &info);
      if (info) {
	fprintf(stderr, "sparse_band: cgbtrf() failed (info = %d)\n", info);
	free(ab);
	goto onError;
      }
      call_cgbtrs('N', x->d.matrix.n, in->l, in->u, nrhs, ab, lda, ipiv,
		  if_vector ? y->d.vector.v.c : SILC_DENSE(y)->v.c,
		  if_vector ? y->d.vector.length : y->d.matrix.m, &info);
      free(ab);
      if (info) {
	fprintf(stderr, "sparse_band: cgbtrs() failed (info = %d)\n", info);
	goto onError;
      }
    }
    break;
  case SILC_DOUBLE_COMPLEX:
    {
      double *ab, *p, *q;
      ab = (double *)malloc(sizeof(double) * 2 * lda * x->d.matrix.n);
      size = sizeof(double) * 2 * ldx;
      p = ab + in->l * 2;
      q = in->v.z;
      for (i = 0; i < x->d.matrix.n; i++) {
	memcpy(p, q, size);
	p += 2 * lda;
	q += 2 * ldx;
      }
      call_zgbtrf(x->d.matrix.m, x->d.matrix.n, in->l, in->u, ab, lda, ipiv,
		  &info);
      if (info) {
	fprintf(stderr, "sparse_band: zgbtrf() failed (info = %d)\n", info);
	free(ab);
	goto onError;
      }
      call_zgbtrs('N', x->d.matrix.n, in->l, in->u, nrhs, ab, lda, ipiv,
		  if_vector ? y->d.vector.v.z : SILC_DENSE(y)->v.z,
		  if_vector ? y->d.vector.length : y->d.matrix.m, &info);
      free(ab);
      if (info) {
	fprintf(stderr, "sparse_band: zgbtrs() failed (info = %d)\n", info);
	goto onError;
      }
    }
    break;
  default:
    should_not_reach_here;
  }
  free(ipiv);
#ifdef TIME_BANDED_LU
  elapsed = silc_wtime() - t;
  printf("sparse_band: general_band_lu_solve: %8lf sec.\n", elapsed);
  _banded_lu_time += elapsed;
#endif
  return 0;

 onError:
  free(ipiv);
  return -1;
}

int
silc_band_func_band_to_crs(int n_params, silc_mfunc_param_t *param)
{
  silc_object_t *object, *new_object;
  silc_band_t *in;
  silc_crs_t *new_in;
  size_t nnz, ld;
  ssize_t i;
  double t, elapsed;

  object = param[0].o;
#ifdef DEBUG
  printf("silc_band_func_band_to_crs(0x%lx)\n", object);
#endif
  if (object->type != SILC_MATRIX_TYPE)
    return -1;
  if (object->format != silc_find_format_id(SILC_FORMAT_BAND)) {
    fprintf(stderr, "sparse_band: not in %s format\n", SILC_FORMAT_BAND);
    return -1;
  }
#ifdef TIME_BAND_TO_CRS
  t = silc_wtime();
#endif
  in = SILC_BAND(object);
  ld = in->l + in->u + 1; /* leading dimension */
  new_in = (silc_crs_t *)malloc(sizeof(silc_crs_t));
  new_in->row = (int *)malloc(sizeof(int) * (object->d.matrix.m + 1));
  nnz = 0; /* number of non-zero elements */

#define _count_nonzeros(test) { \
    size_t column, c, j, k;						\
    for (i = 0; i < in->l; i++) {					\
      new_in->row[i] = nnz;						\
      c = _min(in->u + 1 + i, object->d.matrix.n);			\
      k = in->u + i;							\
      for (j = 0; j < c; j++) {						\
	if (test)							\
	  nnz++;							\
	k += ld - 1;							\
      }									\
    }									\
    for (i = in->l; i < object->d.matrix.m; i++) {			\
      new_in->row[i] = nnz;						\
      column = i - in->l;						\
      c = _min(in->u + 1 + in->l, object->d.matrix.n - column);		\
      k = ld * (column + 1) - 1;					\
      for (j = 0; j < c; j++) {						\
	if (test)							\
	  nnz++;							\
	k += ld - 1;							\
      }									\
    }									\
  }
  switch (object->precision) {
  case SILC_INT:
    _count_nonzeros(in->v.i[k]); break;
  case SILC_LONG:
    _count_nonzeros(in->v.l[k]); break;
  case SILC_FLOAT:
    _count_nonzeros(*(int32_t *)&in->v.s[k]); break;
  case SILC_DOUBLE:
    _count_nonzeros(*(int64_t *)&in->v.d[k]); break;
  case SILC_COMPLEX:
    _count_nonzeros(*(int32_t *)&in->v.c[k*2] ||
		    *(int32_t *)&in->v.c[k*2+1]);
    break;
  case SILC_DOUBLE_COMPLEX:
    _count_nonzeros(*(int64_t *)&in->v.z[k*2] ||
		    *(int64_t *)&in->v.z[k*2+1]);
    break;
  default:
    should_not_reach_here;
  }

  new_in->row[i] = new_in->nnz = nnz;
  new_in->index = (int *)malloc(sizeof(int) * nnz);

#define _conv_to_crs_1(pre, type, test) { \
    size_t column, c, j, k, p;					\
    if (i < in->l) {						\
      column = 0; k = in->u + i;				\
      c = _min(in->u + 1 + i, object->d.matrix.n);		\
    } else {							\
      column = i - in->l; k = ld * (column + 1) - 1;		\
      c = _min(in->u + 1 + in->l, object->d.matrix.n - column); \
    }								\
    p = new_in->row[i];						\
    for (j = 0; j < c; j++) {					\
      if (test) { 						\
        new_in->v.pre[p] = in->v.pre[k];			\
        new_in->index[p] = column;				\
	p++;							\
      }								\
      k += ld - 1; column++;					\
    }								\
  }
#define _conv_to_crs_2(pre, type, test) { \
    size_t column, c, j, k, p;					\
    if (i < in->l) {						\
      column = 0; k = in->u + i;				\
      c = _min(in->u + 1 + i, object->d.matrix.n);		\
    } else {							\
      column = i - in->l; k = ld * (column + 1) - 1;		\
      c = _min(in->u + 1 + in->l, object->d.matrix.n - column); \
    }								\
    p = new_in->row[i];						\
    for (j = 0; j < c; j++) {					\
      if (test) { 						\
        new_in->v.pre[p*2]   = in->v.pre[k*2];			\
        new_in->v.pre[p*2+1] = in->v.pre[k*2+1];		\
        new_in->index[p] = column;				\
	p++;							\
      }								\
      k += ld - 1; column++;					\
    }								\
  }
  switch (object->precision) {
  case SILC_INT:
    new_in->v.i = (int32_t *)malloc(sizeof(int32_t) * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_1(i, int32_t, in->v.i[k]);
    break;
  case SILC_LONG:
    new_in->v.l = (int64_t *)malloc(sizeof(int64_t) * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_1(l, int64_t, in->v.l[k]);
    break;
  case SILC_FLOAT:
    new_in->v.s = (float *)malloc(sizeof(float) * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_1(s, int32_t, *(int32_t *)&in->v.s[k]);
    break;
  case SILC_DOUBLE:
    new_in->v.d = (double *)malloc(sizeof(double) * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_1(d, int64_t, *(int64_t *)&in->v.d[k]);
    break;
  case SILC_COMPLEX:
    new_in->v.c = (float *)malloc(sizeof(float) * 2 * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_2(c, int32_t, (*(int32_t *)&in->v.c[k*2] ||
				  *(int32_t *)&in->v.c[k*2+1]));
    break;
  case SILC_DOUBLE_COMPLEX:
    new_in->v.z = (double *)malloc(sizeof(double) * 2 * nnz);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_crs_2(z, int64_t, (*(int64_t *)&in->v.z[k*2] ||
				  *(int64_t *)&in->v.z[k*2+1]));
    break;
  default:
    should_not_reach_here;
  }
  new_object = silc_object_new_matrix(object->precision,
				      silc_find_format_id(SILC_FORMAT_CRS),
				      object->d.matrix.m,
				      object->d.matrix.n);
  if (!new_object)
    return -1;
  new_object->d.matrix.in = new_in;
#ifdef TIME_BAND_TO_CRS
  elapsed = silc_wtime() - t;
  printf("sparse_band: func_band_to_crs: %8lf sec.\n", elapsed);
#endif
  param[1].o = new_object; /* return value */
  return 0;
}

int
silc_band_func_crs_to_band(int n_params, silc_mfunc_param_t *param)
{
  silc_object_t *object, *new_object;
  silc_crs_t *in;
  silc_band_t *new_in;
  size_t lda;
  ssize_t i;
  double t, elapsed;

  object = param[0].o;
#ifdef DEBUG
  printf("silc_band_func_crs_to_band(0x%lx)\n", object);
#endif
  if (object->type != SILC_MATRIX_TYPE)
    return -1;
  if (object->format != silc_find_format_id(SILC_FORMAT_CRS)) {
    fprintf(stderr, "sparse_band: not in %s format\n", SILC_FORMAT_CRS);
    return -1;
  }
#ifdef TIME_CRS_TO_BAND
  t = silc_wtime();
#endif
  in = SILC_CRS(object);
  new_in = (silc_band_t *)malloc(sizeof(silc_band_t));
  new_in->l = new_in->u = 0;
  for (i = 0; i < object->d.matrix.m; i++) {
    size_t j, k;
    for (k = in->row[i]; k < in->row[i+1]; k++) {
      j = in->index[k];
      if (i > j) {
	if (new_in->l < i - j) new_in->l = i - j;
      } else {
	if (new_in->u < j - i) new_in->u = j - i;
      }
    }
  }
  lda = new_in->l + new_in->u + 1;

#define _conv_to_band_1(pre) { \
    size_t j, k;						 \
    for (k = in->row[i]; k < in->row[i+1]; k++) {		 \
      j = in->index[k];						 \
      new_in->v.pre[lda * j + new_in->u + i - j] = in->v.pre[k]; \
    }								 \
  }
#define _conv_to_band_2(pre) { \
    size_t j, k, l;						 \
    for (k = in->row[i]; k < in->row[i+1]; k++) {		 \
      j = in->index[k];						 \
      l = lda * j + new_in->u + i - j;				 \
      new_in->v.pre[l*2]   = in->v.pre[k*2];			 \
      new_in->v.pre[l*2+1] = in->v.pre[k*2+1];			 \
    }								 \
  }

  switch (object->precision) {
  case SILC_INT:
    new_in->v.i = (int32_t *)calloc(lda * object->d.matrix.n, sizeof(int32_t));
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_1(i);
    break;
  case SILC_LONG:
    new_in->v.l = (int64_t *)calloc(lda * object->d.matrix.n, sizeof(int64_t));
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_1(l);
    break;
  case SILC_FLOAT:
    new_in->v.s = (float *)calloc(lda * object->d.matrix.n, sizeof(float));
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_1(s);
    break;
  case SILC_DOUBLE:
    new_in->v.d = (double *)calloc(lda * object->d.matrix.n, sizeof(double));
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_1(d);
    break;
  case SILC_COMPLEX:
    new_in->v.c = (float *)calloc(lda * object->d.matrix.n, sizeof(float) * 2);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_2(c);
    break;
  case SILC_DOUBLE_COMPLEX:
    new_in->v.z = (double *)calloc(lda * object->d.matrix.n, sizeof(double) * 2);
#pragma omp parallel for
    for (i = 0; i < (ssize_t)object->d.matrix.m; i++)
      _conv_to_band_2(z);
    break;
  default:
    should_not_reach_here;
  }
  new_object = silc_object_new_matrix(object->precision,
				      silc_find_format_id(SILC_FORMAT_BAND),
				      object->d.matrix.m,
				      object->d.matrix.n);
  if (!new_object)
    return -1;
  new_object->d.matrix.in = new_in;
#ifdef TIME_CRS_TO_BAND
  elapsed = silc_wtime() - t;
  printf("sparse_band: func_crs_to_band: %8lf sec.\n", elapsed);
#endif
  param[1].o = new_object; /* return value */
  return 0;
}

int
silc_band_func_rcond(int n_params, silc_mfunc_param_t *param)
{
  silc_object_t *object, *new_object;
  silc_band_t *in;
  int i, j, k, m, n, kl, ku, ld, lda, end, *ipiv, info;
  size_t size;
  const char *format_name;
  union {
    float *s; double *d;
    float *c; double *z;
  } v;
  union {
    float s;
    double d;
  } norm, rcond;

  object = param[0].o;
#ifdef DEBUG
  printf("silc_band_func_rcond(0x%lx)\n", object);
#endif
  if (object->type != SILC_MATRIX_TYPE)
    return -1;
  format_name = silc_find_format_name(object->format);
  if (strcmp(format_name, SILC_FORMAT_BAND)) {
    /* XXX need a workaround */
    fprintf(stderr, "sparse_band: not in %s format\n", SILC_FORMAT_BAND);
    return -1;
  }
  if (object->d.matrix.m != object->d.matrix.n) {
    fprintf(stderr, "sparse_band: rcond: not a square matrix\n");
    return -1;
  }
  if (object->precision == SILC_INT ||
      object->precision == SILC_LONG) {
    fprintf(stderr, "sparse_band: rcond: integer matrix is not supported\n");
    return -1;
  }

  m = object->d.matrix.m;
  n = object->d.matrix.n;
  in = SILC_BAND(object);
  kl = in->l;
  ku = in->u;
  ld = kl + ku + 1;
  lda = kl * 2 + ku + 1; /* leading dimension (augmented) */
  switch (object->precision) {
  case SILC_FLOAT:
    {
      float *p, *q, sum;
      v.s = (float *)malloc(sizeof(float) * lda * n);
      size = sizeof(float) * ld;
      p = v.s + kl;
      q = in->v.s;
      norm.s = 0.0;
      for (j = 0; j < n; j++) {
	memcpy(p, q, size);
	p += lda;
	q += ld;
	sum = 0.0;
	end = _min(m, j + kl + 1);
	for (i = _max(0, j - ku); i < end; i++)
	  sum += fabs(in->v.s[ku+i-j+j*ld]);
	if (norm.s < sum)
	  norm.s = sum;
      }
    }
    break;
  case SILC_DOUBLE:
    {
      double *p, *q, sum;
      v.d = (double *)malloc(sizeof(double) * lda * n);
      size = sizeof(double) * ld;
      p = v.d + kl;
      q = in->v.d;
      norm.d = 0.0;
      for (j = 0; j < n; j++) {
	memcpy(p, q, size);
	p += lda;
	q += ld;
	sum = 0.0;
	end = _min(m, j + kl + 1);
	for (i = _max(0, j - ku); i < end; i++)
	  sum += fabs(in->v.d[ku+i-j+j*ld]);
	if (norm.d < sum)
	  norm.d = sum;
      }
    }
    break;
  case SILC_COMPLEX:
    {
      float *p, *q, re, im, sum;
      v.c = (float *)malloc(sizeof(float) * lda * n * 2);
      size = sizeof(float) * ld * 2;
      p = v.c + kl * 2;
      q = in->v.c;
      norm.s = 0.0;
      for (j = 0; j < n; j++) {
	memcpy(p, q, size);
	p += lda * 2;
	q += ld * 2;
	sum = 0.0;
	end = _min(m, j + kl + 1);
	for (i = _max(0, j - ku); i < end; i++) {
	  k = (ku+i-j+j*ld) * 2;
	  re = in->v.c[k];
	  im = in->v.c[k + 1];
	  sum += sqrt(re * re + im * im);
	}
	if (norm.s < sum)
	  norm.s = sum;
      }
    }
    break;
  case SILC_DOUBLE_COMPLEX:
    {
      double *p, *q, re, im, sum;
      v.z = (double *)malloc(sizeof(double) * lda * n * 2);
      size = sizeof(double) * ld * 2;
      p = v.z + kl * 2;
      q = in->v.z;
      norm.d = 0.0;
      for (j = 0; j < n; j++) {
	memcpy(p, q, size);
	p += lda * 2;
	q += ld * 2;
	sum = 0.0;
	end = _min(m, j + kl + 1);
	for (i = _max(0, j - ku); i < end; i++) {
	  k = (ku+i-j+j*ld) * 2;
	  re = in->v.z[k];
	  im = in->v.z[k + 1];
	  sum += sqrt(re * re + im * im);
	}
	if (norm.d < sum)
	  norm.d = sum;
      }
    }
    break;
  default:
    should_not_reach_here;
  }
#ifdef DEBUG
  switch (object->precision) {
  case SILC_FLOAT:
  case SILC_COMPLEX:
    printf("sparse_band: rcond: anorm = %e\n", norm.s); break;
  case SILC_DOUBLE:
  case SILC_DOUBLE_COMPLEX:
    printf("sparse_band: rcond: anorm = %le\n", norm.d); break;
  default:
    should_not_reach_here;
  }
#endif
  ipiv = (int *)malloc(sizeof(int) * n);
  if (!ipiv) {
    fprintf(stderr, "sparse_band: rcond: out of memory (ipiv)");
    free(v.s);
    return -1;
  }
  /* form an LU factorization of an M-by-N band matrix */
  switch (object->precision) {
  case SILC_FLOAT: 
    call_sgbtrf(m, n, kl, ku, v.s, lda, ipiv, &info); break;
  case SILC_DOUBLE:
    call_dgbtrf(m, n, kl, ku, v.d, lda, ipiv, &info); break;
  case SILC_COMPLEX:
    call_cgbtrf(m, n, kl, ku, v.c, lda, ipiv, &info); break;
  case SILC_DOUBLE_COMPLEX:
    call_zgbtrf(m, n, kl, ku, v.z, lda, ipiv, &info); break;
  default:
    should_not_reach_here;
  }
  if (info == 0) {
    /* estimate the reciprocal of the condition of the matrix in 1-norm */
    switch (object->precision) {
    case SILC_FLOAT:
      {
	float *work;
	int *iwork;
	work = (float *)malloc(sizeof(float) * 3 * n);
	iwork = (int *)malloc(sizeof(int) * n);
	call_sgbcon('1', n, kl, ku, v.s, lda, ipiv, norm.s, &rcond.s,
		    work, iwork, &info);
	free(work);
	free(iwork);
      }
      new_object = silc_object_new_float(rcond.s);
      break;
    case SILC_DOUBLE:
      {
	double *work;
	int *iwork;
	work = (double *)malloc(sizeof(double) * 3 * n);
	iwork = (int *)malloc(sizeof(int) * n);
	call_dgbcon('1', n, kl, ku, v.d, lda, ipiv, norm.d, &rcond.d,
		    work, iwork, &info);
	free(work);
	free(iwork);
      }
      new_object = silc_object_new_double(rcond.d);
      break;
    case SILC_COMPLEX:
      {
	float *work, *rwork;
	work = (float *)malloc(sizeof(float) * 4 * n); 
	rwork = (float *)malloc(sizeof(float) * 2 * n);
	call_cgbcon('1', n, kl, ku, v.c, lda, ipiv, norm.s, &rcond.s,
		    work, rwork, &info);
	free(work);
	free(rwork);
      }
      new_object = silc_object_new_float(rcond.s);
      break;
    case SILC_DOUBLE_COMPLEX:
      {
	double *work, *rwork;
	work = (double *)malloc(sizeof(double) * 4 * n);
	rwork = (double *)malloc(sizeof(double) * 2 * n);
	call_zgbcon('1', n, kl, ku, v.z, lda, ipiv, norm.d, &rcond.d,
		    work, rwork, &info);
	free(work);
	free(rwork);
      }
      new_object = silc_object_new_double(rcond.d);
      break;
    default:
      should_not_reach_here;
    }
  }
  free(v.s);
  free(ipiv);
  if (info != 0)
    return -1;
  param[1].o = new_object;
  return 0;
}

#define SILC_ANY_INTEGER   (SILC_INT|SILC_LONG)
#define SILC_ANY_REAL      (SILC_FLOAT|SILC_DOUBLE)
#define SILC_ANY_COMPLEX   (SILC_COMPLEX|SILC_DOUBLE_COMPLEX)
#define SILC_ANY_PRECISION (SILC_ANY_INTEGER|SILC_ANY_REAL|SILC_ANY_COMPLEX)

static binary_op_table_t _binary_op_table[] = {
  /* name, op, precision, x_type, x_format, y_type, y_format, out, flags */
  {silc_band_general_band_matvec, "silc_band_general_band_matvec",
   SILC_MULTIPLY_OP, SILC_ANY_REAL|SILC_ANY_COMPLEX,
   SILC_MATRIX_TYPE, SILC_FORMAT_BAND, SILC_COLUMN_VECTOR_TYPE, 0, 1, 0},
  {silc_band_mul_matrix_scalar, "silc_band_mul_matrix_scalar",
   SILC_MULTIPLY_OP, SILC_ANY_PRECISION,
   SILC_MATRIX_TYPE, SILC_FORMAT_BAND, SILC_SCALAR_TYPE, 0, 0, 0},
  {silc_band_mul_scalar_matrix, "silc_band_mul_scalar_matrix",
   SILC_MULTIPLY_OP, SILC_ANY_PRECISION,
   SILC_SCALAR_TYPE, 0, SILC_MATRIX_TYPE, SILC_FORMAT_BAND, 1, 0},
  {silc_band_general_band_lu_solve, "silc_band_general_band_lu_solve",
   SILC_LEQ_OP, SILC_ANY_REAL|SILC_ANY_COMPLEX,
   SILC_MATRIX_TYPE, SILC_FORMAT_BAND, SILC_COLUMN_VECTOR_TYPE, 0, 1, 0},
  {silc_band_general_band_lu_solve, "silc_band_general_band_lu_solve",
   SILC_LEQ_OP, SILC_ANY_REAL|SILC_ANY_COMPLEX,
   SILC_MATRIX_TYPE, SILC_FORMAT_BAND,
   SILC_MATRIX_TYPE, SILC_FORMAT_DENSE, 1, 0},
  {NULL} /* sentinel */
};

static function_table_t _function_table[] = {
  /* name, sig_text, flags */
  {silc_band_func_band_to_crs, "silc_band_func_band_to_crs",
   "matrix CRS(matrix)", 0},
  {silc_band_func_crs_to_band, "silc_band_func_crs_to_band",
   "matrix band(matrix)", 0},
  {silc_band_func_rcond, "silc_band_func_rcond",
   "scalar rcond(matrix)", 0},
  {NULL} /* sentinel */
};

void
init_sparse_band_op(char *module_name)
{
  silc_register_module(module_name,
		       NULL,  /* _unary_op_table */
		       _binary_op_table,
		       _function_table,
		       NULL); /* _procedure_table */
}
